@@ -44,6 +44,7 @@
 #include "xap_DialogFactory.h"
 #include "xap_Dlg_FileOpenSaveAs.h"
 
+#include <dialogs/xp/ap_Dialog_CollaborationShare.h>
 #include <dialogs/xp/ap_Dialog_CollaborationJoin.h>
 #include <dialogs/xp/ap_Dialog_CollaborationAccounts.h>
 
@@ -120,9 +121,12 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
 	// store the settings to the profile
 	AbiCollabSessionManager::getManager()->storeProfile();
 
-	// register all available account handlers
+	// unregister all available account handlers
 	AbiCollabSessionManager::getManager()->unregisterAccountHandlers();
 
+	// unregister all import/export sniffers we have registered
+	AbiCollabSessionManager::getManager()->unregisterSniffers();
+	
 	// unregister all dialogs we use
 	AbiCollabSessionManager::getManager()->unregisterDialogs();
 
@@ -150,8 +154,8 @@ static const char * szCollaborationTip = "Collaborate over the internet or local
 static const char * szCollaborationOffer = "Share Document";
 static const char * szCollaborationOfferTip = "Offer the current document for collaboration";
 
-static const char * szCollaborationJoin = "Shared Documents";
-static const char * szCollaborationJoinTip = "Join a collaboration session";
+static const char * szCollaborationJoin = "Open Shared Document";
+static const char * szCollaborationJoinTip = "Open a shared document";
 
 static const char * szCollaborationAccounts = "Accounts";
 static const char * szCollaborationAccountsTip = "Manage collaboration accounts";
@@ -185,14 +189,22 @@ static bool s_abicollab_command_invoke(AV_View* v, EV_EditMethodCallData *d);
 /*!
  * returns true if at least one account is online
  */
-bool any_accounts_online( const std::vector<AccountHandler *>& vecAccounts )
+static bool s_any_accounts_online(bool bIncludeNonManualShareAccounts = true)
 {
+	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
+	UT_return_val_if_fail(pManager, false);
+	
+	const std::vector<AccountHandler *>& vecAccounts = pManager->getAccounts();
+	
 	for (UT_uint32 i = 0; i < vecAccounts.size(); i++)
 	{
 		AccountHandler* pHandler = vecAccounts[i];
 		if (pHandler && pHandler->isOnline())
 		{
-			return true;
+			if (bIncludeNonManualShareAccounts)
+				return true;
+			if (pHandler->canManuallyStartSession())
+				return true;
 		}
 	}
 	return false;
@@ -205,7 +217,10 @@ Defun_EV_GetMenuItemState_Fn(collab_GetState_ShowAuthors)
 	UT_UNUSED(id);
 
 	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
-	if (!any_accounts_online( pManager->getAccounts() )) return EV_MIS_Gray;
+	UT_return_val_if_fail(pManager, EV_MIS_Gray);
+	
+	if (!s_any_accounts_online())
+		return EV_MIS_Gray;
 
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
@@ -222,31 +237,6 @@ Defun_EV_GetMenuItemState_Fn(collab_GetState_ShowAuthors)
 	return EV_MIS_ZERO;
 }
 
-/*!
- * returns checked true if current document is part of a collaboration
- */
-Defun_EV_GetMenuItemState_Fn(collab_GetState_Joined)
-{
-	UT_UNUSED(id);
-
-	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
-	if (!any_accounts_online( pManager->getAccounts() )) return EV_MIS_Gray;
-	
-	ABIWORD_VIEW;
-	UT_return_val_if_fail (pView, EV_MIS_Gray);
-	PD_Document* pDoc = pView->getDocument();
-	UT_return_val_if_fail (pDoc, EV_MIS_Gray);
-	
-	if (pManager->isInSession(pDoc))
-	{
-		if (pManager->isLocallyControlled(pDoc))
-			return EV_MIS_Toggled;
-		else
-			return EV_MIS_Gray;
-	}
-	return EV_MIS_ZERO;
-}
-
 /*!
  * returns checked true if currently recording
  */
@@ -257,7 +247,10 @@ Defun_EV_GetMenuItemState_Fn(collab_GetState_Recording)
 	// only do this in debug mode
 #if !defined(ABICOLLAB_RECORD_ALWAYS) && defined(DEBUG)
 	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
-	if (!any_accounts_online( pManager->getAccounts() )) return EV_MIS_Gray;
+	UT_return_val_if_fail(pManager, EV_MIS_Gray);
+	
+	if (!s_any_accounts_online())
+		return EV_MIS_Gray;
 	
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
@@ -290,19 +283,43 @@ Defun_EV_GetMenuItemState_Fn(collab_GetState_AnyActive)
 	UT_UNUSED(pAV_View);
 	UT_UNUSED(id);
 
-	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
- 	const std::vector<AccountHandler*>& vecAccounts = pManager->getAccounts();
-
-	for (UT_uint32 i = 0; i < vecAccounts.size(); i++)
-	{
-		AccountHandler* pHandler = vecAccounts[i];
-		if (pHandler && pHandler->isOnline())
-			return EV_MIS_ZERO;
-	}
-
-	return EV_MIS_Gray;
+	if (!s_any_accounts_online())
+		return EV_MIS_Gray;
+	return EV_MIS_ZERO;
 }
 
+Defun_EV_GetMenuItemState_Fn(collab_GetState_CanShare)
+{
+	UT_UNUSED(id);
+
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
+	UT_return_val_if_fail(pManager, EV_MIS_Gray);
+
+	// you can't share a document when no account is online
+	if (!s_any_accounts_online(false))
+		return EV_MIS_Gray;
+
+	// you can open the share dialog when the document is not shared yet, or
+	// when it is shared and it is 'owned' locally
+	
+	PD_Document* pDoc = pView->getDocument();
+	UT_return_val_if_fail(pDoc, EV_MIS_Gray);
+	
+	AbiCollab* session = pManager->getSession(pDoc);
+	if (session)
+	{
+		if (session->isLocallyOwned())
+			return EV_MIS_ZERO;
+		else
+			return EV_MIS_Gray;
+	}
+
+	// the document is not shared yet, but there are active accounts...
+	return EV_MIS_ZERO;
+}
 
 
 /*!
@@ -341,11 +358,11 @@ void s_abicollab_add_menus()
 	EV_Menu_Action* myActionOffer = new EV_Menu_Action (
 		collabOfferId,   	  // id that the layout said we could use
 		0,                      // no, we don't have a sub menu.
-		0,                      // no, we don't raise a dialog.
-		1,                      // yes, we have a checkbox.
+		1,                      // yes, we raise a dialog.
+		0,                      // no, we don't have a checkbox.
 		0,                      // no radio buttons for me, thank you
 		"s_abicollab_offer",    // name of callback function to call.
-		collab_GetState_Joined, // Function for whether not label is enabled/disabled checked/unchecked
+		collab_GetState_CanShare,  // Function for whether not label is enabled/disabled checked/unchecked
 		NULL                    // Function to compute Menu Label "Dynamic Label"
 	);
 	pActionSet->addAction(myActionOffer);
@@ -581,27 +598,43 @@ void s_abicollab_remove_menus()
 bool s_abicollab_offer(AV_View* /*v*/, EV_EditMethodCallData* /*d*/)
 {
 	UT_DEBUGMSG(("s_abicollab_offer\n"));
-	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
-	XAP_Frame *pFrame = XAP_App::getApp()->getLastFocussedFrame();
-	UT_return_val_if_fail(pFrame, false);
-	PD_Document* pDoc = static_cast<PD_Document *>(pFrame->getCurrentDoc());
-	UT_return_val_if_fail(pDoc, false);
 
- 	if (pManager->isLocallyControlled(pDoc))
+	AbiCollabSessionManager* pManager = AbiCollabSessionManager::getManager();
+	UT_return_val_if_fail(pManager, false);
+	
+	// Get the current view that the user is in.
+	XAP_Frame *pFrame = XAP_App::getApp()->getLastFocussedFrame();
+	// Get an Accounts dialog instance
+	XAP_DialogFactory* pFactory = static_cast<XAP_DialogFactory *>(XAP_App::getApp()->getDialogFactory());
+	UT_return_val_if_fail(pFactory, false);
+	AP_Dialog_CollaborationShare* pDialog = static_cast<AP_Dialog_CollaborationShare*>(
+				pFactory->requestDialog(AbiCollabSessionManager::getManager()->getDialogShareId())
+			);
+	// Run the dialog
+	pDialog->runModal(pFrame);
+	// Handle the dialog outcome
+	AP_Dialog_CollaborationShare::tAnswer answer = pDialog->getAnswer();
+	
+	switch (answer)
 	{
-		AbiCollab* pSession = pManager->getSessionFromDocumentId(pDoc->getDocUUIDString());
-		if (pSession)
-		{	
-			pManager->closeSession(pSession, true);
-		}
-	}
-	else
-	{
-		UT_UTF8String sSessionId("");
-		// TODO: we could use/generate a proper descriptor when there is only
-		// 1 account where we share this document over
-		pManager->startSession(pDoc, sSessionId, NULL, "");
-	}
+		case AP_Dialog_CollaborationShare::a_OK:
+			{
+				AccountHandler* pAccount = pDialog->getAccount();
+				const std::vector<std::string> vAcl = pDialog->getAcl();
+				// TODO: move the share() function to the AbiCollabSessionManager
+				pDialog->share(pAccount, vAcl);
+			}
+			break;
+		case AP_Dialog_CollaborationShare::a_CANCEL:
+			break;
+		default:
+			UT_ASSERT_HARMLESS(UT_NOT_REACHED);
+			break;
+	}	
+
+	// Delete the dialog
+	pFactory->releaseDialog(pDialog);
+
 	return true;
 }
 
@@ -639,29 +672,31 @@ bool s_abicollab_join(AV_View* /*v*/, EV_EditMethodCallData* /*d*/)
 	
 	switch (answer)
 	{
-		case AP_Dialog_CollaborationJoin::a_CONNECT:
-			UT_return_val_if_fail(pBuddy && pDocHandle, false);
-			pManager->joinSessionInitiate(pBuddy, pDocHandle);	
-			break;
-		case AP_Dialog_CollaborationJoin::a_DISCONNECT:
+		case AP_Dialog_CollaborationJoin::a_OPEN:
 			{
 				UT_return_val_if_fail(pBuddy && pDocHandle, false);
+				// Check if we have already joined this session. If so, then just
+				// ignore the request. Otherwise actually join the session.
 				AbiCollab* pSession = pManager->getSessionFromSessionId(pDocHandle->getSessionId());
-				UT_return_val_if_fail(pSession, false);
-				if (pSession->isLocallyControlled())
-					pManager->closeSession(pSession, true);
+				if (pSession)
+				{
+					UT_DEBUGMSG(("Already connected to session, ignore this join request\n"));
+					// Doing nothing is not too elegant, but it keeps the Open Shared Document
+					// simple (it does not have to show the "connection" state). This case
+					// won't be often triggered in practice I suppose, so cluttering the
+					// dialog to deal with this obscure case would be a shame.
+				}
 				else
-					pManager->disjoinSession(pDocHandle->getSessionId());
+					pManager->joinSessionInitiate(pBuddy, pDocHandle);	
 			}
 			break;
-		case AP_Dialog_CollaborationJoin::a_CLOSE:
+		case AP_Dialog_CollaborationJoin::a_CANCEL:
 			break;
 	}	
 	
 	return true;
 }
 
-
 bool s_abicollab_accounts(AV_View* /*v*/, EV_EditMethodCallData* /*d*/)
 {
 	// Get the current view that the user is in.
