@@ -4,7 +4,7 @@
  * Copyright (C) 2005 by Martin Sevior
  * Copyright (C) 2006,2007 by Marc Maurer <uwog@uwog.net>
  * Copyright (C) 2007 by One Laptop Per Child
- * Copyright (C) 2008 by AbiSource Corporation B.V.
+ * Copyright (C) 2008-2009 by AbiSource Corporation B.V.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -76,16 +76,22 @@ ChangeAdjust::~ChangeAdjust()
 }
 
 // Use this constructor to host a collaboration session
-AbiCollab::AbiCollab(PD_Document* pDoc, const UT_UTF8String& sSessionId, XAP_Frame* pFrame)
+AbiCollab::AbiCollab(PD_Document* pDoc, 
+						const UT_UTF8String& sSessionId, 
+						AccountHandler* pAclAccount,
+						bool bLocallyOwned,
+						XAP_Frame* pFrame)
 	: EV_MouseListener(),
 	m_pDoc(pDoc),
 	m_pFrame(pFrame),
 	m_Import(this, pDoc),
 	m_Export(this, pDoc),
+	m_pAclAccount(pAclAccount),
 	m_iDocListenerId(0),
 	m_bExportMasked(false),
 	m_sId(sSessionId),
 	m_pController(BuddyPtr()),
+	m_bLocallyOwned(bLocallyOwned),
 	m_pActivePacket(NULL),
 	m_bIsReverting(false),
 	m_pRecorder(NULL),
@@ -114,17 +120,21 @@ AbiCollab::AbiCollab(const UT_UTF8String& sSessionId,
 						PD_Document* pDoc, 
 						const UT_UTF8String& docUUID, 
 						UT_sint32 iRev, 
-						BuddyPtr pController, 
+						BuddyPtr pController,
+						AccountHandler* pAclAccount,
+						bool bLocallyOwned,
 						XAP_Frame* pFrame)
 	: EV_MouseListener(),
 	m_pDoc(pDoc),
 	m_pFrame(pFrame),
 	m_Import(this, pDoc),
 	m_Export(this, pDoc),
+	m_pAclAccount(pAclAccount),
 	m_iDocListenerId(0),
 	m_bExportMasked(false),
 	m_sId(sSessionId),
 	m_pController(pController),
+	m_bLocallyOwned(bLocallyOwned),
 	m_pActivePacket(NULL),
 	m_bIsReverting(false),
 	m_pRecorder(NULL),
@@ -197,6 +207,10 @@ void AbiCollab::removeCollaborator(BuddyPtr pCollaborator)
 			m_vCollaborators.erase(it);
 		}
 	}
+
+	// check if we need to revoke the access rights of this collaborator; if so,
+	// then his rights are revoked
+	_checkRevokeAccess(pCollaborator);
 }
 
 void AbiCollab::_removeCollaborator(BuddyPtr pCollaborator, const std::string& docUUID)
@@ -215,11 +229,47 @@ void AbiCollab::_removeCollaborator(BuddyPtr pCollaborator, const std::string& d
 	m_pDoc->removeCaret(docUUID.c_str());
 }
 
+void AbiCollab::_checkRevokeAccess(BuddyPtr pCollaborator)
+{
+	UT_DEBUGMSG(("AbiCollab::_checkRevokeAccess()\n"));
+	UT_return_if_fail(pCollaborator);
+	UT_return_if_fail(isLocallyControlled());
+	UT_return_if_fail(m_pAclAccount);
+
+	// remove this buddy from the access control list if his access rights
+	// are not persistent
+	if (!pCollaborator->getHandler()->hasPersistentAccessControl())
+	{
+		for (std::vector<std::string>::iterator it = m_vAcl.begin(); it != m_vAcl.end(); it++)
+		{
+			if (pCollaborator->getDescriptor(false) == (*it))
+			{
+				UT_DEBUGMSG(("Dropping %s from the ACL\n", (*it).c_str()));
+				m_vAcl.erase(it);
+				break;
+			}
+		}
+	}
+}
+
 void AbiCollab::addCollaborator(BuddyPtr pCollaborator)
 {
 	UT_DEBUGMSG(("AbiCollab::addCollaborator()\n"));
 	UT_return_if_fail(pCollaborator)
-
+	
+	// check if this buddy is in the access control list if we are hosting 
+	// this session
+	if (isLocallyControlled())
+	{
+		AccountHandler* pAccount = pCollaborator->getHandler();
+		UT_return_if_fail(pAccount);
+		if (!pAccount->hasAccess(m_vAcl, pCollaborator))
+		{
+			UT_ASSERT(UT_NOT_IMPLEMENTED);
+			return;
+		}
+	}
+		
 	// check for duplicates (as long as we assume a collaborator can only be part of a collaboration session once)
 	std::map<BuddyPtr, std::string>::iterator it = m_vCollaborators.find(pCollaborator);
 	if (it != m_vCollaborators.end())
@@ -232,26 +282,10 @@ void AbiCollab::addCollaborator(BuddyPtr pCollaborator)
 	m_vCollaborators[pCollaborator] = ""; // will fill the remote document UUID later once we receive a packet from this buddy
 }
 
-void AbiCollab::removeCollaboratorsForAccount(AccountHandler* pHandler)
+void AbiCollab::setAcl(const std::vector<std::string> vAcl)
 {
-	UT_DEBUGMSG(("AbiCollab::removeCollaboratorsForAccount()\n"));
-	UT_return_if_fail(pHandler);
-
-	std::map<BuddyPtr, std::string>::iterator nit;
-	for (std::map<BuddyPtr, std::string>::iterator it = m_vCollaborators.begin(); it != m_vCollaborators.end(); it = nit)
-	{
-		nit = it;
-		nit++;
-		
-		BuddyPtr pBuddy = (*it).first;
-		UT_continue_if_fail(pBuddy);
-		
-		if (pBuddy->getHandler() == pHandler)
-		{
-			_removeCollaborator(pBuddy, (*it).second);
-			m_vCollaborators.erase(it);
-		}
-	}
+	UT_DEBUGMSG(("AbiCollab::setAcl()\n"));
+	m_vAcl = vAcl;
 }
 
 void AbiCollab::_setDocument(PD_Document* pDoc, XAP_Frame* pFrame)
