@@ -59,6 +59,7 @@
 
 // dialog includes
 #include "xap_DialogFactory.h"
+#include <dialogs/xp/ap_Dialog_CollaborationShare.h>
 #include <dialogs/xp/ap_Dialog_CollaborationJoin.h>
 #include <dialogs/xp/ap_Dialog_CollaborationAccounts.h>
 #include <dialogs/xp/ap_Dialog_CollaborationAddAccount.h>
@@ -71,6 +72,9 @@
 #include <session/xp/AbiCollab.h>
 
 // account handler includes
+#ifdef ABICOLLAB_HANDLER_TELEPATHY
+#include <backends/telepathy/unix/DTubeUnixAccountHandler.h>
+#endif
 #ifdef ABICOLLAB_HANDLER_XMPP
 #include <backends/xmpp/xp/XMPPAccountHandler.h>
 #endif
@@ -81,6 +85,7 @@
 #include <backends/sugar/unix/SugarUnixAccountHandler.h>
 #endif
 #ifdef ABICOLLAB_HANDLER_SERVICE
+#include <backends/service/xp/AbiCollabImport.h>
 #include <backends/service/xp/ServiceAccountHandler.h>
 #include <backends/service/xp/tls_tunnel.h>
 #endif
@@ -211,7 +216,8 @@ UT_Error AbiCollabSessionManager::deserializeDocument(PD_Document** pDoc, const
 }
 
 AbiCollabSessionManager::AbiCollabSessionManager(void)
-	: m_iDialogJoin(0),
+	: m_iDialogShare(0),
+	m_iDialogJoin(0),
 	m_iDialogAccounts(0),
 	m_iDialogAddAccount(0),
 	m_iDialogAddBuddy(0)
@@ -230,6 +236,7 @@ AbiCollabSessionManager::~AbiCollabSessionManager(void)
 bool AbiCollabSessionManager::registerDialogs(void)
 {
 	XAP_DialogFactory * pFactory = static_cast<XAP_DialogFactory *>(XAP_App::getApp()->getDialogFactory());
+	m_iDialogShare = pFactory->registerDialog(ap_Dialog_CollaborationShare_Constructor, XAP_DLGT_NON_PERSISTENT);
 	m_iDialogJoin = pFactory->registerDialog(ap_Dialog_CollaborationJoin_Constructor, XAP_DLGT_NON_PERSISTENT);
 	m_iDialogAccounts = pFactory->registerDialog(ap_Dialog_CollaborationAccounts_Constructor, XAP_DLGT_NON_PERSISTENT);
 	m_iDialogAddAccount = pFactory->registerDialog(ap_Dialog_CollaborationAddAccount_Constructor, XAP_DLGT_NON_PERSISTENT);
@@ -241,6 +248,15 @@ bool AbiCollabSessionManager::registerAccountHandlers()
 {
 	UT_DEBUGMSG(("AbiCollabSessionManager::registerAccountHandlers()\n"));
 
+#ifdef ABICOLLAB_HANDLER_TELEPATHY
+	// we don't want to register a d-bus tube account handler here so
+	// swe can construct multiple sugar account handlers later: the 
+	// d-bus tube account handler is a singleton, that should always
+	// be active if it is compiled in
+	UT_DEBUGMSG(("Registering the telepathy account handler!\n"));
+	AccountHandler* pDTubeHandler = new DTubeAccountHandler();
+	addAccount(pDTubeHandler);
+#endif
 #ifdef ABICOLLAB_HANDLER_XMPP
 	m_regAccountHandlers[XMPPAccountHandler::getStaticStorageType()] = XMPPAccountHandlerConstructor;
 #endif
@@ -248,8 +264,8 @@ bool AbiCollabSessionManager::registerAccountHandlers()
 	m_regAccountHandlers[TCPAccountHandler::getStaticStorageType()] = TCPAccountHandlerConstructor;
 #endif
 #ifdef ABICOLLAB_HANDLER_SUGAR
-	// we don't want to regerister a sugar account handler here, 
-	// so we can construct multiple sugar account handlers: the 
+	// we don't want to register a sugar account handler here so
+	// we can construct multiple sugar account handlers later: the 
 	// sugar account handler is a singleton, that should always
 	// be active if it is compiled in
 	UT_DEBUGMSG(("Registering the sugar account handler!\n"));
@@ -259,12 +275,16 @@ bool AbiCollabSessionManager::registerAccountHandlers()
 #ifdef ABICOLLAB_HANDLER_SERVICE
 	if (tls_tunnel::Proxy::tls_tunnel_init())
 		m_regAccountHandlers[ServiceAccountHandler::getStaticStorageType()] = ServiceAccountHandlerConstructor;
+	IE_Imp_AbiCollabSniffer* pAbiCollabSniffer = new IE_Imp_AbiCollabSniffer();
+	IE_Imp::registerImporter(pAbiCollabSniffer);
+	m_vImpSniffers.push_back(pAbiCollabSniffer);
 #endif
 	return true;
 }
 
 bool AbiCollabSessionManager::unregisterAccountHandlers(void)
 {
+	UT_DEBUGMSG(("AbiCollabSessionManager::unregisterAccountHandlers()\n"));
 	// no need to "free/delete" items, as they are just function pointers (ie. basic types)
 	m_regAccountHandlers.clear();
 #ifdef ABICOLLAB_HANDLER_SERVICE
@@ -273,9 +293,21 @@ bool AbiCollabSessionManager::unregisterAccountHandlers(void)
 	return true;
 }
 
+void AbiCollabSessionManager::unregisterSniffers(void)
+{
+	for (UT_uint32 i = 0; i < m_vImpSniffers.size(); i++)
+	{
+		IE_ImpSniffer* pSniffer = m_vImpSniffers[i];
+		UT_continue_if_fail(pSniffer);
+		IE_Imp::unregisterImporter(pSniffer);
+	}
+	m_vImpSniffers.clear();
+}
+
 bool AbiCollabSessionManager::unregisterDialogs(void)
 {
 	XAP_DialogFactory * pFactory = static_cast<XAP_DialogFactory *>(XAP_App::getApp()->getDialogFactory());
+	pFactory->unregisterDialog(m_iDialogShare);
 	pFactory->unregisterDialog(m_iDialogJoin);
 	pFactory->unregisterDialog(m_iDialogAccounts);
 	pFactory->unregisterDialog(m_iDialogAddAccount);
@@ -634,11 +666,14 @@ AbiCollab* AbiCollabSessionManager::getSessionFromSessionId(const UT_UTF8String&
 }
 
 AbiCollab* AbiCollabSessionManager::startSession(PD_Document* pDoc, UT_UTF8String& sSessionId, 
-			XAP_Frame* pFrame, const UT_UTF8String& masterDescriptor)
+			AccountHandler* pAclAccount, bool bLocallyOwned, XAP_Frame* pFrame, 
+			const UT_UTF8String& masterDescriptor)
 {
 	UT_DEBUGMSG(("Starting collaboration session for document with id %s, master descriptor: %s\n",
 			pDoc->getDocUUIDString(), masterDescriptor.utf8_str()));
-	
+	UT_return_val_if_fail(pDoc, NULL);
+	UT_return_val_if_fail(pAclAccount, NULL);
+
 	if (sSessionId == "")
 	{
 		XAP_App* pApp = XAP_App::getApp();	
@@ -712,10 +747,13 @@ AbiCollab* AbiCollabSessionManager::startSession(PD_Document* pDoc, UT_UTF8Strin
 	UT_DEBUGMSG(("Creating a new collaboration session with UUID: %s\n", sSessionId.utf8_str()));
 
 	UT_return_val_if_fail(_setupFrame(&pFrame, pDoc), NULL);
-	AbiCollab* pAbiCollab = new AbiCollab(pDoc, sSessionId, pFrame);
+	AbiCollab* pAbiCollab = new AbiCollab(pDoc, sSessionId, pAclAccount, bLocallyOwned, pFrame);
 	m_vecSessions.push_back(pAbiCollab);
 	
 	// notify all people we are sharing a new document
+	// FIXME: since we only allow a session to be shared on 1 account, we should
+	// only notify the buddies on that account, instead of notifying the buddies
+	// on all accounts.
 	StartSessionEvent event;
 	event.setBroadcast(true);
 	signal(event);
@@ -784,12 +822,14 @@ void AbiCollabSessionManager::joinSessionInitiate(BuddyPtr pBuddy, DocHandle* pD
 void AbiCollabSessionManager::joinSession(const UT_UTF8String& sSessionId, PD_Document* pDoc, 
 												const UT_UTF8String& docUUID, UT_sint32 iRev, 
 												UT_sint32 iAuthorId, BuddyPtr pCollaborator,
+												AccountHandler* pAclAccount, bool bLocallyOwned, 
 												XAP_Frame *pFrame)
 {
 	UT_DEBUGMSG(("AbiCollabSessionManager::joinSession()\n"));
 
 	UT_return_if_fail(pCollaborator);
 	UT_return_if_fail(pDoc);
+	UT_return_if_fail(pAclAccount);
 	
 #ifdef ABICOLLAB_HANDLER_SUGAR		
 	// HACK, remove this some day: the sugar backend should just pass us a frame to use
@@ -799,7 +839,7 @@ void AbiCollabSessionManager::joinSession(const UT_UTF8String& sSessionId, PD_Do
 	UT_return_if_fail(_setupFrame(&pFrame, pDoc));
 #endif
 
-	AbiCollab* pSession = new AbiCollab(sSessionId, pDoc, docUUID, iRev, pCollaborator, pFrame);
+	AbiCollab* pSession = new AbiCollab(sSessionId, pDoc, docUUID, iRev, pCollaborator, pAclAccount, bLocallyOwned, pFrame);
 	m_vecSessions.push_back(pSession);
 
 	// signal everyone that we have joined this session
@@ -962,6 +1002,37 @@ void AbiCollabSessionManager::removeBuddy(BuddyPtr pBuddy, bool graceful)
 	}
 }
 
+// should we move this to AbiCollab.cpp ?
+void AbiCollabSessionManager::updateAcl(AbiCollab* pSession, AccountHandler* pAccount, const std::vector<std::string> vAcl)
+{
+	UT_return_if_fail(pSession);
+	UT_return_if_fail(pAccount);
+
+	// check if all current collaborators are still allowed to collaborate; if not,
+	// then remove them from the session
+	const std::map<BuddyPtr, std::string> collaborators = pSession->getCollaborators();
+	for (std::map<BuddyPtr, std::string>::const_iterator cit = collaborators.begin(); cit != collaborators.end(); cit++)
+	{
+		BuddyPtr pBuddy = (*cit).first;
+		UT_continue_if_fail(pBuddy);
+		AccountHandler* pBuddyAccount = pBuddy->getHandler();
+		UT_continue_if_fail(pBuddyAccount);
+		UT_continue_if_fail(pBuddyAccount == pAccount);
+		if (!pBuddyAccount->hasAccess(vAcl, pBuddy))
+		{
+			// this current collaborator has been banned from the session, so
+			// disconnect him
+			UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
+		}
+	}
+
+	// set the new ACL on the account handler
+	pAccount->setAcl(pSession, vAcl);
+
+	// set the new access control list on the session
+	pSession->setAcl(vAcl);
+}
+
 bool AbiCollabSessionManager::addAccount(AccountHandler* pHandler)
 {
 	UT_return_val_if_fail(pHandler, false);
@@ -1017,14 +1088,12 @@ bool AbiCollabSessionManager::destroyAccount(AccountHandler* pHandler)
 				AbiCollab* pSession = m_vecSessions.getNthItem(j); 
 				UT_continue_if_fail(pSession);
 				
-				// TODO: do we need to do something extra if the session is
-				// locally controlled?
-				pSession->removeCollaboratorsForAccount(pHandler);
-				
-				// if this session has no collaborators anymore, then drop it
-				if (pSession->getCollaborators().size() == 0)
+				// There can only be buddies from 1 account in an active session these days/
+				// That means that if this session's account is the account we are destroying,
+				// then we can kill off the entire session. Do nothing otherwise.
+				if (pSession->getAclAccount() == pHandler)
 				{
-					UT_DEBUGMSG(("All collaborators left from session %s, destroying it!\n", pSession->getSessionId().utf8_str()));
+					UT_DEBUGMSG(("Session %s is running on this account, destroying it!\n", pSession->getSessionId().utf8_str()));
 					destroySession(pSession);
 				}
 			}
@@ -1168,7 +1237,6 @@ bool AbiCollabSessionManager::processPacket(AccountHandler& /*handler*/, Packet*
 	switch (pct) {
 		case PCT_StartSessionEvent:
 		{
-			// TODO: it is rather inefficient to request a buddy for every packet
 			StartSessionEvent event;
 			event.setBroadcast(true);
 			signal(event, buddy);
@@ -1245,7 +1313,9 @@ bool AbiCollabSessionManager::processPacket(AccountHandler& /*handler*/, Packet*
 					
 					// the server hosting this session is gone, so let's disconnect as well
 					if (!destroySession(pSession))
+					{
 						UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+					}
 
 					// signal all 
 					CloseSessionEvent event( destroyedSessionId );
@@ -1416,8 +1486,13 @@ void AbiCollabSessionManager::_deleteSession(AbiCollab* pSession)
 	UT_return_if_fail(pSession);
 	// wait for all async actions to complete
 	// TODO: some sort of feedback to the user would be nice
+	bool caughtQuit = false;
 	while (m_asyncSessionOps[pSession] > 0)
-		_nullUpdate();
+		caughtQuit |= _nullUpdate();
+#if defined(TOOLKIT_WIN)
+	if (caughtQuit)
+		PostQuitMessage(0);
+#endif
 	DELETEP(pSession);
 }
 
@@ -1427,19 +1502,35 @@ void AbiCollabSessionManager::_deleteAccount(AccountHandler* pHandler)
 	UT_return_if_fail(pHandler);
 	// wait for all async actions to complete
 	// TODO: some sort of feedback to the user would be nice
+	bool caughtQuit = false;
 	while (m_asyncAccountOps[pHandler] > 0)
-		_nullUpdate();			
+		caughtQuit |= _nullUpdate();
+#if defined(TOOLKIT_WIN)
+	if (caughtQuit)
+		PostQuitMessage(0);
+#endif
 	DELETEP(pHandler);
 }
 
-void AbiCollabSessionManager::_nullUpdate()
+// Note: the return value only makes sense on Win32, other platforms
+// should ignore it. On Win32, it denotes wether a WM_QUIT message
+// has been eaten (and thus should be reposted to the main message 
+// queue later.
+bool AbiCollabSessionManager::_nullUpdate()
 {
+	bool caughtQuit = false;
 #if defined(TOOLKIT_WIN)
-		MSG msg;
-		for (UT_sint32 i = 0 ; i < 10 ; i++ )
-			if (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE))
-				DispatchMessage(&msg);
-		Sleep(10);
+	MSG msg;
+	for (UT_sint32 i = 0 ; i < 10 ; i++ )
+	{
+		if (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE))
+		{
+			if (msg.message == WM_QUIT)
+				caughtQuit = true;
+			DispatchMessage(&msg);
+		}
+	}
+	Sleep(10);
 #elif defined(TOOLKIT_GTK)
 		for (UT_sint32 i = 0; (i < 10) && gtk_events_pending(); i++)
 			gtk_main_iteration ();
@@ -1449,6 +1540,7 @@ void AbiCollabSessionManager::_nullUpdate()
 #else 
 #error unknown platform
 #endif	
+	return caughtQuit;
 }
 
 bool AbiCollabSessionManager::_canInitiateSessionTakeover(AbiCollab* pSession)
