@@ -35,6 +35,11 @@
 #include "ODi_StartTag.h"
 #include "ODi_ElementStack.h"
 #include "ODi_TableOfContent_ListenerState.h"
+#include "ODi_Abi_Data.h"
+#include "ut_growbuf.h"
+#include "pf_Frag.h"
+#include "ie_exp_RTF.h"
+#include "ut_units.h"
 
 // AbiWord includes
 #include <ut_misc.h>
@@ -48,7 +53,8 @@
 ODi_TextContent_ListenerState::ODi_TextContent_ListenerState (
                 PD_Document* pDocument,
                 ODi_Office_Styles* pStyles,
-                ODi_ElementStack& rElementStack)
+                ODi_ElementStack& rElementStack,
+		ODi_Abi_Data& rAbiData)
                 : ODi_ListenerState("TextContent", rElementStack),
                   m_pAbiDocument ( pDocument ),
                   m_pStyles(pStyles),
@@ -67,7 +73,14 @@ ODi_TextContent_ListenerState::ODi_TextContent_ListenerState (
                   m_bPendingAnnotation(false),
                   m_bPendingAnnotationAuthor(false),
                   m_bPendingAnnotationDate(false),
-                  m_iAnnotation(0)
+                  m_iAnnotation(0),
+		  m_bPageReferencePending(false),
+		  m_iPageNum(0),
+		  m_dXpos(0.0),
+		  m_dYpos(0.0),
+		  m_sProps(""),
+		  m_rAbiData(rAbiData),
+		  m_bPendingTextbox(false)
 {
     UT_ASSERT_HARMLESS(m_pAbiDocument);
     UT_ASSERT_HARMLESS(m_pStyles);
@@ -77,13 +90,11 @@ ODi_TextContent_ListenerState::ODi_TextContent_ListenerState (
 /**
  * Destructor
  */
-ODi_TextContent_ListenerState::~ODi_TextContent_ListenerState() {
-
-    UT_ASSERT_HARMLESS(m_tablesOfContentProps.getItemCount()==0);
-   
+ODi_TextContent_ListenerState::~ODi_TextContent_ListenerState() 
+{
     if (m_tablesOfContentProps.getItemCount() > 0) {
+        UT_DEBUGMSG(("ERROR ODti: table of content props not empty\n"));
         UT_VECTOR_PURGEALL(UT_UTF8String*, m_tablesOfContentProps);
-        m_tablesOfContentProps.clear();
     }
 }
 
@@ -427,7 +438,9 @@ void ODi_TextContent_ListenerState::startElement (const gchar* pName,
     } else if (!strcmp(pName, "draw:frame")) {
         
         if (!strcmp(m_rElementStack.getStartTag(0)->getName(), "text:p") ||
-            !strcmp(m_rElementStack.getStartTag(0)->getName(), "text:h")) {
+            !strcmp(m_rElementStack.getStartTag(0)->getName(), "text:h") ||
+            !strcmp(m_rElementStack.getStartTag(0)->getName(), "office:text")) 
+	{
             
             const gchar* pVal = NULL;
             
@@ -437,7 +450,7 @@ void ODi_TextContent_ListenerState::startElement (const gchar* pName,
             if (pVal && (!strcmp(pVal, "paragraph") ||
                 !strcmp(pVal, "page"))) {
                 // It's postponed because AbiWord uses frames *after* the
-                // paragraph but OpenDocument uses then inside the paragraphs,
+                // paragraph but OpenDocument uses them inside the paragraphs,
                 // right *before* its content.
                 rAction.postponeElementParsing("Frame");
             } else {
@@ -446,20 +459,106 @@ void ODi_TextContent_ListenerState::startElement (const gchar* pName,
                 rAction.pushState("Frame");
             }
             
-        } else if (!strcmp(m_rElementStack.getStartTag(0)->getName(), "office:text")) {
+        } 
+	else if (!strcmp(m_rElementStack.getStartTag(0)->getName(), "office:text")) 
+	{
             
             // A page anchored frame.
-            //
-            // Page anchored frames defined outside a paragraph specifies the
-            // number of the page that they are anchored to. As I can't translate
-            // this info easily (if not at all) into AbiWord I will ignore all
-            // OpenDocument page anchored frames defined that way.
-            
-            // Ignore this element
-            rAction.ignoreElement();
-           
-        } else if (!strcmp(m_rElementStack.getStartTag(0)->getName(),
-                              "text:span")) {
+            // Store this in the PD_Document until after the rest of the
+	  // is layed out.
+	  // First acquired the info we need
+	  const gchar* pVal = NULL;
+	  bool bCont = true;
+	  m_iPageNum = 0;
+	  m_dXpos = 0.0;
+	  m_dYpos = 0.0;
+	  m_sProps.clear();
+	  m_bPageReferencePending = false;
+	  pVal = UT_getAttribute("text:anchor-page-number", ppAtts);
+	  if(!pVal || !*pVal)
+	  {
+	      rAction.ignoreElement();
+	      bCont = false;
+	  }
+	  if(bCont)
+	  {
+	      m_iPageNum = atoi(pVal);
+	  }
+	  pVal = UT_getAttribute("svg:x", ppAtts);
+	  if(!pVal || !*pVal)
+	  {
+	      rAction.ignoreElement();
+	      bCont = false;
+	  }
+	  if(bCont)
+	  {
+	      m_dXpos = UT_convertToInches(pVal);
+	  }
+	  pVal = UT_getAttribute("svg:y", ppAtts);
+	  if(!pVal || !*pVal)
+	  {
+	      rAction.ignoreElement();
+	      bCont = false;
+	  }
+	  if(bCont)
+	  {
+	      m_dYpos = UT_convertToInches(pVal);
+	      m_bPageReferencePending=true;
+	  }
+	  pVal = UT_getAttribute("svg:width", ppAtts);
+	  if(pVal && *pVal)
+	  {
+	      UT_UTF8String_setProperty(m_sProps,"frame-width",pVal);
+	  }
+	  pVal = UT_getAttribute("svg:height", ppAtts);
+	  if(pVal && *pVal)
+	  {
+	      UT_UTF8String_setProperty(m_sProps,"frame-height",pVal);
+	  }
+	  //
+	  // Get wrapping style
+	  //
+	  const gchar* pStyleName = NULL;
+	  pStyleName = UT_getAttribute("draw:style-name", ppAtts);
+	  if(pStyleName)
+	  {
+	      const ODi_Style_Style* pGraphicStyle = m_pStyles->getGraphicStyle(pStyleName, m_bOnContentStream);
+	      if(pGraphicStyle)
+	      {
+		  const UT_UTF8String* pWrap=NULL;
+		  pWrap = pGraphicStyle->getWrap(false);
+		  if(pWrap)
+		  {
+		      if ( !strcmp(pWrap->utf8_str(), "run-through")) 
+		      {
+			  // Floating wrapping.
+			  m_sProps += "; wrap-mode:above-text";
+		      } 
+		      else if ( !strcmp(pWrap->utf8_str(), "left")) 
+		      {
+			  m_sProps += "; wrap-mode:wrapped-to-left";
+		      } 
+		      else if ( !strcmp(pWrap->utf8_str(), "right")) 
+		      {
+			  m_sProps += "; wrap-mode:wrapped-to-right";
+		      } 
+		      else if ( !strcmp(pWrap->utf8_str(), "parallel")) 
+		      {
+			  m_sProps += "; wrap-mode:wrapped-both";
+		      } 
+		      else 
+		      {
+			  // Unsupported.        
+			  // Let's put an arbitrary wrap mode to avoid an error.
+			  m_sProps += "; wrap-mode:wrapped-both";
+		      }
+		  }
+	      }
+	  }
+        } 
+	else if (!strcmp(m_rElementStack.getStartTag(0)->getName(),
+                              "text:span")) 
+        {
             // Must be an inlined image, otherwise we can't handle it.
             
             const gchar* pVal;
@@ -469,24 +568,99 @@ void ODi_TextContent_ListenerState::startElement (const gchar* pName,
             if (!pVal || !strcmp(pVal, "as-char") || !strcmp(pVal, "char")) {
                 _flush();
                 rAction.pushState("Frame");
-            } else {
+            }
+	    else {
                 // Ignore this element
                 rAction.ignoreElement();
             }
 
-        } else {
+        }
+	else 
+        {
             UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
             // TODO: Figure out what to do then.
         }
         
-    } else if (!strcmp(pName, "draw:text-box")) {
-        UT_ASSERT(m_elementParsingLevel == 0);
+    } 
+    else if (!strcmp(pName, "draw:image"))
+    {
+	if(m_bPageReferencePending)
+	{
+	  //
+	  // We have page referenced image to handle.
+	  //  
+	  UT_String dataId; // id of the data item that contains the image.
+	  m_rAbiData.addImageDataItem(dataId, ppAtts);
+	  const gchar* pStyleName;
+	  const ODi_Style_Style* pGraphicStyle;
+	  const UT_UTF8String* pWrap;
+    
+	  pStyleName = m_rElementStack.getStartTag(0)->getAttributeValue("draw:style-name");
+	  UT_ASSERT(pStyleName);
+    
+	  pGraphicStyle = m_pStyles->getGraphicStyle(pStyleName, true);
+	  if(pGraphicStyle)
+	  {    
+	    pWrap = pGraphicStyle->getWrap(false);
+                                                    
+	    if ( !strcmp(pWrap->utf8_str(), "run-through")) 
+	    {
+		// Floating wrapping.
+		m_sProps += "; wrap-mode:above-text";
+		
+	    } 
+	    else if ( !strcmp(pWrap->utf8_str(), "left")) 
+	    {
+		m_sProps += "; wrap-mode:wrapped-to-left";
+	    } 
+	    else if ( !strcmp(pWrap->utf8_str(), "right")) 
+	    {
+		m_sProps += "; wrap-mode:wrapped-to-right";
+	    } 
+	    else if ( !strcmp(pWrap->utf8_str(), "parallel")) 
+	    {
+		m_sProps += "; wrap-mode:wrapped-both";
+	    } 
+	    else 
+	    {
+		// Unsupported.        
+		// Let's put an arbitrary wrap mode to avoid an error.
+		m_sProps += "; wrap-mode:wrapped-both";
+	    }
+	  }
+	    //
+	    // OK lets write this into the document for later use
+	    //
+	  UT_UTF8String sImageId = dataId.c_str();
+	  m_pAbiDocument->addPageReferencedImage(sImageId, m_iPageNum, m_dXpos, m_dYpos, m_sProps.utf8_str());
+	  
+	  m_bPageReferencePending = false;
+	}
+	else
+	{
+	  // Ignore this element
+	  rAction.ignoreElement();
+	}
+    }
+    else if (!strcmp(pName, "draw:text-box")) 
+    {
         
         // We're inside a text-box, parsing its text contents.
         m_inAbiSection = true;
         m_bOnContentStream = true;
-        
+	if(m_bPageReferencePending)
+	{
+	  //
+	  // We have page referenced text box to handle.
+	  // Start collecting text after first gathering the infor we
+	  // for the frame
+	  //  
+	  m_bPendingTextbox = true;
+	  m_sProps += ";bot-style:1; left-style:1; right-style:1; top-style:1";
+	  
+	}        
     } else if (!strcmp(pName, "draw:g")) {
+      UT_DEBUGMSG(("Unallowed drawing element %s \n",pName));
        rAction.ignoreElement();  // ignore drawing shapes since AbiWord can't handle them
 
     } else if (!strcmp(pName, "table:table")) {
@@ -719,10 +893,16 @@ void ODi_TextContent_ListenerState::endElement (const gchar* pName,
         }
         
     } else if (!strcmp(pName, "draw:text-box")) {
-        UT_ASSERT(m_elementParsingLevel == 1);
         
         // We were inside a <draw:text-box> element.
-        rAction.popState();
+	if(m_bPageReferencePending || m_bPendingTextbox)
+	{
+	    m_bPageReferencePending = false;
+	}
+	else
+	{
+	    rAction.popState();
+	}
         
     } else if (!strcmp(pName, "table:table-cell")) {
         UT_ASSERT(m_elementParsingLevel == 1);
@@ -781,7 +961,45 @@ void ODi_TextContent_ListenerState::endElement (const gchar* pName,
             m_bPendingAnnotationDate = false;
             m_bAcceptingText = true;
         }
-
+    }
+    else if (!strcmp(pName, "draw:frame")) {
+      m_bPageReferencePending = false;
+      m_bAcceptingText = true;
+      if(m_bPendingTextbox)
+      {
+	  m_bPendingTextbox = false;
+	  _flush();
+	  PT_DocPosition pos2 = 0;
+	  m_pAbiDocument->getBounds(true,pos2);
+	  UT_ByteBuf * pBuf = new UT_ByteBuf(1024);
+	  pf_Frag * pfLast = m_pAbiDocument->getLastFrag();
+	  pf_Frag * pfFirst = pfLast;
+	  while(pfFirst->getPrev())
+	  {
+	      pfFirst = pfFirst->getPrev();
+	  }
+	  PT_DocPosition pos1 = pfFirst->getPos();
+	  IE_Exp_RTF * pExpRtf = new IE_Exp_RTF(m_pAbiDocument);
+	  PD_DocumentRange docRange(m_pAbiDocument, pos1,pos2);
+	  //
+	  // Copy the Textbox content to RTF and store for later insertion
+	  //
+	  pExpRtf->copyToBuffer(&docRange,pBuf);
+	  delete pExpRtf;
+	  m_pAbiDocument->addPageReferencedTextbox(*pBuf,m_iPageNum, m_dXpos,m_dYpos,m_sProps.utf8_str());
+	  delete pBuf;
+	  pf_Frag * pfNext = pfFirst; 
+	  //
+	  // Remove this textbox content from the PT
+	  //
+	  while(pfFirst)
+	  {
+	      pfNext = pfFirst->getNext();
+	      m_pAbiDocument->deleteFragNoUpdate(pfFirst);
+	      pfFirst = pfNext;
+	  }
+	  m_bAcceptingText = false;
+      }
     }
     
     m_elementParsingLevel--;
@@ -886,8 +1104,7 @@ void ODi_TextContent_ListenerState::_popInlineFmt(void)
         p = (const gchar *)m_vecInlineFmt.getNthItem(k-1);
         m_vecInlineFmt.deleteNthItem(k-1);
         
-        if (p)
-            free((void *)p);
+        FREEP(p);
     }
 }
 
@@ -904,11 +1121,20 @@ void ODi_TextContent_ListenerState::_insureInSection(
                                          const UT_UTF8String* pMasterPageName) {
     
     if (m_inAbiSection) {
+      //
+      // FIXME Sevior says I do not know what this ifdeffed code was 
+      // trying to do
+      // If we're in a section we should not change the section properties.
+      // I ifdeffed the code out in case some other bug shows we need it.
+      //
+#if 0
         if (pMasterPageName == NULL ||
             (pMasterPageName != NULL && pMasterPageName->empty()) ) {
             // There's nothing to be done.
-            return;
+                return;
         }
+#endif
+	return;
     }
     
     
@@ -977,7 +1203,45 @@ void ODi_TextContent_ListenerState::_openAbiSection(
             dataID = pMasterPageStyle->getSectionDataID();
             UT_ASSERT(!allProps.empty());
         }
-        
+	//
+	// Page size is defined from the first section properties
+	//
+        if(!m_openedFirstAbiSection)
+	{
+	    UT_UTF8String sProp(""),sWidth(""),sHeight(""),sOri("");
+	    bool bValid = true;
+	    
+	    sProp="page-width";
+	    sWidth = UT_UTF8String_getPropVal(allProps,sProp);
+	    if(sWidth.size()==0)
+	        bValid = false;
+
+	    sProp="page-height";
+	    sHeight = UT_UTF8String_getPropVal(allProps,sProp);
+	    if(sHeight.size()==0)
+	        bValid = false;
+
+	    sProp="page-orientation";
+	    sOri = UT_UTF8String_getPropVal(allProps,sProp);
+	    if(sOri.size()==0)
+	        bValid = false;
+	    if(bValid)
+	    {
+	        UT_UTF8String sUnits = UT_dimensionName(UT_determineDimension(sWidth.utf8_str()));
+	        const gchar * atts[13] ={"pagetype","Custom",
+					 "orientation",NULL,
+					 "width",NULL,
+					 "height",NULL,
+					 "units",NULL,
+					 "page-scale","1.0",
+					 NULL};
+		atts[3] = sOri.utf8_str();
+		atts[5] = sWidth.utf8_str();
+		atts[7] = sHeight.utf8_str();
+		atts[9] = sUnits.utf8_str();
+		m_pAbiDocument->setPageSizeFromFile(atts);
+	    }
+	}
         m_openedFirstAbiSection = true;
     }
 
@@ -1060,7 +1324,7 @@ void ODi_TextContent_ListenerState::_openAbiSection(
     //
     // That's the only reason for the existence of *pending* paragraph
     // (column or page) breaks.
-    _flushPendingParagraphBreak();
+     _flushPendingParagraphBreak();
 
     m_inAbiSection = true;
     m_bAcceptingText = false;
