@@ -16,6 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
+ 
+/*
+	In versions previous to Windows Vista we use owner-draw menus. 
+	In Vista we use SetMenuItemBitmaps that is more powerful that previous Windows editions
+*/
 
 #include <windows.h>
 #include <stdio.h>
@@ -42,8 +47,11 @@
 #include "xap_Win32App.h"
 #include "xap_Win32Toolbar_Icons.h"
 #include "ap_Win32App.h"
+#include "ut_Win32OS.h"
 
 #define SPACE_ICONTEXT	4	// Pixels between the icon and the text
+#define BITMAP_WITDH	16
+#define BITMAP_HEIGHT	16
 
 
 /*
@@ -125,7 +133,7 @@ static const EV_Menu_Bitmap s_bitmaps[] =
 
 */
 static const char * _ev_GetLabelName(XAP_Win32App * pWin32App,
-									 XAP_Frame * pFrame, 
+									 XAP_Frame * /*pFrame*/, 
 									 const EV_EditEventMapper * pEEM,
 									 const EV_Menu_Action * pAction,
 									 EV_Menu_Label * pLabel)
@@ -177,8 +185,8 @@ static const char * _ev_GetLabelName(XAP_Win32App * pWin32App,
 	}
 
 	static char buf[128];
-	memset(buf,0,NrElements(buf));
-	strncpy(buf,szLabelName,NrElements(buf)-len);
+	memset(buf,0,G_N_ELEMENTS(buf));
+	strncpy(buf,szLabelName,G_N_ELEMENTS(buf)-len);
 
 	// append "..." to menu item if it raises a dialog
 	if (pAction->raisesDialog())
@@ -216,23 +224,8 @@ EV_Win32Menu::EV_Win32Menu(XAP_Win32App * pWin32App,
     SystemParametersInfoA(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICSA), &ncm, 0);
 	m_hFont = CreateFontIndirectA(&ncm.lfMenuFont);	
 
-	m_nBitmapCX = m_nBitmapCY = 24;	// Default value overwritten later
-
-	UT_RGBColor color;
-	HBITMAP hBitmap = EV_Win32Menu::_loadBitmap(s_bitmaps[0].id, m_nBitmapCX, m_nBitmapCY, color);
-	
-	if (hBitmap)	/* Default size for bitmaps*/
-	{
-		BITMAP bitmap;
-		
-		if (GetObjectA(hBitmap, sizeof(BITMAP), &bitmap))
-		{
-			m_nBitmapCX = bitmap.bmWidth+2;
-			m_nBitmapCY = bitmap.bmHeight+2;
-		}
-
-		DeleteObject(hBitmap);
-	}						
+	m_nBitmapCX = BITMAP_WITDH + 2;
+	m_nBitmapCY = BITMAP_HEIGHT + 2;
 
 	UT_return_if_fail( m_pMenuLabelSet );
 	
@@ -261,11 +254,17 @@ EV_Win32Menu::~EV_Win32Menu()
 {	
 	// we let the derived classes handle destruction of m_myMenu if appropriate.
 	// TODO: this is never colld
+	
+	for (std::vector<HBITMAP>::const_iterator i = m_vechBitmaps.begin(); 
+		 i != m_vechBitmaps.end(); i++)
+	{	    
+	    DeleteObject (*i);
+	}
 		
 }
 
 bool EV_Win32Menu::onCommand(AV_View * pView,
-							 HWND hWnd, WPARAM wParam)
+							 HWND /*hWnd*/, WPARAM wParam)
 {
 	// TODO do we need the hWnd parameter....
 
@@ -326,12 +325,8 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 	{
 		
 		EV_Menu_LayoutItem * pLayoutItem = m_pMenuLayout->getLayoutItem(k);
-		if(!pLayoutItem)
-		{
-			UT_ASSERT_HARMLESS(pLayoutItem);
-			continue;
-		}
-		
+		UT_continue_if_fail(pLayoutItem);
+
 		XAP_Menu_Id id = pLayoutItem->getMenuId();
 		const EV_Menu_Action * pAction = pMenuActionSet->getAction(id);
 		UT_ASSERT(pAction);
@@ -388,8 +383,14 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 							
 						if (!m_bTrack && stack.getDepth()==2)
 							AppendMenu(m, flags,u, szLabelName);
-						else
-							AppendMenu(m, flags|MF_OWNERDRAW,u, (const char*) item);
+						else {
+							if (UT_IsWinVista()) {
+								AppendMenu(m, flags|MF_STRING,u, szLabelName);
+								_setBitmapforID(m, id, u);
+							}
+							else
+								AppendMenu(m, flags|MF_OWNERDRAW,u, (const char*) item);
+						}
 					}		
 						
 				}
@@ -402,8 +403,13 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 						item->pMenu= this;						
 						strcpy (item->szText, szLabelName);
 						m_vecItems.addItem(item);
-
-						AppendMenu(m, MF_OWNERDRAW , u, (const char*) item);
+						
+						if (UT_IsWinVista()) {
+							AppendMenu(m, MF_STRING, u, szLabelName);
+							_setBitmapforID (m, id, u);
+						}
+						else
+							AppendMenu(m, MF_OWNERDRAW, u, (const char*) item);
 					}
 				}
 
@@ -442,7 +448,7 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 		}
 	}
 
-#ifdef UT_DEBUG
+#ifdef DEBUG
 	HMENU wDbg = NULL;
 	bResult = stack.pop((void **)&wDbg);
 	UT_ASSERT(bResult);
@@ -452,7 +458,7 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 	return true;
 }
 
-bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HMENU hMenuBar)
+bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND /*hWnd*/, HMENU hMenuBar)
 {
 	// deal with WM_INITMENU.
 
@@ -521,7 +527,7 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 				char bufMIF[128];
 				mif.cbSize = sizeof(mif);
 				mif.dwTypeData = bufMIF;
-				mif.cch = NrElements(bufMIF)-1;
+				mif.cch = G_N_ELEMENTS(bufMIF)-1;
 				mif.fMask = MIIM_STATE | MIIM_TYPE | MIIM_ID;
 				BOOL bPresent = GetMenuItemInfo(hMenuBar,cmd,FALSE,&mif);
 
@@ -549,13 +555,20 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 					// dynamic label has changed
 					XAP_Menu_Id id = MenuIdFromWmCommand(cmd);
 					EV_Menu_Item*	item;
-					for(UT_uint32 i = 0; i< m_vecItems.getItemCount(); i++)
+					for(UT_sint32 i = 0; i< m_vecItems.getItemCount(); i++)
 					{
 						item = (EV_Menu_Item*)m_vecItems.getNthItem(i);
 						if (id==item->id)
 						{
 							strcpy (item->szText, szLabelName);					
 							//UT_DEBUGMSG(("Menu changing text->%s\n",szLabelName));
+							if (UT_IsWinVista()) 
+							{
+								mif.fState = uCheck | uEnable | uBold;
+								mif.fType = MFT_STRING;
+								mif.dwTypeData = (LPTSTR)szLabelName;
+								SetMenuItemInfo (hMenuBar,cmd,FALSE,&mif);
+							}
 							break;
 						}
 					}
@@ -568,7 +581,13 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 					strcpy (item->szText, szLabelName);					
 					m_vecItems.addItem(item);
 					//UT_DEBUGMSG(("Menu adding menu->%s\n",szLabelName));
-					AppendMenu(m, MF_OWNERDRAW,cmd, (const char*) item);					
+										
+					if (UT_IsWinVista()) {
+						AppendMenu(m, MF_STRING, cmd, szLabelName);
+						_setBitmapforID(m, id, cmd);
+					}
+					else
+						AppendMenu(m, MF_OWNERDRAW,cmd, (const char*) item);
 				}
 				
 				EV_Menu_ItemState mis = pAction->getMenuItemState(pView);
@@ -621,7 +640,7 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 		}
 	}
 
-#ifdef UT_DEBUG
+#ifdef DEBUG
 	HMENU wDbg = NULL;
 	bResult = stackMenu.pop((void **)&wDbg);
 	UT_ASSERT(bResult);
@@ -634,7 +653,7 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 /*
 	Caller should DeleteObject the handle returned	
 */
-HBITMAP EV_Win32Menu::_loadBitmap(XAP_Menu_Id id, int x, int y, UT_RGBColor color)
+HBITMAP EV_Win32Menu::_loadBitmap(XAP_Menu_Id id, int width, int height, UT_RGBColor color)
 {
 	
 	HBITMAP hBitmap = NULL;
@@ -648,11 +667,24 @@ HBITMAP EV_Win32Menu::_loadBitmap(XAP_Menu_Id id, int x, int y, UT_RGBColor colo
 
 	if (!pBitmaps->id) return hBitmap;	
 
-	AP_Win32Toolbar_Icons::getBitmapForIcon(GetDesktopWindow(), x,y, &color, pBitmaps->szName,	&hBitmap);					
+	AP_Win32Toolbar_Icons::getBitmapForIcon(GetDesktopWindow(), width, height, &color, pBitmaps->szName,	&hBitmap);					
 	
 	return hBitmap; 
 }
 
+// Sets the Bitmap in Windows Vista
+void EV_Win32Menu::_setBitmapforID (HMENU hMenu, XAP_Menu_Id id, UINT cmd)
+{
+	DWORD dwColor = GetSysColor (COLOR_MENU);
+	UT_RGBColor Color(GetRValue(dwColor),GetGValue(dwColor),GetBValue(dwColor));
+	HBITMAP hBitmap =  EV_Win32Menu::_loadBitmap(id, BITMAP_WITDH, BITMAP_HEIGHT, Color);
+							
+	if (hBitmap != NULL) {
+		SetMenuItemBitmaps (hMenu, cmd, MF_BYCOMMAND, hBitmap, hBitmap);		
+		m_vechBitmaps.push_back (hBitmap);
+	}
+}
+
 /*
 	It tells us which are menu bars
 */
@@ -661,7 +693,7 @@ bool EV_Win32Menu::_isAMenuBar(XAP_Menu_Id id, HMENU hMenu)
 	XAP_Menu_Id ids[] = { AP_MENU_ID_FILE, AP_MENU_ID_EDIT, AP_MENU_ID_VIEW,AP_MENU_ID_INSERT,
 		AP_MENU_ID_TOOLS, AP_MENU_ID_WINDOW, AP_MENU_ID_HELP,AP_MENU_ID_TABLE, AP_MENU_ID_FORMAT, NULL};
 
-	for (int i=0; i<(sizeof(ids)/sizeof(XAP_Menu_Id)); i++)
+	for (UT_uint32 i=0; i<(sizeof(ids)/sizeof(XAP_Menu_Id)); i++)
 	{
 		if (ids[i]==id)
 		{		
@@ -690,7 +722,7 @@ bool EV_Win32Menu::_isAMenuBar(XAP_Menu_Id id, HMENU hMenu)
 /*
 	Process message WM_MEASUREITEM
 */
-void EV_Win32Menu::onMeasureItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
+void EV_Win32Menu::onMeasureItem(HWND hwnd, WPARAM /*wParam*/, LPARAM lParam)
 {
 	SIZE size;
 	LPMEASUREITEMSTRUCT lpmis = (LPMEASUREITEMSTRUCT) lParam; 
@@ -716,7 +748,7 @@ void EV_Win32Menu::onMeasureItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 /*
 	Process message  WM_MENUCHAR. Process the hotkey messages	
 */
-LPARAM EV_Win32Menu::onMenuChar(HWND hwnd, WPARAM wParam, LPARAM lParam)
+LPARAM EV_Win32Menu::onMenuChar(HWND /*hwnd*/, WPARAM wParam, LPARAM lParam)
 {	
 	HMENU hMenu = (HMENU) lParam;
 	MENUITEMINFO	menuInfo;
@@ -759,7 +791,7 @@ LPARAM EV_Win32Menu::onMenuChar(HWND hwnd, WPARAM wParam, LPARAM lParam)
 /*
 	Process message  WM_DRAWITEM
 */
-void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
+void EV_Win32Menu::onDrawItem(HWND /*hwnd*/, WPARAM /*wParam*/, LPARAM lParam)
 {
 
 	LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT) lParam;  
@@ -767,7 +799,6 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
     COLORREF crText;    
     COLORREF crBkgnd;   
 	DWORD dwColor;
-	int x = 255, y=255;									
 	RECT rect;
 	UT_String sTextRight, sTextLeft;
 	HBITMAP hBitmap;	
@@ -806,7 +837,7 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	dwColor = GetSysColor(colorID);
 	
 	UT_RGBColor Color(GetRValue(dwColor),GetGValue(dwColor),GetBValue(dwColor));
-	hBitmap =  EV_Win32Menu::_loadBitmap(item->id,  x, y, Color);			
+	hBitmap =  EV_Win32Menu::_loadBitmap(item->id, BITMAP_WITDH, BITMAP_HEIGHT, Color);
 
 	if (hBitmap)	
 	{
@@ -883,9 +914,9 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		SelectObject(hdcMem,(void *)hBitmap);				
 
 		if(m_iDIR)
-			BitBlt(lpdis->hDC, lpdis->rcItem.right - 20, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
+			BitBlt(lpdis->hDC, lpdis->rcItem.right - 20, lpdis->rcItem.top+1, BITMAP_WITDH, BITMAP_HEIGHT, hdcMem, 0, 0, SRCCOPY );
 		else
-			BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
+			BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, BITMAP_WITDH, BITMAP_HEIGHT, hdcMem, 0, 0, SRCCOPY );
 		
 		DeleteDC(hdcMem);					
 				
@@ -922,8 +953,8 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 }
 
 								
-bool EV_Win32Menu::onMenuSelect(XAP_Frame * pFrame, AV_View * pView,
-								   HWND hWnd, HMENU hMenu, WPARAM wParam)
+bool EV_Win32Menu::onMenuSelect(XAP_Frame * pFrame, AV_View * /*pView*/,
+								   HWND /*hWnd*/, HMENU hMenu, WPARAM wParam)
 {
 	UINT nItemID = (UINT)LOWORD(wParam);
 	UINT nFlags  = (UINT)HIWORD(wParam);
