@@ -50,6 +50,7 @@ class ABI_EXPORT GR_UnixCairoGraphicsBase
 
 	virtual GR_Image*	createNewImage(const char* pszName,
 									   const UT_ByteBuf* pBB,
+		                               const std::string& mimetype,
 									   UT_sint32 iDisplayWidth,
 									   UT_sint32 iDisplayHeight,
 									   GR_Image::GRType =GR_Image::GRT_Raster);
@@ -72,6 +73,8 @@ public:
 	static GR_Graphics *   graphicsAllocator(GR_AllocInfo&);
 	GdkWindow *  getWindow () {return m_pWin;}
 
+	virtual GR_Font * getGUIFont(void);
+	
 	virtual void		setCursor(GR_Graphics::Cursor c);
 	void                createPixmapFromXPM(char ** pXPM,GdkPixmap *source,
 											GdkBitmap * mask);
