@@ -1,6 +1,7 @@
-/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*- */
 /* AbiWord
  * Copyright (C) 2004-2007 Tomas Frydrych
+ * Copyright (C) 2009 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,12 +23,13 @@
 #include "config.h"
 #endif
 
+#include <string>
+
 #include "gr_CairoGraphics.h"
 #include "gr_Painter.h"
 
 #include "xap_App.h"
 #include "xap_Prefs.h"
-#include "xap_EncodingManager.h"
 #include "xap_Strings.h"
 #include "xap_Frame.h"
 
@@ -35,12 +37,14 @@
 #include "ut_misc.h"
 #include "ut_vector.h"
 #include "ut_locale.h"
+#include "ut_std_string.h"
 
 // need this to include what Pango considers 'low-level' api
 #define PANGO_ENABLE_ENGINE
 
 #include <pango/pango-item.h>
 #include <pango/pango-engine.h>
+#include <pango/pangocairo.h>
 
 #include <math.h>
 
@@ -57,6 +61,101 @@ UT_VersionInfo GR_CairoGraphics::s_Version;
 int            GR_CairoGraphics::s_iMaxScript = 0;
 
 
+
+/*!
+ * The idea is to create a
+ * new image from the rectangular segment in device units defined by 
+ * UT_Rect rec. The Image should be deleted by the calling routine.
+ */
+GR_Image * GR_CairoRasterImage::createImageSegment(GR_Graphics * pG,const UT_Rect & rec)
+{
+	UT_sint32 x = pG->tdu(rec.left);
+	UT_sint32 y = pG->tdu(rec.top);
+	if(x < 0)
+	{
+		x = 0;
+	}
+	if(y < 0)
+	{
+		y = 0;
+	}
+	UT_sint32 width = pG->tdu(rec.width);
+	UT_sint32 height = pG->tdu(rec.height);
+	UT_sint32 dH = getDisplayHeight();
+	UT_sint32 dW = getDisplayWidth();
+	if(height > dH)
+	{
+		height = dH;
+	}
+	if(width > dW)
+	{
+		width = dW;
+	}
+	if(x + width > dW)
+	{
+		width = dW - x;
+	}
+	if(y + height > dH)
+	{
+		height = dH - y;
+	}
+	if(width <= 0)
+	{
+	        x = dW -1;
+		width = 1;
+	}
+	if(height <= 0)
+	{
+		y = dH -1;
+		height = 1;
+	}
+	std::string sName("");
+	getName(sName);
+	sName += UT_std_string_sprintf("_segemnt_%d_%d_%d_%d",x,y,width,height);
+
+	GR_CairoRasterImage * pImage = makeSubimage(sName, x, y, width, height);
+	if(pImage) 
+	{
+		pImage->setDisplaySize(width,height);
+	}
+	return pImage;
+}
+
+
+
+GR_CairoPatternImpl::GR_CairoPatternImpl(const char * fileName)
+	: m_pattern(NULL)
+{
+	cairo_surface_t * surface = cairo_image_surface_create_from_png(fileName);
+	m_pattern = cairo_pattern_create_for_surface(surface);
+	cairo_pattern_set_extend(m_pattern, CAIRO_EXTEND_REPEAT);
+	cairo_surface_destroy(surface);
+}
+
+GR_CairoPatternImpl::GR_CairoPatternImpl(cairo_surface_t * surf)
+	: m_pattern(cairo_pattern_create_for_surface(surf))
+{
+	cairo_pattern_set_extend(m_pattern, CAIRO_EXTEND_REPEAT);
+}
+
+GR_CairoPatternImpl::GR_CairoPatternImpl(const GR_CairoPatternImpl & p)
+	: UT_ColorPatImpl(p)
+	, m_pattern(cairo_pattern_reference(p.m_pattern))
+{
+}
+
+GR_CairoPatternImpl::~GR_CairoPatternImpl()
+{
+	cairo_pattern_destroy(m_pattern);
+}
+
+
+UT_ColorPatImpl * GR_CairoPatternImpl::clone() const
+{
+	return new GR_CairoPatternImpl(*this);
+}
+
+
 static void _pango_item_list_free(GList * items) 
 {
 	GList * l;
@@ -71,34 +170,34 @@ static void _pango_item_list_free(GList * items)
 
 
 
-class GR_UnixPangoItem: public GR_Item
+class GR_CairoPangoItem: public GR_Item
 {
 	friend class GR_CairoGraphics;
 	friend class GR_UnixPangoPrintGraphics;
 	friend class GR_PangoRenderInfo;
 	
   public:
-	virtual ~GR_UnixPangoItem(){ if (m_pi) {pango_item_free(m_pi);}};
+	virtual ~GR_CairoPangoItem(){ if (m_pi) {pango_item_free(m_pi);}};
 	
 	virtual GR_ScriptType getType() const {return (GR_ScriptType)m_iType;}
 	
 	virtual GR_Item *     makeCopy() const
 	    {
-			return new GR_UnixPangoItem(pango_item_copy(m_pi));
+			return new GR_CairoPangoItem(pango_item_copy(m_pi));
 		}
 	
-	virtual GRRI_Type     getClassId() const {return GRRI_UNIX_PANGO;}
+	virtual GRRI_Type     getClassId() const {return GRRI_CAIRO_PANGO;}
 
   protected:
-	GR_UnixPangoItem(PangoItem *pi);
-	GR_UnixPangoItem() : m_pi(NULL) { }; // just a dummy used to terminate
+	GR_CairoPangoItem(PangoItem *pi);
+	GR_CairoPangoItem() : m_pi(NULL) { }; // just a dummy used to terminate
 	                                     // GR_Itemization list
 
 	PangoItem *m_pi;
 	UT_uint32 m_iType;
 };
 
-GR_UnixPangoItem::GR_UnixPangoItem(PangoItem *pi):
+GR_CairoPangoItem::GR_CairoPangoItem(PangoItem *pi):
 	m_pi(pi)
 {
 	// there does not seem to be anything that we could use to identify the
@@ -156,7 +255,7 @@ class GR_PangoRenderInfo : public GR_RenderInfo
 		}
 	};
 
-	virtual GRRI_Type getType() const {return GRRI_UNIX_PANGO;}
+	virtual GRRI_Type getType() const {return GRRI_CAIRO_PANGO;}
 	virtual bool append(GR_RenderInfo &ri, bool bReverse = false);
 	virtual bool split (GR_RenderInfo *&pri, bool bReverse = false);
 	virtual bool cut(UT_uint32 offset, UT_uint32 iLen, bool bReverse = false);
@@ -214,7 +313,7 @@ bool GR_PangoRenderInfo::getUTF8Text()
 
 	UT_TextIterator & text = *m_pText;
 	sUTF8->clear();
-
+	sUTF8->reserve( text.getUpperLimit());
 	// we intentionally run this as far as the iterator lets us, even if that is
 	// past the end of this item
 	for(; text.getStatus() == UTIter_OK; ++text)
@@ -250,6 +349,13 @@ GR_CairoGraphics::GR_CairoGraphics(cairo_t *cr, UT_uint32 iDeviceResolution)
 	m_cr(cr),
 	m_cursor(GR_CURSOR_INVALID),
 	m_cs(GR_Graphics::GR_COLORSPACE_COLOR),
+	m_curColorDirty(false),
+	m_clipRectDirty(false),
+	m_lineWidth(1.0),
+	m_joinStyle(JOIN_MITER),
+	m_capStyle(CAP_BUTT),
+	m_lineStyle(LINE_SOLID),
+	m_linePropsDirty(false),
 	m_bIsSymbol(false),
 	m_bIsDingbat(false),
 	m_iPrevX1(0),
@@ -277,6 +383,13 @@ GR_CairoGraphics::GR_CairoGraphics()
 	m_cr(NULL),
 	m_cursor(GR_CURSOR_INVALID),
 	m_cs(GR_Graphics::GR_COLORSPACE_COLOR),
+	m_curColorDirty(false),
+	m_clipRectDirty(false),
+	m_lineWidth(1.0),
+	m_joinStyle(JOIN_MITER),
+	m_capStyle(CAP_BUTT),
+	m_lineStyle(LINE_SOLID),
+	m_linePropsDirty(false),
 	m_bIsSymbol(false),
 	m_bIsDingbat(false),
 	m_iPrevX1(0),
@@ -293,6 +406,7 @@ GR_CairoGraphics::GR_CairoGraphics()
 
 void GR_CairoGraphics::_initCairo()
 {
+	UT_ASSERT(m_cr);
 	cairo_translate(m_cr, 0.5, 0.5);
 	cairo_set_line_width (m_cr, 1);
 }
@@ -307,15 +421,16 @@ void GR_CairoGraphics::_initPango()
 	pango_cairo_font_map_set_resolution(PANGO_CAIRO_FONT_MAP(m_pLayoutFontMap), getResolution());	
 	m_pLayoutContext = pango_cairo_font_map_create_context(PANGO_CAIRO_FONT_MAP(m_pLayoutFontMap));
 
-	UT_DEBUGMSG(("Created LayoutFontMap %p Layout Context %p \n", m_pLayoutFontMap,	m_pLayoutContext));
-
+	UT_DEBUGMSG(("Created LayoutFontMap %p Layout Context %p resolution %d device resolution %d \n", 
+				 m_pLayoutFontMap,	m_pLayoutContext, getResolution(),
+				 m_iDeviceResolution));
 }
 
 GR_CairoGraphics::~GR_CairoGraphics()
 {
 	xxx_UT_DEBUGMSG(("Deleting UnixPangoGraphics %x \n",this));
-
-	cairo_destroy(m_cr), m_cr = NULL;
+	cairo_destroy(m_cr);
+	m_cr = NULL;
 
 	if(m_pAdjustedPangoFont!= NULL)
 	{
@@ -413,7 +528,7 @@ void GR_CairoGraphics::setColor3D(GR_Color3D c)
 {
 	UT_ASSERT(c < COUNT_3D_COLORS);
 
-	cairo_set_source_rgb(m_cr, m_3dColors[c].m_red/255., m_3dColors[c].m_grn/255., m_3dColors[c].m_blu/255.);
+	setColor(m_3dColors[c]);
 }
 
 bool GR_CairoGraphics::getColor3D(GR_Color3D name, UT_RGBColor &color)
@@ -512,7 +627,7 @@ bool GR_CairoGraphics::itemize(UT_TextIterator & text, GR_Itemization & I)
 	{
 		xxx_UT_DEBUGMSG(("itemize: creating item %d\n", i));
 		PangoItem *pItem = (PangoItem *)g_list_nth(gItems, i)->data;
-		GR_UnixPangoItem * pI = new GR_UnixPangoItem(pItem);
+		GR_CairoPangoItem * pI = new GR_CairoPangoItem(pItem);
 
 #if 0 //def DEBUG
 		PangoFont * pf = pI->m_pi->analysis.font;
@@ -532,7 +647,7 @@ bool GR_CairoGraphics::itemize(UT_TextIterator & text, GR_Itemization & I)
 		iOffset += pItem->num_chars;
 	}
 
-	I.addItem(iPosEnd - iPosStart + 1, new GR_UnixPangoItem());
+	I.addItem(iPosEnd - iPosStart + 1, new GR_CairoPangoItem());
 
 	g_list_free(gItems);
 	
@@ -553,20 +668,32 @@ GR_CairoGraphics::_calculateLogicalOffsets (PangoGlyphString * pGlyphs,
 	
 	int * pLogOffsets = new int [pGlyphs->num_glyphs];
  
-    // See http://www.abisource.com/mailinglists/abiword-dev/2006/Feb/0081.html
-    // for insight how this is supposeed to work and possible optimizations.
-
 	// In LTR text, the values in log_clusters are guaranteed to be increasing,
 	// in RTL text, the values in log_clusters are decreasing
 	
+	glong offset = 0;
+	const gchar *s = pUtf8;
+	
 	if (iVisDir == UT_BIDI_LTR ||
 		(pGlyphs->num_glyphs > 1 &&
 		 pGlyphs->log_clusters[0] < pGlyphs->log_clusters[1]))
 	{
 		for(int i = 0; i < pGlyphs->num_glyphs; ++i)
 		{
+
 			int iOff = pGlyphs->log_clusters[i];
-			pLogOffsets[i] =  g_utf8_pointer_to_offset (pUtf8, pUtf8 + iOff);
+			
+		//      below is equivalent to 
+		//	pLogOffsets[i] =  g_utf8_pointer_to_offset (pUtf8, pUtf8 + iOff);
+		// 	but avoids quadratic behavior because we use s and offset from 
+		//      previous iteration
+
+    		        while (s <  pUtf8 + iOff)
+		        {
+				s = g_utf8_next_char (s);
+				offset++;
+		        }
+			pLogOffsets[i] =  offset;
 		}
 	}
 	else // GR_ShapingInfo.m_iVisDir == UT_BIDI_RTL)
@@ -574,7 +701,12 @@ GR_CairoGraphics::_calculateLogicalOffsets (PangoGlyphString * pGlyphs,
 		for(int i = pGlyphs->num_glyphs - 1; i >= 0; --i)
 		{
 			int iOff = pGlyphs->log_clusters[i];
-			pLogOffsets[i] =  g_utf8_pointer_to_offset (pUtf8, pUtf8 + iOff);
+  		        while (s <  pUtf8 + iOff)
+		        {
+				s = g_utf8_next_char (s);
+				offset++;
+		        }			
+			pLogOffsets[i] =  offset;
 		}
 	}
 
@@ -585,10 +717,10 @@ bool GR_CairoGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 {
 	xxx_UT_DEBUGMSG(("GR_CairoGraphics::shape, len %d\n", si.m_iLength));
 	UT_return_val_if_fail(si.m_pItem &&
-						  si.m_pItem->getClassId() == GRRI_UNIX_PANGO &&
+						  si.m_pItem->getClassId() == GRRI_CAIRO_PANGO &&
 						  si.m_pFont, false);
 	
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem *)si.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem *)si.m_pItem;
 
 	PangoFontset * pfs = NULL;
 	PangoFont    * pFontSubst = NULL;
@@ -602,7 +734,7 @@ bool GR_CairoGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 	}
 	else
 	{
-		UT_return_val_if_fail(ri->getType() == GRRI_UNIX_PANGO, false);
+		UT_return_val_if_fail(ri->getType() == GRRI_CAIRO_PANGO, false);
 	}
 
 	GR_PangoRenderInfo * RI = (GR_PangoRenderInfo *)ri;
@@ -637,6 +769,7 @@ bool GR_CairoGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 	}
 	
 	UT_UTF8String utf8;
+	utf8.reserve(si.m_iLength);
 	bool previousWasSpace = si.m_previousWasSpace;
 
 	UT_sint32 i;
@@ -843,9 +976,9 @@ bool GR_CairoGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 UT_sint32 GR_CairoGraphics::getTextWidth(GR_RenderInfo & ri)
 {
 	xxx_UT_DEBUGMSG(("GR_CairoGraphics::getTextWidth\n"));
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO, 0);
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO, 0);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem *)RI.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem *)RI.m_pItem;
 	
 	UT_return_val_if_fail( RI.m_pGlyphs && RI.m_pLogOffsets && pItem, 0 );
 	
@@ -986,7 +1119,7 @@ void GR_CairoGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 {
 	// the only thing we need to do here is to make sure that the glyph metrics
 	// are calculated to a correct zoom level.
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 
 	if(RI.m_iZoom != getZoomPercentage())
@@ -1112,10 +1245,11 @@ PangoFont *  GR_CairoGraphics::_adjustedLayoutPangoFont (GR_PangoFont * pFont, P
 */
 void GR_CairoGraphics::renderChars(GR_RenderInfo & ri)
 {
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
+	UT_ASSERT(m_cr);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 	GR_PangoFont * pFont = (GR_PangoFont *)RI.m_pFont;
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem *)RI.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem *)RI.m_pItem;
 	UT_return_if_fail(pItem && pFont && pFont->getPangoFont());
 	xxx_UT_DEBUGMSG(("GR_CairoGraphics::renderChars length %d \n",
 					 RI.m_iLength));
@@ -1123,6 +1257,7 @@ void GR_CairoGraphics::renderChars(GR_RenderInfo & ri)
 	if(RI.m_iLength == 0)
 		return;
 
+	_setProps();
 	//
 	// Actually want the zoomed device font here
 	//
@@ -1214,7 +1349,7 @@ void GR_CairoGraphics::renderChars(GR_RenderInfo & ri)
 
 		// both of these can be 0 (iGlyphsEnd == 0 => only 1 glyph)
 		//	UT_return_if_fail( iGlyphsStart >= 0 && iGlyphsEnd >= 0 );
-		UT_DEBUGMSG(("Drawing glyph subset from %d to %d (offsets %d, %d)\n",
+		xxx_UT_DEBUGMSG(("Drawing glyph subset from %d to %d (offsets %d, %d)\n",
 					 iGlyphsStart, iGlyphsEnd,
 					 iOffsetStart, iOffsetEnd));
 		
@@ -1229,6 +1364,136 @@ void GR_CairoGraphics::renderChars(GR_RenderInfo & ri)
 	}
 }
 
+
+cairo_surface_t * GR_CairoGraphics::_getCairoSurfaceFromContext(cairo_t *cr, 
+                                                                     const cairo_rectangle_t & rect)
+{
+	cairo_surface_t * surface = cairo_surface_create_similar(cairo_get_target(cr), 
+	                                       CAIRO_CONTENT_COLOR_ALPHA, 
+	                                       rect.width, rect.height);
+
+	cairo_surface_t * source = cairo_get_target(cr);
+	cairo_surface_flush(source);
+
+	cairo_t * dest = cairo_create(surface);
+	cairo_set_source_surface(dest, source, rect.x, rect.y);
+	cairo_paint(dest);
+	cairo_destroy(dest);
+	return surface;
+}
+
+
+void GR_CairoGraphics::_setSource(cairo_t *cr, const UT_RGBColor &clr)
+{
+	const GR_CairoPatternImpl * pat 
+		= NULL;//dynamic_cast<const GR_CairoPatternImpl *>(clr.pattern());
+	if(pat) {
+		cairo_set_source(cr, pat->getPattern());
+	}
+	else {
+		cairo_set_source_rgb(cr, clr.m_red/255., clr.m_grn/255., clr.m_blu/255.);
+	}
+}
+
+
+static cairo_line_cap_t mapCapStyle(GR_Graphics::CapStyle in)
+{
+	switch (in) {
+	case GR_Graphics::CAP_ROUND:
+		return CAIRO_LINE_CAP_ROUND;
+	case GR_Graphics::CAP_PROJECTING:
+		return CAIRO_LINE_CAP_SQUARE;
+	case GR_Graphics::CAP_BUTT:
+	default:
+		return CAIRO_LINE_CAP_BUTT;
+    }
+}
+
+/*
+ * \param width line width
+ * \param dashes Array for dash pattern
+ * \param n_dashes IN: length of dashes, OUT: number of needed dash pattern fields
+ */
+static void mapDashStyle(GR_Graphics::LineStyle in, double width, double *dashes, int *n_dashes)
+{
+	switch (in) {
+	case GR_Graphics::LINE_ON_OFF_DASH:
+	case GR_Graphics::LINE_DOUBLE_DASH:				// see GDK_LINE_DOUBLE_DASH, but it was never used
+		UT_ASSERT(*n_dashes > 0);
+		dashes[0] = 4 * width;
+		*n_dashes = 1;
+		break;
+	case GR_Graphics::LINE_DOTTED:
+		UT_ASSERT(*n_dashes > 0);
+		dashes[0] = width;
+		*n_dashes = 1;
+		break;
+	case GR_Graphics::LINE_SOLID:
+	default:
+		*n_dashes = 0;
+	}
+}
+
+static cairo_line_join_t mapJoinStyle(GR_Graphics::JoinStyle in)
+{
+	switch ( in ) {
+	case GR_Graphics::JOIN_ROUND:
+		return CAIRO_LINE_JOIN_ROUND;
+	case GR_Graphics::JOIN_BEVEL:
+		return CAIRO_LINE_JOIN_BEVEL;
+	case GR_Graphics::JOIN_MITER:
+	default:
+		return CAIRO_LINE_JOIN_MITER;
+    }
+}
+
+
+void GR_CairoGraphics::_setProps()
+{
+	UT_ASSERT(m_cr);
+
+	if(m_curColorDirty) 
+	{
+		_setSource(m_cr, m_curColor);
+
+		m_curColorDirty = false;
+	}
+	if(m_clipRectDirty)
+	{
+		cairo_reset_clip(m_cr);
+		if (m_pRect)
+		{
+			double x, y, width, height;
+			x = _tdudX(m_pRect->left);
+			y = _tdudY(m_pRect->top);
+			width = _tduR(m_pRect->width);
+			height = _tduR(m_pRect->height);
+			cairo_rectangle(m_cr, x, y, width, height);
+			cairo_clip(m_cr);
+		}
+		m_clipRectDirty = false;
+	}
+	if(m_linePropsDirty)
+	{
+		double dashes[2];
+		double width;
+		int n_dashes;
+		width = tduD(m_lineWidth);
+		if(width < 1.0)
+			width = 1.0;
+		cairo_set_line_width (m_cr, width);
+		cairo_set_line_join (m_cr, mapJoinStyle(m_joinStyle));
+		cairo_set_line_cap (m_cr, mapCapStyle(m_capStyle));
+
+		width = cairo_get_line_width(m_cr);
+		n_dashes = G_N_ELEMENTS(dashes);
+		mapDashStyle(m_lineStyle, width, dashes, &n_dashes);
+		cairo_set_dash(m_cr, dashes, n_dashes, 0);
+
+		m_linePropsDirty = false;
+	}
+}
+
 void GR_CairoGraphics::_scaleCharacterMetrics(GR_PangoRenderInfo & RI)
 {
 	UT_uint32 iZoom = getZoomPercentage();
@@ -1267,7 +1532,7 @@ void GR_CairoGraphics::_scaleJustification(GR_PangoRenderInfo & RI)
 */
 void GR_CairoGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 {
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 
 	_scaleCharacterMetrics(RI);
@@ -1291,7 +1556,7 @@ bool GR_CairoGraphics::_scriptBreak(GR_PangoRenderInfo &ri)
 {
 	UT_return_val_if_fail(ri.m_pText && ri.m_pGlyphs && ri.m_pItem, false);
 
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem*)ri.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem*)ri.m_pItem;
 
 	// fill the static buffer with UTF8 text
 	UT_return_val_if_fail(ri.getUTF8Text(), false);
@@ -1315,7 +1580,7 @@ bool GR_CairoGraphics::_scriptBreak(GR_PangoRenderInfo &ri)
 bool GR_CairoGraphics::canBreak(GR_RenderInfo & ri, UT_sint32 &iNext,
 									bool bAfter)
 {
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO &&
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO &&
 						  ri.m_iOffset < ri.m_iLength, false);
 	
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
@@ -1378,7 +1643,7 @@ bool GR_CairoGraphics::needsSpecialCaretPositioning(GR_RenderInfo &)
 UT_uint32 GR_CairoGraphics::adjustCaretPosition(GR_RenderInfo & ri,
 													bool bForward)
 {
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO, 0);
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO, 0);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 	
 	if(!RI.s_pLogAttrs || RI.s_pOwnerLogAttrs != &ri)
@@ -1401,7 +1666,7 @@ UT_uint32 GR_CairoGraphics::adjustCaretPosition(GR_RenderInfo & ri,
 
 void GR_CairoGraphics::adjustDeletePosition(GR_RenderInfo & ri)
 {
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 
 	if(ri.m_iOffset + ri.m_iLength >= (UT_sint32)RI.m_iCharCount)
@@ -1461,7 +1726,7 @@ void GR_CairoGraphics::adjustDeletePosition(GR_RenderInfo & ri)
 UT_sint32 GR_CairoGraphics::resetJustification(GR_RenderInfo & ri,
 												   bool bPermanent)
 {
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO, 0);
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO, 0);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 
 	if(!RI.m_pJustify)
@@ -1499,7 +1764,7 @@ UT_sint32 GR_CairoGraphics::resetJustification(GR_RenderInfo & ri,
 
 UT_sint32 GR_CairoGraphics::countJustificationPoints(const GR_RenderInfo & ri) const
 {
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO, 0);
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO, 0);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
 
 	UT_return_val_if_fail(RI.m_pText, 0);
@@ -1551,7 +1816,7 @@ These are determined in fp_TextRun using calculations in layout units
  */
 void GR_CairoGraphics::justify(GR_RenderInfo & ri)
 {
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &) ri;
 	if(!RI.m_iJustificationPoints || !RI.m_iJustificationAmount ||
 	   !RI.m_pGlyphs)
@@ -1703,9 +1968,9 @@ void GR_CairoGraphics::justify(GR_RenderInfo & ri)
 UT_uint32 GR_CairoGraphics::XYToPosition(const GR_RenderInfo & ri, UT_sint32 x, 
 											 UT_sint32 /*y*/) const
 {
-	UT_return_val_if_fail(ri.getType() == GRRI_UNIX_PANGO, 0);
+	UT_return_val_if_fail(ri.getType() == GRRI_CAIRO_PANGO, 0);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &) ri;
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem *)RI.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem *)RI.m_pItem;
 	UT_return_val_if_fail(pItem, 0);
 
 	// TODO: this is very inefficient: to cache or not to cache ?
@@ -1769,9 +2034,9 @@ void GR_CairoGraphics::positionToXY(const GR_RenderInfo & ri,
 										UT_sint32& x2, UT_sint32& /*y2*/,
 										UT_sint32& /*height*/, bool& /*bDirection*/) const
 {
-	UT_return_if_fail(ri.getType() == GRRI_UNIX_PANGO);
+	UT_return_if_fail(ri.getType() == GRRI_CAIRO_PANGO);
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &) ri;
-	GR_UnixPangoItem * pItem = (GR_UnixPangoItem *)RI.m_pItem;
+	GR_CairoPangoItem * pItem = (GR_CairoPangoItem *)RI.m_pItem;
   
 	if(!pItem)
 		return;
@@ -1815,7 +2080,7 @@ void GR_CairoGraphics::positionToXY(const GR_RenderInfo & ri,
 		// withing range of our string
 		pOffset = g_utf8_offset_to_pointer (pUtf8, RI.m_iOffset);
 	}
-	else if(i > 1)
+	else if(i >= 1)
 	{
 		// this is the case where the requested offset is past the end
 		// of our string; we will use the last char; as we have more than one
@@ -1852,6 +2117,7 @@ void GR_CairoGraphics::drawChars(const UT_UCSChar* pChars,
 									UT_sint32 xoff, UT_sint32 yoff,
 									 int * pCharWidth)
 {
+	_setProps();
 	UT_UTF8String utf8;
 	xxx_UT_DEBUGMSG(("isDingBat %d \n",isDingbat()));
 	if(isSymbol())
@@ -2101,6 +2367,7 @@ cleanup:
 void GR_CairoGraphics::drawImage(GR_Image* pImg,
 									 UT_sint32 xDest, UT_sint32 yDest)
 {
+	_setProps();
 	UT_ASSERT(pImg);
 
    	UT_sint32 iImageWidth = pImg->getDisplayWidth();
@@ -2157,8 +2424,8 @@ void GR_CairoGraphics::setFont(const GR_Font * pFont)
 		if(strstr(szLCFontName,"dingbat"))
 			_setIsDingbat(true);
 	}
-	g_free (szLCFontName); szLCFontName = NULL;
-	
+	FREEP(szLCFontName);
+		
 	if(!m_pPFont->isGuiFont() && m_pPFont->getZoom() != getZoomPercentage())
 	{
 		m_pPFont->reloadFont(this);
@@ -2219,7 +2486,6 @@ UT_uint32 GR_CairoGraphics::getFontHeight(const GR_Font *pFont)
 	UT_return_val_if_fail( pFont, 0 );
 
 	const GR_PangoFont * pFP = static_cast<const GR_PangoFont*>(pFont);
-	xxx_UT_DEBUGMSG(("Font Height Pango %d \n",pFP->getAscent() + pFP->getDescent()));
 	return pFP->getAscent() + pFP->getDescent();
 }
 
@@ -2352,7 +2618,7 @@ GR_Font* GR_CairoGraphics::_findFont(const char* pszFontFamily,
 										 const char* pszLang)
 {
 	double dPointSize = UT_convertToPoints(pszFontSize);
-	UT_String s;
+	std::string s;
 
 	// Pango is picky about the string we pass to it -- it cannot handle any
 	// 'normal' values, and it will stop parsing when it encounters one.
@@ -2361,10 +2627,14 @@ GR_Font* GR_CairoGraphics::_findFont(const char* pszFontFamily,
 	const char * pWeight = pszFontWeight;
 	const char * pStretch = pszFontStretch;
 
-	if(pszFontFamily && !strcmp(pszFontFamily, "Symbol"))
-	{
-		pszFontFamily = "Standard Symbols L";
-	}
+//  This substitution does not seem to be need on my Linux system
+//  and acutally cause problem on Mac with Cairo as it never finds the
+//  font "Standards Symbols L" -- hub
+//
+//	if(pszFontFamily && !strcmp(pszFontFamily, "Symbol"))
+//	{
+//		pszFontFamily = "Standard Symbols L";
+//	}
 	   
 	if(pszFontStyle && *pszFontStyle == 'n')
 		pStyle = "";
@@ -2389,7 +2659,7 @@ GR_Font* GR_CairoGraphics::_findFont(const char* pszFontFamily,
 	if(!pszLang || !*pszLang)
 		pszLang = "en-US";
 	
-	UT_String_sprintf(s, "%s, %s %s %s %s",
+	s = UT_std_string_sprintf("%s, %s %s %s %s",
 					  pszFontFamily,
 					  pStyle,
 					  pVariant,
@@ -2507,31 +2777,56 @@ const std::vector<std::string> & GR_CairoGraphics::getAllFontNames(void)
 	PangoFontMap *fontmap = pango_cairo_font_map_get_default();
 	PangoContext *context = pango_cairo_font_map_create_context(PANGO_CAIRO_FONT_MAP(fontmap));	
 	if (fontmap && context)
-		{
-			PangoFontFamily **font_families;
-			int n_families;
-			pango_font_map_list_families(fontmap, &font_families, &n_families);
+	{
+		PangoFontFamily **font_families;
+		int n_families;
+		pango_font_map_list_families(fontmap, &font_families, &n_families);
 			
-			for(UT_sint32 i = 0; i < n_families; ++i)
-				{
-					const char *family = pango_font_family_get_name(font_families[i]);
-					
-					if (bExclude)
-						{
-							XAP_FontSettings & Fonts = pPrefs->getFontSettings();
-							if (Fonts.isOnExcludeList(family))
-								{
-									UT_DEBUGMSG(("@@@@ ===== Excluding font [%s] =====\n",
-												 family));
-									continue;
-								}
-						}
-					
-					Vec.push_back(family);
-				}
-			g_object_unref (G_OBJECT (context)), context = NULL;
-		}
+		for(UT_sint32 i = 0; i < n_families; ++i)
+		{
+			const char *family = pango_font_family_get_name(font_families[i]);
 
+			if (bExclude)
+			{
+				XAP_FontSettings & Fonts = pPrefs->getFontSettings();
+				if (Fonts.isOnExcludeList(family))
+				{
+					UT_DEBUGMSG(("@@@@ ===== Excluding font [%s] =====\n",
+								 family));
+					continue;
+				}
+			}
+			PangoFontFace ** faces;
+			int n_faces;
+			bool is_scalable = true;
+			pango_font_family_list_faces(font_families[i], &faces, &n_faces);
+			for(int j = 0; j < n_faces; j++) 
+			{
+				int * sizes = NULL;
+				int n_sizes = 0;
+				pango_font_face_list_sizes(faces[j], &sizes, &n_sizes);
+				if(sizes) 
+				{
+					g_free(sizes);
+					is_scalable = false;
+					UT_DEBUGMSG(("@@@@ ===== Excluding NON scalable font [%s] =====\n",
+								 family));
+					break;
+				}
+			}
+			g_free(faces);
+			if(is_scalable) 
+			{
+				Vec.push_back(family);
+			}
+		}
+		g_free(font_families);
+	}
+	if(context)
+	{
+		g_object_unref (G_OBJECT (context));
+		context = NULL;
+	}
     std::sort(Vec.begin(), Vec.end());
 
 	return Vec;
@@ -2603,98 +2898,44 @@ void GR_CairoGraphics::getColor(UT_RGBColor& clr)
 void GR_CairoGraphics::setColor(const UT_RGBColor& clr)
 {
 	m_curColor = clr;
-	cairo_set_source_rgb(m_cr, clr.m_red/255., clr.m_grn/255., clr.m_blu/255.);
+	m_curColorDirty = true;
 }
 
 void GR_CairoGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
 							   UT_sint32 x2, UT_sint32 y2)
 {
+	_setProps();
+
 	UT_sint32 idx1 = _tduX(x1);
 	UT_sint32 idx2 = _tduX(x2);
 
 	UT_sint32 idy1 = _tduY(y1);
 	UT_sint32 idy2 = _tduY(y2);
-	
-	cairo_move_to (m_cr, idx1, idy1);
-	cairo_line_to (m_cr, idx2, idy2);
+	cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+	cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
+	cairo_move_to (m_cr,idx1, idy1);
+	cairo_line_to (m_cr,idx2, idy2);
 	cairo_stroke (m_cr);
+	cairo_set_antialias(m_cr,prevAntiAlias);
 }
 
 void GR_CairoGraphics::setLineWidth(UT_sint32 iLineWidth)
 {
-	double width = tduD(iLineWidth);
-
-	cairo_set_line_width (m_cr, width);
+	m_lineWidth = iLineWidth;
+	m_linePropsDirty = true;
 }
 
-static cairo_line_cap_t mapCapStyle(GR_Graphics::CapStyle in)
-{
-	switch (in) {
-	case GR_Graphics::CAP_ROUND:
-		return CAIRO_LINE_CAP_ROUND;
-	case GR_Graphics::CAP_PROJECTING:
-		return CAIRO_LINE_CAP_SQUARE;
-	case GR_Graphics::CAP_BUTT:
-	default:
-		return CAIRO_LINE_CAP_BUTT;
-    }
-}
-
-/*
- * \param width line width
- * \param dashes Array for dash pattern
- * \param n_dashes IN: length of dashes, OUT: number of needed dash pattern fields
- */
-static void mapDashStyle(GR_Graphics::LineStyle in, double width, double *dashes, int *n_dashes)
-{
-	switch (in) {
-	case GR_Graphics::LINE_ON_OFF_DASH:
-	case GR_Graphics::LINE_DOUBLE_DASH:				// see GDK_LINE_DOUBLE_DASH, but it was never used
-		UT_ASSERT(*n_dashes > 0);
-		dashes[0] = 4 * width;
-		*n_dashes = 1;
-		break;
-	case GR_Graphics::LINE_DOTTED:
-		UT_ASSERT(*n_dashes > 0);
-		dashes[0] = width;
-		*n_dashes = 1;
-		break;
-	case GR_Graphics::LINE_SOLID:
-	default:
-		*n_dashes = 0;
-	}
-}
-
-static cairo_line_join_t mapJoinStyle(GR_Graphics::JoinStyle in)
-{
-	switch ( in ) {
-	case GR_Graphics::JOIN_ROUND:
-		return CAIRO_LINE_JOIN_ROUND;
-	case GR_Graphics::JOIN_BEVEL:
-		return CAIRO_LINE_JOIN_BEVEL;
-	case GR_Graphics::JOIN_MITER:
-	default:
-		return CAIRO_LINE_JOIN_MITER;
-    }
-}
 
 void GR_CairoGraphics::setLineProperties ( double inWidth, 
 										  GR_Graphics::JoinStyle inJoinStyle,
 										  GR_Graphics::CapStyle inCapStyle,
 										  GR_Graphics::LineStyle inLineStyle )
 {
-	double dashes[2];
-	double width;
-	int n_dashes;
-
-	cairo_set_line_width (m_cr, tduD(inWidth));
-	cairo_set_line_join (m_cr, mapJoinStyle(inJoinStyle));
-	cairo_set_line_cap (m_cr, mapCapStyle(inCapStyle));
-
-	width = cairo_get_line_width(m_cr);
-	n_dashes = G_N_ELEMENTS(dashes);
-	mapDashStyle(inLineStyle, width, dashes, &n_dashes);
-	cairo_set_dash(m_cr, dashes, n_dashes, 0);
+	m_lineWidth = inWidth;
+	m_joinStyle = inJoinStyle;
+	m_capStyle = inCapStyle;
+	m_lineStyle = inLineStyle;
+	m_linePropsDirty = true;
 }
 
 /*
@@ -2706,6 +2947,8 @@ void GR_CairoGraphics::setLineProperties ( double inWidth,
 void GR_CairoGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
 							  UT_sint32 y2)
 {
+	_setProps();
+
 	UT_sint32 idx1 = _tduX(x1);
 	UT_sint32 idx2 = _tduX(x2);
 
@@ -2747,9 +2990,11 @@ void GR_CairoGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
 		}
 		r.left = tlu(idx1);
 		r.top = tlu(idy1);
-		r.width = tlu(idx2 - idx1 +1);
+		r.width = tlu(idx2 - idx1 +2);
 		r.height = tlu(idy2 - idy1+2);
 		saveRectangle(r,m_iPrevRect);
+		cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+		cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
 		cairo_save(m_cr);
 		cairo_set_source_rgb(m_cr, 0.0 , 0.0 , 0.0);
 
@@ -2757,14 +3002,19 @@ void GR_CairoGraphics::xorLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2,
 		cairo_line_to(m_cr, idx2, idy2);
 		cairo_stroke(m_cr);
 		cairo_restore(m_cr);
+		cairo_set_antialias(m_cr,prevAntiAlias);
 	}
 }
 
 void GR_CairoGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 {
+	_setProps();
+
 	UT_uint32 i;
 
 	UT_return_if_fail(nPoints > 1);
+	cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+	cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
 
 	i = 0;
 	cairo_move_to(m_cr, _tdudX(pts[i].x), _tdudY(pts[i].y));
@@ -2774,6 +3024,7 @@ void GR_CairoGraphics::polyLine(UT_Point * pts, UT_uint32 nPoints)
 		cairo_line_to(m_cr, _tdudX(pts[i].x), _tdudY(pts[i].y));
 	}
 	cairo_stroke(m_cr);
+	cairo_set_antialias(m_cr,prevAntiAlias);
 }
 
 void GR_CairoGraphics::invertRect(const UT_Rect* /* pRect */)
@@ -2807,60 +3058,23 @@ double GR_CairoGraphics::_tdudY(UT_sint32 layoutUnits) const
 void GR_CairoGraphics::setClipRect(const UT_Rect* pRect)
 {
 	m_pRect = pRect;
-	cairo_reset_clip(m_cr);
-	if (pRect)
-	{
-		double x, y, width, height;
-		x = _tdudX(pRect->left);
-		y = _tdudY(pRect->top);
-		width = _tduR(pRect->width);
-		height = _tduR(pRect->height);
-		cairo_rectangle(m_cr, x, y, width, height);
-		cairo_clip(m_cr);
-	}
+	m_clipRectDirty = true;
 }
 
 void GR_CairoGraphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 							   UT_sint32 w, UT_sint32 h)
 {
+	_setProps();
+	cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+	cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
 	cairo_save(m_cr);
 
-	cairo_set_source_rgb(m_cr, c.m_red/255., c.m_grn/255., c.m_blu/255.);
+	_setSource(m_cr, c);
 	cairo_rectangle(m_cr, _tdudX(x), _tdudY(y), _tduR(w), _tduR(h));
 	cairo_fill(m_cr);
 
 	cairo_restore(m_cr);
-}
-
-GR_Font * GR_CairoGraphics::getGUIFont(void)
-{
-	if (!m_pPFontGUI)
-	{
-		// get the font resource
-		GtkStyle *tempStyle = gtk_style_new();
-		const char *guiFontName = pango_font_description_get_family(tempStyle->font_desc);
-		if (!guiFontName)
-			guiFontName = "'Times New Roman'";
-
-		UT_UTF8String s = XAP_EncodingManager::get_instance()->getLanguageISOName();
-
-		const char * pCountry
-			= XAP_EncodingManager::get_instance()->getLanguageISOTerritory();
-		
-		if(pCountry)
-		{
-			s += "-";
-			s += pCountry;
-		}
-		
-		m_pPFontGUI = new GR_PangoFont(guiFontName, 11.0, this, s.utf8_str(), true);
-
-		g_object_unref(G_OBJECT(tempStyle));
-		
-		UT_ASSERT(m_pPFontGUI);
-	}
-
-	return m_pPFontGUI;
+	cairo_set_antialias(m_cr,prevAntiAlias);
 }
 
 /*!
@@ -2950,15 +3164,19 @@ void GR_CairoGraphics::fillRect(GR_Color3D c, UT_Rect &r)
  */
 void GR_CairoGraphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h)
 {
+	_setProps();
 	UT_ASSERT(m_bHave3DColors && c < COUNT_3D_COLORS);
+	cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+	cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
 
 	cairo_save (m_cr);
 
-	cairo_set_source_rgb(m_cr, m_3dColors[c].m_red/255., m_3dColors[c].m_grn/255., m_3dColors[c].m_blu/255.);
+	_setSource(m_cr, m_3dColors[c]);
 	cairo_rectangle(m_cr, tdu(x), tdu(y), tdu(w), tdu(h));
 	cairo_fill(m_cr);
 
 	cairo_restore (m_cr);
+	cairo_set_antialias(m_cr,prevAntiAlias);
 }
 
 /*!
@@ -2967,10 +3185,12 @@ void GR_CairoGraphics::fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint3
 void GR_CairoGraphics::polygon(UT_RGBColor& c, UT_Point *pts,
 								   UT_uint32 nPoints)
 {
+	_setProps();
 	UT_uint32 i;
 
 	UT_return_if_fail(nPoints > 1);
-
+	cairo_antialias_t prevAntiAlias = cairo_get_antialias(m_cr);
+	cairo_set_antialias(m_cr,CAIRO_ANTIALIAS_NONE);
 	cairo_save(m_cr);
 
 	i = 0;
@@ -2979,10 +3199,11 @@ void GR_CairoGraphics::polygon(UT_RGBColor& c, UT_Point *pts,
 	for (; i < nPoints; i++) {
 		cairo_line_to(m_cr, _tdudX(pts[i].x), _tdudY(pts[i].y));
 	}
-	cairo_set_source_rgb(m_cr, c.m_red/255., c.m_grn/255., c.m_blu/255.);
+	_setSource(m_cr, c);
 	cairo_fill(m_cr);	
 
 	cairo_restore(m_cr);
+	cairo_set_antialias(m_cr,prevAntiAlias);
 }
 
 void GR_CairoGraphics::clearArea(UT_sint32 x, UT_sint32 y,
@@ -3062,19 +3283,21 @@ void GR_PangoFont::reloadFont(GR_CairoGraphics * pG)
 		return;
 	
 	m_iZoom = iZoom;
+
+	UT_DEBUGMSG(("GR_PangoFont::reloadFont() zoom %% %d\n", iZoom));
 	
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
- 	UT_String sLay;
- 	UT_String sDev;
+ 	std::string sLay;
+ 	std::string sDev;
 	if(!m_bGuiFont && pG->queryProperties(GR_Graphics::DGP_SCREEN))
  	{
- 		UT_String_sprintf(sDev, "%s %f", m_sDesc.c_str(), m_dPointSize * (double)m_iZoom / 100.0);
- 		UT_String_sprintf(sLay, "%s %f", m_sLayoutDesc.c_str(), m_dPointSize);
+ 		sDev = UT_std_string_sprintf("%s %f", m_sDesc.c_str(), m_dPointSize * (double)m_iZoom / 100.0);
+ 		sLay = UT_std_string_sprintf("%s %f", m_sLayoutDesc.c_str(), m_dPointSize);
  	}
 	else
  	{
- 		UT_String_sprintf(sDev, "%s %f", m_sDesc.c_str(), m_dPointSize);
- 		UT_String_sprintf(sLay, "%s %f", m_sLayoutDesc.c_str(), m_dPointSize);
+ 		sDev = UT_std_string_sprintf("%s %f", m_sDesc.c_str(), m_dPointSize);
+ 		sLay = UT_std_string_sprintf("%s %f", m_sLayoutDesc.c_str(), m_dPointSize);
  	}		
   
  	if(m_pfdLay)
@@ -3114,6 +3337,8 @@ void GR_PangoFont::reloadFont(GR_CairoGraphics * pG)
 	// pango_metrics_ functions return in points * PANGO_SCALE (points * 1024)
  	m_iAscent = (UT_uint32) pango_font_metrics_get_ascent(pfm)/PANGO_SCALE;
  	m_iDescent = (UT_uint32) pango_font_metrics_get_descent(pfm)/PANGO_SCALE;
+	UT_DEBUGMSG(("metrics asc %d desc %d\n", m_iAscent, m_iDescent));
+
  	xxx_UT_DEBUGMSG(("Layout Font Ascent %d point size %f zoom %d \n",m_iAscent, m_dPointSize, m_iZoom));
 	pango_font_metrics_unref(pfm);
 
@@ -3260,8 +3485,8 @@ const char* GR_PangoFont::getFamily() const
 bool GR_PangoRenderInfo::canAppend(GR_RenderInfo &ri) const
 {
 	GR_PangoRenderInfo & RI = (GR_PangoRenderInfo &)ri;
-	GR_UnixPangoItem * pItem1 = (GR_UnixPangoItem *)m_pItem;
-	GR_UnixPangoItem * pItem2 = (GR_UnixPangoItem *)RI.m_pItem;
+	GR_CairoPangoItem * pItem1 = (GR_CairoPangoItem *)m_pItem;
+	GR_CairoPangoItem * pItem2 = (GR_CairoPangoItem *)RI.m_pItem;
 
 	/* Do not merger runs that have not been shapped yet */
 	if (!pItem1 || !pItem2)
