@@ -373,7 +373,7 @@ void GR_Graphics::endPaint ()
 
 UT_sint32 GR_Graphics::tdu(UT_sint32 layoutUnits) const
 {
-	double d = (static_cast<double>(layoutUnits) * static_cast<double>(getDeviceResolution()) * static_cast<double>(getZoomPercentage())) / (100. * static_cast<double>(getResolution())) + 0.1;
+	double d = (static_cast<double>(layoutUnits) * static_cast<double>(getDeviceResolution()) * static_cast<double>(getZoomPercentage())) * (1./100. / static_cast<double>(getResolution())) + 0.1;
 	return static_cast<UT_sint32>(d);
 }
 
@@ -600,7 +600,8 @@ void GR_Graphics::drawCharsRelativeToBaseline(const UT_UCSChar* pChars,
  * doesn't scale if the resolution or zoom changes. Instead you must create
  * a new image.
  */
-GR_Image* GR_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
+GR_Image* GR_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, const std::string& mimetype,
+									  UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
 {
    GR_VectorImage * vectorImage = NULL;
 
@@ -613,7 +614,7 @@ GR_Image* GR_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB
    }
 
    if (vectorImage) {
-      vectorImage->convertFromBuffer(pBB, iDisplayWidth, iDisplayHeight);
+      vectorImage->convertFromBuffer(pBB, mimetype, iDisplayWidth, iDisplayHeight);
    }
 
    return vectorImage;
@@ -866,6 +867,7 @@ void GR_Graphics::doRepaint( UT_Rect * rClip)
 */
 void GR_Graphics::fillRect(GR_Image * pImg, const UT_Rect & src, const UT_Rect & dest)
 {
+	UT_return_if_fail(pImg);
 	GR_Image * pImageSection = pImg->createImageSegment(this, src);
 	UT_return_if_fail(pImageSection);
 	drawImage(pImageSection,dest.left,dest.top);
@@ -1815,12 +1817,12 @@ bool GR_GraphicsFactory::isRegistered(UT_uint32 iClassId) const
 	return true;
 }
 
-#if defined(TOOLKIT_GTK)
-#include "gr_UnixNullGraphics.h"
+#if defined(WITH_CAIRO)
+#include "gr_CairoNullGraphics.h"
 #elif defined(TOOLKIT_WIN)
 #include "gr_Win32Graphics.h"
-#elif defined(TOOLKIT_COCOA)
-#warning implement offscreen
+#else
+#warning un-handled case
 #endif
 
 /**
@@ -1831,13 +1833,13 @@ GR_Graphics* GR_Graphics::newNullGraphics()
 {
 	// todo: support other platforms when possible
 	
-#if defined(TOOLKIT_GTK)
-	GR_UnixNullGraphicsAllocInfo ai;
-	return XAP_App::getApp()->newGraphics(GRID_UNIX_NULL, (GR_AllocInfo&)ai);
+#if defined(WITH_CAIRO)
+	GR_CairoNullGraphicsAllocInfo ai;
+	return XAP_App::getApp()->newGraphics(GRID_CAIRO_NULL, (GR_AllocInfo&)ai);
 #elif defined(TOOLKIT_WIN)
 	GR_Win32AllocInfo ai (GR_Win32Graphics::createbestmetafilehdc(), GR_Win32Graphics::getDocInfo(), NULL);
 	return XAP_App::getApp()->newGraphics(GRID_WIN32, (GR_AllocInfo&)ai);
-#elif defined(TOOLKIT_COCOA)
+#else
 #endif
 	
 	return NULL;
