@@ -75,6 +75,7 @@
 #include <fcntl.h>
 #endif
 
+#include <string>
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
@@ -1229,13 +1230,18 @@ UT_go_file_create_impl (char const *uri, GError **err)
 {
 	char *filename;
 	int fd;
-
 	g_return_val_if_fail (uri != NULL, NULL);
 
+	std::string path = uri;
+        bool is_uri = UT_go_path_is_uri(path.c_str());
+        bool is_filename = is_uri ? false : path.find_last_of(G_DIR_SEPARATOR) == std::string::npos;
+	bool is_path = !is_uri && !is_filename;
+
 	filename = UT_go_filename_from_uri (uri);
-	if (filename) {
-		GsfOutput *result = gsf_output_stdio_new (filename, err);
-		g_free (filename);
+	if (is_path || filename) {
+		GsfOutput *result = gsf_output_stdio_new (filename?filename:uri, err);
+		if(filename) 
+			g_free (filename);
 		return result;
 	}
 
