@@ -280,13 +280,13 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 				pPrefs->getPrefsValueBool(static_cast<const gchar *>(XAP_PREF_KEY_UseSuffix), &wantSuffix);
 				UT_DEBUGMSG(("UseSuffix: %d\n", wantSuffix));
 
-				if (nFileType > 0) // 0 means autodetect
+				if (nFileType > 0 && getDialogId() != XAP_DIALOG_ID_FILE_SAVE_IMAGE) // 0 means autodetect
 				{
-					if (UT_pathSuffix(szDialogFilename))
+					if (!UT_pathSuffix(szDialogFilename).empty())
 					{
 						// warn if we have a suffix that doesn't match the selected file type
 						IE_ExpSniffer* pSniffer = IE_Exp::snifferForFileType(m_nTypeList[nIndex]);
-						if (pSniffer && !pSniffer->recognizeSuffix(UT_pathSuffix(szDialogFilename)))
+						if (pSniffer && !pSniffer->recognizeSuffix(UT_pathSuffix(szDialogFilename).c_str()))
 						{
 							UT_UTF8String msg;
 							const XAP_StringSet * pSS = m_pApp->getStringSet();
@@ -537,6 +537,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 				break;
 			}
 		case XAP_DIALOG_ID_FILE_SAVEAS:
+		case XAP_DIALOG_ID_FILE_SAVE_IMAGE:
 			{
 				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_SaveAsTitle,szTitle);
 				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel,szFileTypeLabel);
@@ -652,7 +653,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	gtk_widget_show(pulldown_hbox);
 
 	// pulldown label
-	GtkWidget * filetypes_label = gtk_label_new(szFileTypeLabel.c_str());
+	GtkWidget * filetypes_label = gtk_label_new_with_mnemonic(convertMnemonics(szFileTypeLabel).c_str());
 	gtk_label_set_justify(GTK_LABEL(filetypes_label), GTK_JUSTIFY_RIGHT);
 	gtk_misc_set_alignment(GTK_MISC(filetypes_label), 1.0, 0.5);
 	gtk_widget_show(filetypes_label);
@@ -662,6 +663,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	filetypes_pulldown = gtk_combo_box_new();
 	gtk_widget_show(filetypes_pulldown);
 	gtk_box_pack_end(GTK_BOX(pulldown_hbox), filetypes_pulldown, TRUE, TRUE, 0);
+    gtk_label_set_mnemonic_widget(GTK_LABEL(filetypes_label), filetypes_pulldown);
 #endif
 	//
 	// add the filters to the dropdown list
@@ -703,7 +705,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	m_wFileTypes_PullDown = filetypes_pulldown;
 	// dialog; open dialog always does auto-detect
 	// TODO: should this also apply to the open dialog?
-	if (m_id == XAP_DIALOG_ID_FILE_SAVEAS)
+	if (m_id == XAP_DIALOG_ID_FILE_SAVEAS || m_id == XAP_DIALOG_ID_FILE_SAVE_IMAGE)
 	{
 		gtk_combo_box_set_active(combo, activeItemIndex + 1);
 	}
@@ -793,9 +795,9 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 #endif
 			if(m_id == XAP_DIALOG_ID_FILE_SAVEAS)
 			{
-				const char * szInitialSuffix = UT_pathSuffix(m_szInitialPathname);
+				std::string szInitialSuffix = UT_pathSuffix(m_szInitialPathname);
 				std::string szSaveTypeSuffix = IE_Exp::preferredSuffixForFileType(m_nDefaultFileType).utf8_str();
-				if(szInitialSuffix && !szSaveTypeSuffix.empty() 
+				if(!szInitialSuffix.empty() && !szSaveTypeSuffix.empty() 
 					&& (szSaveTypeSuffix != szInitialSuffix))
 				{
 					std::string sFileName = m_szInitialPathname;
