@@ -32,18 +32,18 @@
 #include "fl_FootnoteLayout.h"
 #include "fp_Page.h"
 
-fb_ColumnBreaker::fb_ColumnBreaker() :
+fb_ColumnBreaker::fb_ColumnBreaker(fl_DocSectionLayout * pDSL) :
 	m_pStartPage(NULL),
 	m_bStartFromStart(true),
 	m_bReBreak(false),
-	m_pDocSec(NULL),
+	m_pDocSec(pDSL),
 	m_pCurrentBlock(NULL)
 {
 }
 
 void fb_ColumnBreaker::setStartPage(fp_Page * pPage)
 {
-	if(!m_bStartFromStart)
+  //	if(!m_bStartFromStart)
 	{
 		if(m_pStartPage == NULL)
 		{
@@ -87,9 +87,9 @@ bool fb_ColumnBreaker::_displayAnnotations(void)
  * Returns NULL if no rebreak is required. 
  Otherwise returns a pointer to the page requiring the rebreak.
 */
-fp_Page * fb_ColumnBreaker::needsRebreak(fl_DocSectionLayout * pSL)
+fp_Page * fb_ColumnBreaker::needsRebreak(void)
 {
-    fl_ContainerLayout * pCL = pSL->getLastLayout();
+    fl_ContainerLayout * pCL = m_pDocSec->getLastLayout();
     fl_BlockLayout * pBL = NULL;
     if(pCL && (pCL->getContainerType() == FL_CONTAINER_BLOCK))
     {
@@ -101,25 +101,32 @@ fp_Page * fb_ColumnBreaker::needsRebreak(fl_DocSectionLayout * pSL)
     }
     else
     {
-        UT_ASSERT_HARMLESS(pCL);
-	return _getLastValidPage(pSL);
+	return _getLastValidPage();
     }
     if(pBL)
     {
         fp_Line * pLine = static_cast<fp_Line *>(pBL->getLastContainer());
 	if(pLine == NULL)
 	{
-	     return _getLastValidPage(pSL);
+	     return _getLastValidPage();
 	}
 	fp_Page * pPage = pLine->getPage();
 	if(pPage == NULL)
 	{
-	     return _getLastValidPage(pSL);
+	     return _getLastValidPage();
 	}
-	else if(pLine->getY() > pPage->getHeight())
+	else if(pLine->getY() >  m_pDocSec->getActualColumnHeight())
 	{
 	     return pPage;
 	}
+	else
+	{
+	    fp_Column * pCol = pPage->getNthColumnLeader(0);
+	    if(pCol->getHeight() >  m_pDocSec->getActualColumnHeight())
+	    {
+		return pPage;
+	    }
+	}
     }
     return NULL;
 }
@@ -139,34 +146,74 @@ fp_Page * fb_ColumnBreaker::needsRebreak(fl_DocSectionLayout * pSL)
  FIXME we should try to detect the error in the code rather than rebreak.
 */
 
-UT_sint32 fb_ColumnBreaker::breakSection(fl_DocSectionLayout * pSL)
+UT_sint32 fb_ColumnBreaker::breakSection()
 {
   fp_Page * pStartPage = m_pStartPage;
-  pSL->setNeedsSectionBreak(false,m_pStartPage);
-  UT_sint32 iVal = _breakSection(pSL, pStartPage);
+  m_pDocSec->setNeedsSectionBreak(false,m_pStartPage);
+  FL_DocLayout * pDL =  m_pDocSec->getDocLayout();
+  m_bStartFromStart = true;
+  UT_sint32 iPage = pDL->findPage(pStartPage);  
+  if(iPage < 0)
+       pStartPage = NULL;
+  UT_sint32 iVal = _breakSection(pStartPage);
   UT_sint32 icnt = 0;
-  pStartPage = needsRebreak(pSL);
-  while(pStartPage && (icnt < 10))
+  pStartPage = needsRebreak();
+  if(m_pStartPage)
   {
-      iVal = _breakSection(pSL,pStartPage);
-      pStartPage = needsRebreak(pSL);
+      icnt = 0;
+      pStartPage = m_pStartPage;
+  }
+  while(pStartPage && (icnt < 50))
+  {
+
+      UT_DEBUGMSG(("Try to get a decent break Column %d times \n",icnt));
+      iPage = pDL->findPage(pStartPage);
+      if(iPage < 0)
+      {
+	  pStartPage = NULL;
+      }
+      else
+      {
+	  UT_DEBUGMSG(("Retrying from page %d \n",iPage));
+	  if(icnt >15)
+	  {
+	      if(pStartPage->getAvailableHeight() <0)
+	      {
+		  while(pStartPage->countFootnoteContainers()>0)
+		  {
+		      fp_FootnoteContainer* pFC = pStartPage->getNthFootnoteContainer(0);
+		      pStartPage->removeFootnoteContainer(pFC);
+		  }
+	      }
+	  }
+      }
+
+      iVal = _breakSection(pStartPage);
+      pStartPage = needsRebreak();
+      if(m_pStartPage)
+      {
+  	  pStartPage = m_pStartPage;
+	  if(icnt > 10)
+	  {
+	      pStartPage = pStartPage->getPrev();
+	  }
+      }
       icnt++;
   }
-  FL_DocLayout * pDL = pSL->getDocLayout();
   pDL->deleteEmptyColumnsAndPages();
   return iVal;
 }
 
-fp_Page * fb_ColumnBreaker::_getLastValidPage(fl_DocSectionLayout * pSL)
+fp_Page * fb_ColumnBreaker::_getLastValidPage(void)
 {
   fp_Page * pPage = NULL;
   fp_Page * pFoundPage = NULL;
   UT_sint32 i = 0;
-  FL_DocLayout * pDL = pSL->getDocLayout();
+  FL_DocLayout * pDL = m_pDocSec->getDocLayout();
   for(i=0; i<pDL->countPages();i++)
   {
       pPage = pDL->getNthPage(i);
-      if(pPage->getOwningSection() == pSL)
+      if(pPage->getOwningSection() == m_pDocSec)
       {
 	  pFoundPage = pPage;
       }
@@ -177,23 +224,32 @@ fp_Page * fb_ColumnBreaker::_getLastValidPage(fl_DocSectionLayout * pSL)
 }
 
 
-UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * pStartPage)
+UT_sint32 fb_ColumnBreaker::_breakSection(fp_Page * pStartPage)
 {
 	m_bReBreak = false;
-	m_pDocSec = pSL;
+	m_pStartPage = NULL;
+	FL_DocLayout * pDocLayout = m_pDocSec->getDocLayout();
 	fl_ContainerLayout* pFirstLayout = NULL;
 	fp_Container* pOuterContainer = NULL;
 	fp_Column* pCurColumn = NULL;
 	fp_Column* pPrevColumn = NULL;
-	xxx_UT_DEBUGMSG(("Doing ColumnBreak for section %x at page %x \n",pSL,m_pStartPage));
-//	UT_ASSERT(pSL->needsSectionBreak());
-	pFirstLayout = pSL->getFirstLayout();
+	UT_sint32 iCurPage = 0;
+	UT_sint32 iColCons =  0;
+	xxx_UT_DEBUGMSG(("Doing ColumnBreak for section %x at page %x \n",m_pDecSec,m_pStartPage));
+//	UT_ASSERT(m_pDocSec->needsSectionBreak());
+	pFirstLayout = m_pDocSec->getFirstLayout();
 	if (!pFirstLayout)
 	{
 		m_pStartPage = NULL;
 		m_bStartFromStart = false;
 		return 0;
 	}
+	if(pStartPage)
+	{
+	    iCurPage =  pDocLayout->findPage(pStartPage);
+	    if(iCurPage == 0)
+	        pStartPage=NULL;
+	}
 	pOuterContainer = pFirstLayout->getFirstContainer();
 	if(pOuterContainer
 		&& pOuterContainer->getContainerType() == FP_CONTAINER_FRAME)
@@ -205,9 +261,9 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 		    }
 
 	}
-	pCurColumn = static_cast<fp_Column*>(pSL->getFirstContainer());
-	xxx_UT_DEBUGMSG(("fb_ColumnBreaker: For DocSec %x first column %x \n",pSL,pCurColumn));
-	if(pStartPage)
+	pCurColumn = static_cast<fp_Column*>(m_pDocSec->getFirstContainer());
+	xxx_UT_DEBUGMSG(("fb_ColumnBreaker: For DocSec %x first column %x \n",m_pDecSec,pCurColumn));
+	if(pStartPage && (pStartPage->getNthColumnLeader(0)))
 	{
 		xxx_UT_DEBUGMSG(("Layout from page %x \n",pStartPage));
 		pCurColumn = pStartPage->getNthColumnLeader(0);
@@ -237,8 +293,50 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 	if (pCurColumn) {
 		pPrevPage = pCurColumn->getPage();
 	}
+	xxx_UT_DEBUGMSG(("Starting ColumnBreak now pOuterContainer %p \n",pOuterContainer));
 	while (pCurColumn)
 	{
+	        if(  m_pStartPage != NULL)
+		{
+		  // Abort and start again
+		    break;
+		}
+		iColCons = -1;
+		if(pCurColumn && pCurColumn->getPrev())
+		{
+		    fp_Column * pPrevCol = static_cast<fp_Column *>(pCurColumn->getPrev());
+		    fp_Page * pPrevPrevPage = pPrevCol->getPage();
+		    if((pPrevCol->getHeight() == 0) && !pPrevCol->containsPageBreak())
+		    {
+		        bool bPageBreak = false;
+		        fp_Column * pCol2 = static_cast<fp_Column *>(pPrevCol->getPrev());
+			while(pCol2 && (pCol2->getPage() == pPrevPrevPage) && !pCol2->containsPageBreak())
+			{
+			    pCol2 = static_cast<fp_Column *>(pCol2->getPrev());
+			}
+			bPageBreak = (pCol2 && pCol2->containsPageBreak());
+		      // Abort and start again from this page
+			if(!bPageBreak)
+			{
+			    m_pStartPage = pPrevPrevPage;
+			    break;
+			}
+		    }
+		    if(pPrevCol->getHeight() >  m_pDocSec->getActualColumnHeight())
+		    {
+		      // Abort and start again from this page
+
+			m_pStartPage = pPrevPrevPage;
+			break;
+		    }
+		}
+	        xxx_UT_DEBUGMSG(("New column pOuterContainer %p \n",pOuterContainer));
+		bool bSaveset = false;
+		if(pOuterContainer)
+		{
+		        pDocLayout->setSaveContainerPointer(pOuterContainer); 
+			bSaveset = true;
+		}
 		if(pCurColumn->getPage() != pPrevPage)
 		{
 			if(pPrevPage)
@@ -247,8 +345,13 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 // check and update wrapped text around positioned objects
 //
 			        UT_sint32 iPage = m_pDocSec->getDocLayout()->findPage(pPrevPage);
-				m_pDocSec->getDocLayout()->setFramePageNumbers(iPage);
+				pDocLayout->setFramePageNumbers(iPage);
+				//pOuterContainer->setAllowDelete(false);
+				// 
+				// pOuterContainer can be deleted by this call
+				//
 				fp_Container * pNextContainer = pPrevPage->updatePageForWrapping(pCurColumn);
+				//pOuterContainer->setAllowDelete(true);
 				xxx_UT_DEBUGMSG(("Returned container updatePage %x \n",pNextContainer));
 				if(pNextContainer != NULL)
 				{
@@ -256,12 +359,28 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 				}
 			}
 		}
+		if((pDocLayout->getRebuiltBlock() == NULL) && bSaveset)
+		{
+		    pDocLayout->getSavedContainerPointer()->setAllowDelete(true);
+		}
+		else if(bSaveset)
+		{
+		  //
+		  // pOuterContainer has been deleted!
+		  //
+		    if(pDocLayout->getSavedContainerPointer() == pOuterContainer)
+		    {
+		        UT_DEBUGMSG(("AbiWord Saved from a dangling pointer in fb_ColumnBreak \n"));
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+			pOuterContainer =  pDocLayout->getRebuiltBlock()->getFirstContainer();
+		    }
+		}
 		pPrevPage = pCurColumn->getPage();
 		fp_Container* pFirstContainerToKeep = pOuterContainer;
 		xxx_UT_DEBUGMSG(("SEVIOR: first to keep 1 %x \n",pFirstContainerToKeep));
 		fp_Container* pLastContainerToKeep = NULL;
 		fp_Container* pOffendingContainer = NULL;
-		UT_sint32 iMaxSecCol = pSL->getMaxSectionColumnHeight();
+		UT_sint32 iMaxSecCol = m_pDocSec->getActualColumnHeight();
  		UT_sint32 iMaxColHeight = pCurColumn->getMaxHeight();
 		UT_sint32 iFootnoteHeight = 0;
 		bool bEquivColumnBreak = false;
@@ -284,18 +403,18 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			fp_Container* pPrevContainer = 
 				static_cast<fp_Container *>(pCurContainer->getPrevContainerInSection());
 			if(pPrevContainer && 
-			   pPrevContainer->getContainerType() == FP_CONTAINER_LINE)
+			   (pPrevContainer->getContainerType() == FP_CONTAINER_LINE))
 			{
 				fp_Line * pL = static_cast<fp_Line *>(pPrevContainer);
-				{
-					if (pL->containsForcedPageBreak()
+				
+				if (pL && pL->containsForcedPageBreak() && pL->getContainer() 
 				&& (pCurColumn->getPage() == pL->getContainer()->getPage()))
-					{
-						pCurColumn->validate();
-						pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
-						continue;
-					}
+				{
+					  //						pCurColumn->validate();
+				  pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
+				  continue;
 				}
+				
 			}
 		}
 		bool bBreakOnColumnBreak = false;
@@ -306,7 +425,12 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 		fp_Container * pPrevWorking = pCurContainer;
 		while (pCurContainer)
 		{
-			xxx_UT_DEBUGMSG(("curContainer %x type %d \n",pCurContainer,pCurContainer->getContainerType()));
+		        iColCons++;
+		        xxx_UT_DEBUGMSG(("curContainer pointer %p type %s \n",iLoop,pCurContainer,pCurContainer->getContainerString()));
+			if(pCurContainer->getDocSectionLayout() != m_pDocSec)
+			{
+			        break;
+			}
 			if(pCurContainer->getContainerType() == FP_CONTAINER_FOOTNOTE || ((pCurContainer->getContainerType() == FP_CONTAINER_ANNOTATION)))
 			{
 //
@@ -342,6 +466,8 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 //
 						if(pVTab->getContainerType() == FP_CONTAINER_TABLE)
 						{
+						        xxx_UT_DEBUGMSG(("Deleting broken tables for %p \n",pVTab));
+						        UT_ASSERT(static_cast<fp_TableContainer *>(pVTab)->getMasterTable() == NULL);
 							static_cast<fp_TableContainer *>(pVTab)->deleteBrokenTables(true,true);
 						}
 						else
@@ -383,7 +509,7 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 					// Ok.  Now, deduct the proper amount from iMaxColHeight.
 
 					// OK get a vector of the footnote containers in this line.
-				 
+				        UT_sint32 iTheseFootnotes = 0; 
 					UT_GenericVector<fp_FootnoteContainer*> vecFootnotes;
 					pCurLine->getFootnoteContainers(&vecFootnotes);
 					fp_Page *pCurPage = pCurColumn->getPage();
@@ -392,19 +518,27 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 					for(i=0; i< vecFootnotes.getItemCount();i++)
 					{
 						fp_FootnoteContainer * pFC = vecFootnotes.getNthItem(i);
+						if(pFC->getHeight() > iMaxSecCol)
+						{
+						    UT_DEBUGMSG(("Footnote container height %d \n",pFC->getHeight()));
+						    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+						    pFC->setHeight(iMaxSecCol -600);
+						}
 						if(pFC && ((pFC->getPage() == NULL) || (pFC->getPage() != pCurPage)))
 						{
+						        iTheseFootnotes += pFC->getHeight();
 							iFootnoteHeight += pFC->getHeight();
-						}				
+						}
 					}	
-					UT_DEBUGMSG(("got footnote section height %d\n", iFootnoteHeight));
-					iWorkingColHeight += iFootnoteHeight;
+					xxx_UT_DEBUGMSG(("got footnote section height %d\n", iFootnoteHeight));
+					iWorkingColHeight += iTheseFootnotes;
 				}
 				if(pCurLine->containsAnnotations() && _displayAnnotations())
 				  {
 					// Ok.  Now, deduct the proper amount from iMaxColHeight.
 
 					// OK get a vector of the Annotation containers in this line.
+				        UT_sint32 iTheseAnnotations = 0; 
 				 
 					UT_GenericVector<fp_AnnotationContainer*> vecAnnotations;
 					pCurLine->getAnnotationContainers(&vecAnnotations);
@@ -416,19 +550,19 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 						fp_AnnotationContainer * pAC = vecAnnotations.getNthItem(i);
 						if(pAC && ((pAC->getPage() == NULL) || (pAC->getPage() != pCurPage)))
 						{
+						        iTheseAnnotations += pAC->getHeight();
 							iFootnoteHeight += pAC->getHeight();
 						}				
 					}	
-					UT_DEBUGMSG(("got Total section height %d\n", iFootnoteHeight));
-					iWorkingColHeight += iFootnoteHeight;
+					iWorkingColHeight += iTheseAnnotations;
 
 				  }
 			}
-
+			xxx_UT_DEBUGMSG(("WorkingColHeight = %d \n",iWorkingColHeight));
+			xxx_UT_DEBUGMSG(("iTotalContainerSpace = %d MarginAfter %d \n",iTotalContainerSpace,iContainerMarginAfter));
 			if ((iWorkingColHeight + iTotalContainerSpace) > iMaxColHeight)
 			{
 				pOffendingContainer = pCurContainer;
-
 				/*
 				  We have found the offending container (the first one
 				  which won't fit in the column) and we now need to
@@ -744,10 +878,10 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 		bEquivColumnBreak = bEquivColumnBreak && ( iMaxColHeight < (iWorkingColHeight + iTotalContainerSpace));
 		if (pLastContainerToKeep)
 		{
-			while(pLastContainerToKeep && (pLastContainerToKeep->getContainerType() == FP_CONTAINER_FOOTNOTE )
-			      && (pLastContainerToKeep->getContainerType() == FP_CONTAINER_ANNOTATION))
+		        while(pLastContainerToKeep && ((pLastContainerToKeep->getContainerType() == FP_CONTAINER_FOOTNOTE )
+						 || (pLastContainerToKeep->getContainerType() == FP_CONTAINER_ANNOTATION)))
 			{
-				pLastContainerToKeep = pLastContainerToKeep->getPrevContainerInSection();
+			        pLastContainerToKeep = pLastContainerToKeep->getPrevContainerInSection();
 			}
 			if(pLastContainerToKeep)
 			{
@@ -780,6 +914,10 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 				pCurContainer = _getNext(pCurContainer);
 				continue;
 			}
+			if(pCurContainer->getDocSectionLayout() != m_pDocSec)
+			{
+			    break;
+			}
 			if (pCurContainer->getContainer() != pCurColumn || (pCurColumn->findCon(pCurContainer) < 0) )
 			{
 //
@@ -804,7 +942,7 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 #endif
 					pVert->removeContainer(pCurContainer,true);
 				}
-				if(pCurContainer->getContainer() != pCurColumn)
+				if((pCurContainer->getContainer() != pCurColumn) && (pCurContainer->getDocSectionLayout() == pCurColumn->getDocSectionLayout()))
 				{
 					if(pCurContainer->getContainerType() != FP_CONTAINER_FRAME)
 					{
@@ -816,9 +954,8 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 						fp_Page * pNextPage = pCurColumn->getPage();
 						if(pNextPage && pPage && (pNextPage != pPage))
 						{
-							FL_DocLayout * pDL = m_pDocSec->getDocLayout();
 							fp_FrameContainer * pFC = reinterpret_cast<fp_FrameContainer *>(pCurContainer);
-							if((pDL->findPage(pPage) >= 0) && (pDL->findPage(pNextPage) >= 0))
+							if(( pDocLayout->findPage(pPage) >= 0) && ( pDocLayout->findPage(pNextPage) >= 0))
 							{
 								if((pPage->findFrameContainer(pFC) >=0) &&
 								   (pNextPage->findFrameContainer(pFC) < 0))
@@ -830,6 +967,7 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 						}
 					}
 				}
+				
 			}
 //
 // Now make sure footnotes and annotations are on the same page as the reference.
@@ -950,7 +1088,7 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 							}
 						}
 					}
-					if(pCurTable->containsAnnotations())
+					if(pCurTable->containsAnnotations()  && _displayAnnotations())
 					{
 						// OK get a vector of the footnote containers in this line.
 						UT_GenericVector<fp_AnnotationContainer*> vecAnnotations;
@@ -986,7 +1124,7 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			}
 //
 // Code to fix order in the column
-//
+// 
 #if 1
 			if((pCurColumn->findCon(pCurContainer) >= 0) && (pCurColumn->findCon(pCurContainer)  != conPos))
 			{
@@ -1001,7 +1139,9 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 #endif
 			conPos++;
 			if (pCurContainer == pLastContainerToKeep)
+			{
 				break;
+			}
 			else
 			{
 				if((pLastContainerToKeep!=NULL) && (_getNext(pCurContainer) == NULL))
@@ -1018,6 +1158,9 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			}
 		}
 
+		//
+		// end of add container to columns
+		//
 		if (pLastContainerToKeep
 			&& pCurColumn->getLastContainer() != pLastContainerToKeep)
 		{
@@ -1045,9 +1188,9 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 				{
 					if(bBreakOnColumnBreak || bEquivColumnBreak)
 						pNextColumn = static_cast<fp_Column*>
-						 (pSL->getNewContainer(_getNext(pLastContainerToKeep)));
+						 (m_pDocSec->getNewContainer(_getNext(pLastContainerToKeep)));
 					else
-						pNextColumn = static_cast<fp_Column*>(pSL->getNewContainer(NULL));
+						pNextColumn = static_cast<fp_Column*>(m_pDocSec->getNewContainer(NULL));
 				}
 			}
 			while (pLastContainerToKeep->getContainerType()
@@ -1063,7 +1206,9 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			while (pCurColumn != NULL && pCurColumn != pNextColumn)
 			{
 				xxx_UT_DEBUGMSG(("Start of bump loop pNextColumn %x \n",pNextColumn));
-				bool isTOCTABLE = ((pOuterContainer->getContainerType() == FP_CONTAINER_TABLE) || (pOuterContainer->getContainerType() == FP_CONTAINER_TABLE));
+				bool isTOCTABLE = false;
+				if(pOuterContainer && ((pOuterContainer->getContainerType() == FP_CONTAINER_TABLE) || (pOuterContainer->getContainerType() == FP_CONTAINER_TABLE)))
+				        isTOCTABLE= true;
 				pCurColumn->bumpContainers(pLastContainerToKeep);
 
 				pCurColumn->layout();
@@ -1121,7 +1266,6 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			if(pOuterContainer)
 				bTOCTest = (pOuterContainer->getContainerType() == 
 							  FP_CONTAINER_TOC);
-
 			pCurColumn->layout();
 
 			// pCurColumn->layout() might delete a broken table or TOC; fixup.
@@ -1137,18 +1281,18 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 				pOuterContainer =  _getNext(pCurColumn->getLastContainer());
 				bTOCTest = true;
 			}
-//			pCurColumn->validate();
+			//			pCurColumn->validate();
 			pPrevColumn = pCurColumn;
 			pCurColumn = static_cast<fp_Column *>(pCurColumn->getNext());
 			if(pCurColumn == NULL && (bTableTest || bTOCTest) && pOuterContainer != NULL)
 			{
 				if(pOuterContainer->isVBreakable())
 				{
-					pCurColumn = static_cast<fp_Column *>(pSL->getNewContainer(NULL));
+					pCurColumn = static_cast<fp_Column *>(m_pDocSec->getNewContainer(NULL));
 				}
 				else
 				{
-					pCurColumn = static_cast<fp_Column *>(pSL->getNewContainer(pOuterContainer));
+					pCurColumn = static_cast<fp_Column *>(m_pDocSec->getNewContainer(pOuterContainer));
 				}
 			}
 			else if(pCurColumn == NULL && pLastContainerToKeep && _getNext(pLastContainerToKeep))
@@ -1156,11 +1300,11 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 				fp_Container * pCon = _getNext(pLastContainerToKeep);
 				if(pCon->isVBreakable())
 				{
-					pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(NULL));
+					pCurColumn =  static_cast<fp_Column *>(m_pDocSec->getNewContainer(NULL));
 				}
 				else
 				{
-					pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(pCon));
+					pCurColumn =  static_cast<fp_Column *>(m_pDocSec->getNewContainer(pCon));
 				}
 			}
 		}
@@ -1169,11 +1313,11 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			fp_Container * pCon = _getNext(pLastContainerToKeep);
 			if(pCon->isVBreakable())
 			{
-				pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(NULL));
+				pCurColumn =  static_cast<fp_Column *>(m_pDocSec->getNewContainer(NULL));
 			}
 			else
 			{
-				pCurColumn =  static_cast<fp_Column *>(pSL->getNewContainer(pCon));
+				pCurColumn =  static_cast<fp_Column *>(m_pDocSec->getNewContainer(pCon));
 			}
 		}
 //
@@ -1190,12 +1334,17 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 			if(pNextContainer == NULL)
 			{
 				pCurColumn = NULL;
+				pOuterContainer = NULL;
 			}
 			else
 			{
 				pOuterContainer = pNextContainer;
 			}
 		}
+		if(pCurContainer && (pCurContainer->getDocSectionLayout() != m_pDocSec))
+		{
+		        pCurColumn = NULL;
+		}
 //
 // Loop back for next pCurContainer. Finish if pCurColumn == NULL.
 //
@@ -1204,21 +1353,23 @@ UT_sint32 fb_ColumnBreaker::_breakSection(fl_DocSectionLayout * pSL, fp_Page * p
 //
 // End of massive while loop here
 //
-	m_pStartPage = NULL;
-	m_bStartFromStart = false;
 	if(m_bReBreak)
 	{
 		UT_ASSERT(0);
-		breakSection(pSL);
+		m_pStartPage = NULL;
+		m_bStartFromStart = false;
+		breakSection();
 	}
 //
 // Look if the next DocSectionLayout needs section break.(
 // This happens if a column height changes. If so, do it!
 //
-	if(pSL->getNextDocSection() && pSL->needsSectionBreak())
+#if 0
+	if(m_pDocSec->getNextDocSection() && m_pDocSec->needsSectionBreak())
 	{
-		pSL->completeBreakSection();
+		m_pDocSec->completeBreakSection();
 	}
+#endif
 	return 0; // TODO return code
 }
 
@@ -1292,7 +1443,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 {
 	bool bDoTableBreak;
 
-    xxx_UT_DEBUGMSG(("breakTable:!!!!!!!!!!!! Offending Table is %x \n",pOffendingContainer));
+        xxx_UT_DEBUGMSG(("breakTable:!!!!!!!!!!!! Offending Table is %p \n",pOffendingContainer));
 	fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pOffendingContainer);
 	if(!pTab->isThisBroken())
 	{
@@ -1369,7 +1520,9 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 //
 		if(iBreakAt < 30)
 		{
-			pOffendingContainer = static_cast<fp_Container *>(pTab);
+		        xxx_UT_DEBUGMSG(("Break too small %d , bumping container height of Tab %d \n",iBreakAt,pTab->getHeight()));
+		        pLastContainerToKeep = static_cast<fp_Container *>(pTab->getPrevContainerInSection());
+		        pOffendingContainer = static_cast<fp_Container *>(pBroke);
 		}
 		else
 		{
@@ -1380,7 +1533,7 @@ bool fb_ColumnBreaker::_breakTable(fp_Container*& pOffendingContainer,
 //
 			fp_TableContainer * pNewTab = static_cast<fp_TableContainer *>(pBroke->VBreakAt(iBreakAt));
 			pOffendingContainer = static_cast<fp_Container *>(pNewTab);
-		    xxx_UT_DEBUGMSG(("SEVIOR: Created broken table %x \n",pOffendingContainer));
+		        xxx_UT_DEBUGMSG(("SEVIOR: Created broken table %p height %d \n",pOffendingContainer,pOffendingContainer->getHeight()));
 			UT_ASSERT(pBroke->getHeight() > 0);
 			UT_ASSERT(pNewTab->getHeight() > 0);
 			pLastContainerToKeep = static_cast<fp_Container *>(pTab);
@@ -1513,8 +1666,13 @@ fp_Container * fb_ColumnBreaker::_getNext(fp_Container * pCon)
 	if(pCon->getContainerType() != FP_CONTAINER_ENDNOTE)
 	{
 		pNext = pCon->getNextContainerInSection();
+		if(pNext)
+		{
+		    UT_return_val_if_fail(pNext->getDocSectionLayout() == m_pDocSec,NULL);
+		}
 		if(pNext != NULL)
 		{
+		        xxx_UT_DEBUGMSG(("_getNext: 1 Returning %p \n",pNext));
 			return pNext;
 		}
 		xxx_UT_DEBUGMSG(("_getNext: Returning First endnote \n"));
@@ -1540,12 +1698,25 @@ fp_Container * fb_ColumnBreaker::_getNext(fp_Container * pCon)
 #endif
 	}
 #if DEBUG
-		if(pNext)
-		{
-			UT_ASSERT(pNext->getContainerType() != FP_CONTAINER_FRAME);
-		}
+	if(pNext)
+	{
+	    UT_ASSERT(pNext->getContainerType() != FP_CONTAINER_FRAME);
+	}
 #endif
+	xxx_UT_DEBUGMSG(("_getNext: 2 Returning %p \n",pNext));
 
 	return pNext;
 }
-	
+#if 0	
+bool fb_ColumnBreaker::_verifyAllInColumns(void)
+{
+  fp_Container * pCon = m_pDocSec->getFirstContainer();
+  while(pCon)
+    {
+      if((pCon->getContainerType() == FP_CONTAINER_LINE) || (pCon->getContainerType() == FP_CONTAINER_TABLE))
+	{
+	  
+	} 
+    }
+}
+#endif
