@@ -56,6 +56,7 @@ fp_VerticalContainer::fp_VerticalContainer(FP_ContainerType iType, fl_SectionLay
 	m_imaxContainerHeight(0)
 {
 	clearWrappedLines();
+	UT_ASSERT(getDocSectionLayout());
 }
 
 /*!
@@ -690,7 +691,7 @@ void fp_VerticalContainer::getScreenOffsets(fp_ContainerObject* pContainer,
 	}
 	fp_Container * pPrev = NULL;
 	fp_TableContainer * pTab = NULL;
-	while(!pCon->isColumnType() && !bTable)
+	while(pCon && !pCon->isColumnType() && !bTable)
 	{
 		my_xoff += pCon->getX();
 		xxx_UT_DEBUGMSG(("Screen offsets my_xoff %d pCon %x type %s \n",my_xoff,pCon,pCon->getContainerString()));
@@ -765,7 +766,9 @@ void fp_VerticalContainer::getScreenOffsets(fp_ContainerObject* pContainer,
 		}
 		pPrev = pCon;
 		pCon = pCon->getContainer();
+		UT_return_if_fail(pCon);
 	}
+	UT_return_if_fail(pCon);
 	UT_sint32 col_x =0;
 	UT_sint32 col_y =0;
 	xoff = my_xoff + pOrig->getX();
@@ -865,6 +868,7 @@ void fp_VerticalContainer::removeContainer(fp_Container* pContainer,bool bClear)
 bool fp_VerticalContainer::insertContainer(fp_Container* pNewContainer)
 {
 	UT_return_val_if_fail(pNewContainer,false);
+	UT_return_val_if_fail((pNewContainer->getContainerType() == FP_CONTAINER_ENDNOTE) || (pNewContainer->getDocSectionLayout() == getDocSectionLayout()),false);
 	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 	pNewContainer->clearScreen();
 	xxx_UT_DEBUGMSG(("Insert  Container after CS %x in column %x \n",pNewContainer,this));
@@ -891,6 +895,7 @@ UT_sint32	fp_VerticalContainer::getColumnGap(void) const
 bool fp_VerticalContainer::addContainer(fp_Container* pNewContainer)
 {
 	UT_return_val_if_fail(pNewContainer,false);
+	UT_return_val_if_fail((pNewContainer->getContainerType() == FP_CONTAINER_ENDNOTE) || (pNewContainer->getDocSectionLayout() == getDocSectionLayout()),false);
 	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 	if(pNewContainer->getContainer() != NULL)
 	{
@@ -916,6 +921,7 @@ bool fp_VerticalContainer::insertContainerAfter(fp_Container*	pNewContainer, fp_
 {
 	UT_ASSERT(pAfterContainer);
 	UT_return_val_if_fail(pNewContainer, false);
+	UT_return_val_if_fail((pNewContainer->getContainerType() == FP_CONTAINER_ENDNOTE) || (pNewContainer->getDocSectionLayout() == getDocSectionLayout()),false);
 	UT_ASSERT(pNewContainer->getContainerType() != FP_CONTAINER_ANNOTATION);
 
 	UT_sint32 count = countCons();
@@ -951,7 +957,6 @@ bool fp_VerticalContainer::insertContainerAfter(fp_Container*	pNewContainer, fp_
 	return true;
 }
 
-
 /*!
   Clear container content from screen.
 
@@ -1563,6 +1568,36 @@ fp_Container* fp_VerticalContainer::getFirstContainer(void) const
 	}
 }
 
+UT_sint32  fp_VerticalContainer::countWrapped(void)
+{
+  UT_sint32 nWrapped = 0;
+  UT_sint32 i = 0;
+  for(i=0; i<countCons();i++)
+  {
+      fp_Container * pCon = static_cast<fp_Container*>(getNthCon(i));
+      if(pCon->getContainerType() == FP_CONTAINER_LINE)
+      {
+	  fp_Line * pLine = static_cast<fp_Line *>(pCon);
+	  xxx_UT_DEBUGMSG(("Line %d MaxWidth %d \n",i,pLine->getMaxWidth()));
+	  if(pLine->isWrapped())
+	  {
+	      nWrapped++;
+	  }
+	  else if(pLine->isSameYAsPrevious())
+	  {
+	      nWrapped++;
+	  }
+	  else if((pLine->getMaxWidth() > 0) && (pLine->getMaxWidth() < getWidth()))
+	  {
+	      nWrapped++;
+
+	  }
+      }
+  }
+  return nWrapped;
+}
+
+
 /*!
  Return last line in the container
  \return The last line, or NULL if the container is empty
@@ -1596,6 +1631,7 @@ void fp_VerticalContainer::bumpContainers(fp_ContainerObject* pLastContainerToKe
 	fp_TOCContainer *pTOC2 = NULL;
 	fp_VerticalContainer* pNextContainer = static_cast<fp_VerticalContainer*>(getNext());
 	UT_return_if_fail(pNextContainer);
+	UT_return_if_fail((pNextContainer->getContainerType() == FP_CONTAINER_ENDNOTE) || (pNextContainer->getDocSectionLayout() == getDocSectionLayout()));
 	if (pNextContainer->isEmpty())
 	{
 		for (i=ndx; i< countCons(); i++)
@@ -1744,6 +1780,21 @@ void fp_Column::collapseEndnotes(void)
 	}
 }
 
+
+/*!
+ * Returns true of the column contains a page break.
+ */
+bool fp_Column::containsPageBreak(void) const
+{
+    fp_Container * pCon = getLastContainer();
+    if(pCon && (pCon->getContainerType() ==FP_CONTAINER_LINE))
+    {
+	fp_Line * pLine = static_cast<fp_Line *>(pCon);
+	return pLine->containsForcedPageBreak();
+    }
+    return false;
+}
+
 void fp_Column::setPage(fp_Page * pPage)
 {
 	if(pPage == NULL)
@@ -1877,6 +1928,12 @@ void fp_Column::layout(void)
 			{
 				addWrappedLine(pLine);
 			}
+#if 0
+			else if((pLine->getMaxWidth() > 0) && (pLine->getMaxWidth() < getWidth()))
+			{
+				addWrappedLine(pLine);
+			}
+#endif
 			if(pLine->isSameYAsPrevious() && pLine->getPrev())
 			{
 				UT_sint32 iPrevY = static_cast<fp_Line *>(pLine->getPrev())->getY();
@@ -2042,11 +2099,17 @@ void fp_Column::layout(void)
 UT_sint32 fp_Column::getMaxHeight(void) const
 {
 	const fp_VerticalContainer * pVC = static_cast<const fp_VerticalContainer *>(this);
+	UT_sint32 iMaxHeight = 0;
 	if(!getPage())
 	{
-		return pVC->getMaxHeight();
+		iMaxHeight = pVC->getMaxHeight();
 	}
-	return getPage()->getAvailableHeightForColumn(this);
+	else
+	{
+	        iMaxHeight = getPage()->getAvailableHeightForColumn(this);
+	}
+	//UT_ASSERT(iMaxHeight > 0);
+	return iMaxHeight;
 }
 
 fl_DocSectionLayout* fp_Column::getDocSectionLayout(void) const
