@@ -85,7 +85,8 @@ void fp_EmbedRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 	// 'cause we need to retrieve the font-size
 	const PP_AttrProp * pBlockAP = NULL;
 	const PP_AttrProp * pSectionAP = NULL;
-	if(pG == NULL)
+	FL_DocLayout * pLayout = getBlock()->getDocLayout();
+	if(pG == NULL && pLayout->isQuickPrint() )
 	{
 	     pG = getGraphics();
 	     if((m_iEmbedUID >= 0) && getEmbedManager())
@@ -98,7 +99,6 @@ void fp_EmbedRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 	
 	getBlockAP(pBlockAP);
 
-	FL_DocLayout * pLayout = getBlock()->getDocLayout();
 	const GR_Font * pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP,pG);
 	if(pLayout->isQuickPrint() && pG->queryProperties(GR_Graphics::DGP_PAPER))
 
@@ -119,6 +119,8 @@ void fp_EmbedRun::_lookupProperties(const PP_AttrProp * pSpanAP,
 	{
 		_setFont(pFont);
 	}
+	if(pG == NULL)
+	  pG = getGraphics();
 	m_iPointHeight = pG->getFontAscent(pFont) + pG->getFontDescent(pFont);
 	const char* pszSize = PP_evalProperty("font-size",pSpanAP,pBlockAP,pSectionAP,
 					      getBlock()->getDocument(), true);
