@@ -232,6 +232,10 @@ UT_sint32 fp_Page::getColumnGap(void) const
 	return getOwningSection()->getColumnGap();
 }
 
+void fp_Page::clearCountWrapNumber(void)
+{
+	m_iCountWrapPasses = 0;
+}
 /*!
  * This method scans the page, looking for lines that overlap wrapped
  * positioned frames. As it finds them it records the line and the block
@@ -249,7 +253,7 @@ UT_sint32 fp_Page::getColumnGap(void) const
  */
 fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 {
-	if(m_iCountWrapPasses > 199)
+	if(m_iCountWrapPasses > 19)
 	{
 		return NULL;
 	}
@@ -263,9 +267,9 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 			xxx_UT_DEBUGMSG(("Prev page %x Prev page number %d Number Frames %d \n",getPrev(),iPage,getPrev()->countAboveFrameContainers() ));
 		}
 #endif
-	if(m_iCountWrapPasses > 100)
+	if(m_iCountWrapPasses > 10)
     {
-	  xxx_UT_DEBUGMSG(("Number of wrapped passed = %d \n",m_iCountWrapPasses));
+		UT_DEBUGMSG(("Number of wrapped passed = %d \n",m_iCountWrapPasses));
 	}
 	UT_sint32 i= 0;
 	UT_sint32 nWrapped = 0;
@@ -284,7 +288,16 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 		}
 		while(pCol)
 		{
-			nWrapped += pCol->getNumWrapped();
+			if(m_iCountWrapPasses > 10)
+			{
+				nWrapped += pCol->countWrapped();
+			}
+			else
+			{
+				nWrapped += pCol->getNumWrapped();
+			}
+			UT_DEBUGMSG(("Page number %d \n",getDocLayout()->findPage(this)));
+			UT_DEBUGMSG(("NumWrapped %d \n",nWrapped));
 			pCol = static_cast<fp_Column *>(pCol->getFollower());
 		}
 	}
@@ -297,6 +310,7 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 			nWrappedObjs++;
 		}
 	}
+
 	if((nWrapped == 0) && (nWrappedObjs == 0))
 	{
 		xxx_UT_DEBUGMSG(("page %x nWrapped %d nWrappedObjs %d does not need updating \n",this,nWrapped,nWrappedObjs));
@@ -319,8 +333,27 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 				}
 				
 		}
-		if(nWrappedObjs == 0)
+		if((nWrappedObjs == 0) &&(m_iCountWrapPasses == 0))
+		{
 			return NULL;
+		}
+		else if((nWrappedObjs == 0) && (getNext() == NULL))
+		{
+				return NULL;
+		}
+		else if(nWrappedObjs == 0)
+		{
+			fp_Column * pNextNoWrapCol = getNext()->getNthColumnLeader(0);
+			if(pNextNoWrapCol == NULL)
+			{
+				// FIXME this appears to happen sometimes.
+				// we should work out a better thing to do this this
+				return NULL;
+			}
+			fp_Container * pNewFirstWrapCon = static_cast<fp_Container *>(pNextNoWrapCol->getNthCon(0));
+			getNext()->clearCountWrapNumber();
+			return pNewFirstWrapCon;
+		}
 	}
 	bool bFormatAllWrapped = ((nWrapped > 0) && (nWrappedObjs == 0));
 	UT_GenericVector<_BL *> vecBL;
@@ -404,6 +437,11 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 						}
 						pNextCol = pFirstCol;
 						fp_Container * pNewFirstCon = static_cast<fp_Container *>(pNextCol->getNthCon(0));
+						if(!pNewFirstCon)
+						{
+							pBL = vecCollapse.getNthItem(0);
+							pNewFirstCon = static_cast<fp_Container *>(pBL->getFirstRun()->getLine());
+						}
 						return pNewFirstCon;
 					}
 					else
@@ -785,7 +823,7 @@ fp_TableContainer * fp_Page::getContainingTable(PT_DocPosition pos)
 
 UT_sint32 fp_Page::getAvailableHeight(void) const
 {
-	fl_DocSectionLayout * pDSL = getNthColumnLeader(0)->getDocSectionLayout();
+	fl_DocSectionLayout * pDSL = getOwningSection();
 	UT_sint32 avail = getHeight() - pDSL->getTopMargin() - pDSL->getBottomMargin();
 	UT_sint32 i =0;
 	for(i=0; i< static_cast<UT_sint32>(countFootnoteContainers()); i++)
@@ -804,6 +842,25 @@ UT_sint32 fp_Page::getAvailableHeight(void) const
 	return avail;
 }
 
+/*!
+ * Returns true if there is a column with a page break in it
+ */
+bool fp_Page::containsPageBreak(void) const
+{
+	fp_Column * pCol = NULL;
+	UT_sint32 i = 0;
+	for(i=0; i<countColumnLeaders();i++)
+	{
+		pCol = getNthColumnLeader(0);
+		while(pCol)
+		{
+			if(pCol->containsPageBreak())
+				return true;
+			pCol = pCol->getFollower();
+		}
+	}
+	return false;
+}
 /*!
  * This method returns the height available to the requested column. It 
  * subtracts the height given to previous columns on the page as well as the 
@@ -816,7 +873,7 @@ UT_sint32 fp_Page::getAvailableHeightForColumn(const fp_Column * pColumn) const
 	fl_DocSectionLayout * pDSL = pCurLeader->getDocSectionLayout();
 	UT_sint32 avail = getHeight() - pDSL->getTopMargin() - pDSL->getBottomMargin();
 	UT_sint32 i =0;
-	xxx_UT_DEBUGMSG(("fp_Page:: Total avail after margins subtracted %d \n",avail));
+	UT_DEBUGMSG(("fp_Page:: Total avail after margins subtracted %d \n",avail));
 	UT_sint32 nLeaders = static_cast<UT_sint32>(countColumnLeaders()); 
 	while((pCurLeader != pLeader) && (nLeaders > 1))	
 	{
@@ -855,6 +912,7 @@ UT_sint32 fp_Page::getAvailableHeightForColumn(const fp_Column * pColumn) const
 					avail -= pAC->getHeight();
 			}
 	}
+	//UT_ASSERT(avail > 0);
 	return avail;
 
 }
@@ -1085,15 +1143,26 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool /*bAlwaysUseWhiteBackground*/)
 		}
 		
 		dg_DrawArgs da = *pDA;
-#if 0
-		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) &&
-		   !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
+		da.xoff += pFC->getX();
+		da.yoff += pFC->getY();
+		pFC->draw(&da);
+	}
+	//
+	// Handle Tight wrapped frames
+	//
+	count = m_vecAboveFrames.getItemCount();
+	for (i=0; i<count; i++)
+	{
+		fp_FrameContainer* pFC = m_vecAboveFrames.getNthItem(i);
+		if(!pFC->isTightWrapped())
+			continue;
+		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
+		if(m_rDamageRect.intersectsRect(&r))
 		{
-			fp_Column* pFirstColumnLeader = getNthColumnLeader(0);
-			fl_DocSectionLayout* pFirstSectionLayout = (pFirstColumnLeader->getDocSectionLayout());
-			da.yoff -= pFirstSectionLayout->getTopMargin();
+			pFC->setOverWrote();
 		}
-#endif
+		
+		dg_DrawArgs da = *pDA;
 		da.xoff += pFC->getX();
 		da.yoff += pFC->getY();
 		pFC->draw(&da);
@@ -1115,7 +1184,7 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool /*bAlwaysUseWhiteBackground*/)
 			da.xoff += pCol->getX();
 			da.yoff += pCol->getY(pDA->pG);
 
-			xxx_UT_DEBUGMSG(("Draw in page col X offset %d \n",da.xoff));
+			UT_DEBUGMSG(("Draw in page col Y offset %d \n",da.yoff));
 			pCol->draw(&da);
 
 			fp_Column *pNextCol = pCol->getFollower();
@@ -1198,6 +1267,8 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool /*bAlwaysUseWhiteBackground*/)
 	for (i=0; i<count; i++)
 	{
 		fp_FrameContainer* pFC = m_vecAboveFrames.getNthItem(i);
+		if(pFC->isTightWrapped())
+			continue;
 		UT_Rect r(pFC->getX(),pFC->getY(),pFC->getWidth(),pFC->getHeight());
 		if(m_rDamageRect.intersectsRect(&r))
 		{
@@ -1250,6 +1321,17 @@ void   fp_Page::expandDamageRect(UT_sint32 x, UT_sint32 y,
 	return;
 }
 
+/*!
+ * Returns true if the objects intersects the damaged region
+ */
+bool   fp_Page::intersectsDamagedRect(fp_ContainerObject * pObj)
+{
+	UT_Rect * pRec = pObj->getScreenRect();
+	bool bIntersects = m_rDamageRect.intersectsRect(pRec);
+	delete pRec;
+	return bIntersects;
+}
+
 void   fp_Page::redrawDamagedFrames(dg_DrawArgs* pDA)
 {
 	// draw Frames
@@ -1298,6 +1380,8 @@ UT_sint32 fp_Page::countColumnLeaders(void) const
 
 fp_Column* fp_Page::getNthColumnLeader(UT_sint32 n) const
 {
+	if(n >= m_vecColumnLeaders.getItemCount())
+		return NULL;
 	return m_vecColumnLeaders.getNthItem(n);
 }
 
