@@ -95,10 +95,14 @@ public:
 		abi_format,
 		abi_frame_column_xpos,
 		abi_frame_column_ypos,
+		abi_frame_expand_height,
 		abi_frame_height,
+		abi_frame_horiz_align,
+		abi_frame_min_height,
 		abi_frame_page_xpos,
 		abi_frame_page_ypos,
 		abi_frame_position_to,
+		abi_frame_rel_width,
 		abi_frame_type,
 		abi_frame_width,
 		abi_frame_xpos,
