@@ -26,6 +26,8 @@
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 
+#include "gr_CocoaCairoGraphics.h"
+
 #include "xap_App.h"
 #include "xap_CocoaApp.h"
 #include "xap_Frame.h"
@@ -80,8 +82,8 @@ void AP_CocoaDialog_FormatFrame::runModeless(XAP_Frame * /*pFrame*/)
 	/* make a new Cocoa GC
 	 */
 	DELETEP(m_pPreviewWidget);
-	GR_CocoaAllocInfo ai(view);
-	m_pPreviewWidget = (GR_CocoaGraphics *) m_pApp->newGraphics(ai);
+	GR_CocoaCairoAllocInfo ai(view);
+	m_pPreviewWidget = (GR_CocoaCairoGraphics *) m_pApp->newGraphics(ai);
 
 	/* TODO: we need a good widget to query with a probable non-white (i.e. gray, or
 	 *       a similar bgcolor as our parent widget) background. This should be fine.
@@ -742,17 +744,22 @@ void AP_CocoaDialog_FormatFrame::_storeWindowData(void)
 	[_borderColorWell setEnabled:bEnabled];
 	[_borderStepper   setEnabled:bEnabled];
 	[_borderNumber    setEnabled:bEnabled];
+	
+	[_bgColorLabel setEnabled:bEnabled];
+	[_borderColorLabel setEnabled:bEnabled];
 
 	[ _rightBorderColorWell	setEnabled:bEnabled];
 	[  _leftBorderColorWell	setEnabled:bEnabled];
 	[   _topBorderColorWell	setEnabled:bEnabled];
 	[_bottomBorderColorWell	setEnabled:bEnabled];
 
+	[ _borderStepper setEnabled:bEnabled];
 	[ _rightBorderStepper setEnabled:bEnabled];
 	[  _leftBorderStepper setEnabled:bEnabled];
 	[   _topBorderStepper setEnabled:bEnabled];
 	[_bottomBorderStepper setEnabled:bEnabled];
 
+	[ _borderNumberForm setEnabled:bEnabled];
 	[ _rightBorderNumberForm setEnabled:bEnabled];
 	[  _leftBorderNumberForm setEnabled:bEnabled];
 	[   _topBorderNumberForm setEnabled:bEnabled];
@@ -764,12 +771,12 @@ void AP_CocoaDialog_FormatFrame::_storeWindowData(void)
 	[_bottomBorderBtn setEnabled:bEnabled];
 
 	if (bEnabled)
-		{
-			NSWindow * window = [self window];
+	{
+		NSWindow * window = [self window];
 
-			// [window makeKeyAndOrderFront:self];
-			[window makeFirstResponder:_borderNumberForm];
-		}
+		// [window makeKeyAndOrderFront:self];
+		[window makeFirstResponder:_borderNumberForm];
+	}
 }
 
 - (void)setWrapState:(int)state
