@@ -1,6 +1,7 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2003 Marc Maurer
+ * Copyright (c) 2009 Hubert Figuiere
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -70,6 +71,19 @@ AP_Dialog_FormatTable::AP_Dialog_FormatTable(XAP_DialogFactory * pDlgFactory, XA
 	  m_pImage(NULL),
 	  m_pGraphic(NULL)
 {
+	//
+	// These are hardwired into the GUI.
+	//
+	const char * sThickness[FORMAT_TABLE_NUMTHICKNESS] ={"0.25pt","0.5pt",
+													   "0.75pt","1.0pt",
+													   "1.5pt","2.25pt","3pt",
+													   "4.5pt","6.0pt"};
+	UT_sint32 i = 0;
+	for(i=0; i< FORMAT_TABLE_NUMTHICKNESS ;i++)
+	{
+		m_dThickness[i] = UT_convertToInches(sThickness[i]);
+	}
+
 	if(m_vecProps.getItemCount() > 0)
 		m_vecProps.clear();
 	  
@@ -244,24 +258,21 @@ void AP_Dialog_FormatTable::askForGraphicPathName(void)
 	m_sImagePath.clear();
 	UT_String_sprintf(m_sImagePath,"%d",uid);
 
+    const UT_ByteBuf * pBB = m_pGraphic->getBuffer();
 	if(m_pGraphic->getType() == FGT_Raster)
 	{
-		UT_sint32 iImageWidth;
-		UT_sint32 iImageHeight;
-		const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
-		UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 		m_pImage = static_cast<GR_Image *>(
 			pG->createNewImage( m_sImagePath.c_str(),
-								pBB,
-								iImageWidth,
-								iImageHeight,
+								pBB, pFG->getMimeType(),
+								pFG->getWidth(),
+								pFG->getHeight(),
 								GR_Image::GRT_Raster));
 	}
 	else
 	{
 		m_pImage = static_cast<GR_Image *>(
 			pG->createNewImage( m_sImagePath.c_str(),
-								static_cast<FG_GraphicVector *>(pFG)->getVector_SVG(),
+								pBB, pFG->getMimeType(),
 								m_pFormatTablePreview->getWindowWidth()-2,
 								m_pFormatTablePreview->getWindowHeight()-2,
 								GR_Image::GRT_Vector));
@@ -420,24 +431,21 @@ void AP_Dialog_FormatTable::setCurCellProps(void)
 						m_pGraphic = pFG;
 						m_sImagePath = pFG->getDataId();
 						GR_Graphics * pG = m_pFormatTablePreview->getGraphics();
+                        const UT_ByteBuf * pBB = pFG->getBuffer();
 						if(m_pGraphic->getType() == FGT_Raster)
 						{
-							UT_sint32 iImageWidth;
-							UT_sint32 iImageHeight;
-							const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
-							UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 							m_pImage = static_cast<GR_Image *>(
 								pG->createNewImage( m_sImagePath.c_str(),
-													pBB,
-													iImageWidth,
-													iImageHeight,
+													pBB, pFG->getMimeType(),
+													pFG->getWidth(),
+													pFG->getHeight(),
 													GR_Image::GRT_Raster));
 						}
 						else
 						{
 							m_pImage = static_cast<GR_Image *>(
 								pG->createNewImage( m_sImagePath.c_str(),
-													static_cast<FG_GraphicVector *>(pFG)->getVector_SVG(),
+													pBB, pFG->getMimeType(),
 													m_pFormatTablePreview->getWindowWidth()-2,
 													m_pFormatTablePreview->getWindowHeight()-2,
 													GR_Image::GRT_Vector));
@@ -653,6 +661,28 @@ bool AP_Dialog_FormatTable::getLeftToggled()
 	return _getToggleButtonStatus("left-style");
 }
 
+
+guint AP_Dialog_FormatTable::_findClosestThickness(const char *sthickness) const
+{
+	double thickness = UT_convertToInches(sthickness);
+	guint i =0;
+	guint closest = 0;
+	double dClose = 100000000.;
+	for(i=0; i<FORMAT_TABLE_NUMTHICKNESS; i++)
+	{
+		double diff = thickness - m_dThickness[i];
+		if(diff < 0)
+			diff = -diff;
+		if(diff < dClose)
+		{
+			closest = i;
+			dClose = diff;
+		}
+	}
+	return closest;
+}
+
+
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
@@ -696,12 +726,12 @@ void AP_FormatTable_preview::draw(void)
 		GR_Image * pImg = m_pFormatTable->getImage();
 		FG_Graphic * pFG = m_pFormatTable->getGraphic();
 		const char * szName = pFG->getDataId();
+        const UT_ByteBuf * pBB = pFG->getBuffer();
 		if(pFG->getType() == FGT_Raster)
 		{
-			const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 			pImg = static_cast<GR_Image *>(
 				m_gc->createNewImage( szName,
-									pBB,
+									pBB, pFG->getMimeType(),
 									pageRect.width - 2*border,
 									pageRect.height - 2*border,
 									GR_Image::GRT_Raster));
@@ -710,7 +740,7 @@ void AP_FormatTable_preview::draw(void)
 		{
 			pImg = static_cast<GR_Image *>(
 				m_gc->createNewImage( szName,
-									static_cast<FG_GraphicVector *>(pFG)->getVector_SVG(),
+                                      pBB, pFG->getMimeType(),
 									pageRect.width - 2*border,
 									pageRect.height - 2*border,
 									GR_Image::GRT_Vector));
