@@ -56,14 +56,11 @@ AP_Dialog_Styles::AP_Dialog_Styles(XAP_DialogFactory * pDlgFactory, XAP_Dialog_I
 	m_pParaPreview = NULL;
 	m_pCharPreview = NULL;
 	m_pAbiPreview = NULL;
-	
-	if(m_vecCharProps.getItemCount() > 0)
-		m_vecCharProps.clear();
 }
 
 AP_Dialog_Styles::~AP_Dialog_Styles(void)
 {
-	UT_uint32 i;
+	UT_sint32 i;
 	DELETEP(m_pParaPreview);
 	DELETEP(m_pCharPreview);
 	DELETEP(m_pAbiPreview);
@@ -335,17 +332,17 @@ void AP_Dialog_Styles::fillVecWithProps(const gchar * szStyle, bool bReplaceAttr
 
 
 /*!
- * This method returns a pointer to the const char * value associated with the
+ * This method returns a std::string of the const char * value associated with the
  * the property szProp. Stolen from ap_Dialog_Lists and ap_Dialog_Styles.
  * It assumes properties and values are stored the array like this:
  * vecProp(n)   :   vecProp(n+1)
  * "property"   :   "value"
  */
-const gchar * AP_Dialog_Styles::getPropsVal(const gchar * szProp) const
+const std::string AP_Dialog_Styles::getPropsVal(const gchar * szProp) const
 {
 	UT_sint32 i = m_vecAllProps.getItemCount();
 	if(i <= 0)
-		return NULL;
+		return "";
 	UT_sint32 j;
 	const gchar * pszV = NULL;
 	for(j= 0; j<i ;j=j+2)
@@ -357,7 +354,7 @@ const gchar * AP_Dialog_Styles::getPropsVal(const gchar * szProp) const
 	if( j < i )
 		return  (const gchar *) m_vecAllProps.getNthItem(j+1);
 	else
-		return NULL;
+		return "";
 }
 
 
@@ -486,19 +483,26 @@ void AP_Dialog_Styles::ModifyFont(void)
 	// for a/p which are constant across the selection (always
 	// present) we will set the field in the dialog.  for things
 	// which change across the selection, we ask the dialog not
-	// to set the field (by passing null).
+	// to set the field (by passing "").
 
-	pDialog->setFontFamily(getPropsVal("font-family"));
-	pDialog->setFontSize(getPropsVal("font-size"));
-	pDialog->setFontWeight(getPropsVal("font-weight"));
-	pDialog->setFontStyle(getPropsVal("font-style"));
-	pDialog->setColor(getPropsVal("color"));
-	pDialog->setBGColor(getPropsVal("bgcolor"));
+	const std::string sFontFamily = getPropsVal("font-family");
+	const std::string sFontSize = getPropsVal("font-size");
+	const std::string sFontWeight = getPropsVal("font-weight");
+	const std::string sFontStyle = getPropsVal("font-style");
+	const std::string sColor = getPropsVal("color");
+	const std::string sBGColor = getPropsVal("bgcolor");
+
+	pDialog->setFontFamily(sFontFamily);
+	pDialog->setFontSize(sFontSize);
+	pDialog->setFontWeight(sFontWeight);
+	pDialog->setFontStyle(sFontStyle);
+	pDialog->setColor(sColor);
+	pDialog->setBGColor(sBGColor);
 //
 // Set the background color for the preview
 //
 	static gchar  background[8];
-	UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
+	const UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
 	sprintf(background, "%02x%02x%02x",bgCol->m_red,
 			bgCol->m_grn,bgCol->m_blu);
 	pDialog->setBackGroundColor( (const gchar *) background);
@@ -508,21 +512,21 @@ void AP_Dialog_Styles::ModifyFont(void)
 	// worry about initializing a combo box with a choice
 	// (and because they are all stuck under one CSS attribute).
 
-	bool bUnderline = false;
-	bool bOverline = false;
-	bool bStrikeOut = false;
-	bool bTopline = false;
-	bool bBottomline = false;
-	const gchar * s = getPropsVal("text-decoration");
-	if (s)
+	bool bUnderline2 = false;
+	bool bOverline2 = false;
+	bool bStrikeOut2 = false;
+	bool bTopline2 = false;
+	bool bBottomline2 = false;
+	const std::string sDecoration = getPropsVal("text-decoration");
+	if (!sDecoration.empty())
 	{
-		bUnderline = (strstr(s, "underline") != NULL);
-		bOverline = (strstr(s, "overline") != NULL);
-		bStrikeOut = (strstr(s, "line-through") != NULL);
-		bTopline = (strstr(s, "topline") != NULL);
-		bBottomline = (strstr(s, "bottomline") != NULL);
+		bUnderline2 = (strstr(sDecoration.c_str(), "underline") != NULL);
+		bOverline2 = (strstr(sDecoration.c_str(), "overline") != NULL);
+		bStrikeOut2 = (strstr(sDecoration.c_str(), "line-through") != NULL);
+		bTopline2 = (strstr(sDecoration.c_str(), "topline") != NULL);
+		bBottomline2 = (strstr(sDecoration.c_str(), "bottomline") != NULL);
 	}
-	pDialog->setFontDecoration(bUnderline,bOverline,bStrikeOut,bTopline,bBottomline);
+	pDialog->setFontDecoration(bUnderline2,bOverline2,bStrikeOut2,bTopline2,bBottomline2);
 /*
     bool bDirection;
 	s = UT_getAttribute("dir", props_in);
@@ -543,36 +547,36 @@ void AP_Dialog_Styles::ModifyFont(void)
 
 	if (bOK)
 	{
-		const gchar * s;
+		const gchar * s1;
 
-		if (pDialog->getChangedFontFamily(&s))
+		if (pDialog->getChangedFontFamily(&s1))
 		{
-			addOrReplaceVecProp("font-family", s);
+			addOrReplaceVecProp("font-family", s1);
 		}
 
-		if (pDialog->getChangedFontSize(&s))
+		if (pDialog->getChangedFontSize(&s1))
 		{
-			addOrReplaceVecProp("font-size", s);
+			addOrReplaceVecProp("font-size", s1);
 		}
 
-		if (pDialog->getChangedFontWeight(&s))
+		if (pDialog->getChangedFontWeight(&s1))
 		{
-			addOrReplaceVecProp("font-weight", s);
+			addOrReplaceVecProp("font-weight", s1);
 		}
 
-		if (pDialog->getChangedFontStyle(&s))
+		if (pDialog->getChangedFontStyle(&s1))
 		{
-			addOrReplaceVecProp("font-style", s);
+			addOrReplaceVecProp("font-style", s1);
 		}
 
-		if (pDialog->getChangedColor(&s))
+		if (pDialog->getChangedColor(&s1))
 		{
-			addOrReplaceVecProp("color", s);
+			addOrReplaceVecProp("color", s1);
 		}
 
-		if (pDialog->getChangedBGColor(&s))
+		if (pDialog->getChangedBGColor(&s1))
 		{
-			addOrReplaceVecProp("bgcolor", s);
+			addOrReplaceVecProp("bgcolor", s1);
 		}
 
 		bool bUnderline = false;
@@ -642,7 +646,7 @@ void AP_Dialog_Styles::_tabCallback(const char *szTabStops,
  * Used to extract data out of the Tabs dialog.
  */
 static void
-s_TabSaveCallBack (AP_Dialog_Tab * pDlg, FV_View * pView,
+s_TabSaveCallBack (AP_Dialog_Tab * /*pDlg*/, FV_View * /*pView*/,
 				   const char * szTabStops, const char * szDflTabStop,
 				   void * closure)
 {
@@ -707,45 +711,55 @@ void AP_Dialog_Styles::ModifyLists(void)
 //
 // Fill input list for Lists dialog
 //
-	if(getPropsVal("list-style"))
+	const std::string sListStyle = getPropsVal("list-style");
+	const std::string sFieldFont = getPropsVal("field-font");
+	const std::string sStartValue = getPropsVal("start-value");
+	const std::string sListDelim = getPropsVal("list-delim");
+	const std::string sMarginLeft = getPropsVal("margin-left");
+	const std::string sListDecimal = getPropsVal("list-decimal");
+	const std::string sTextIndent = getPropsVal("text-indent");
+
+	if(!sListStyle.empty())
 	{
 		vp.addItem("list-style");
-		vp.addItem(getPropsVal("list-style"));
+		vp.addItem(sListStyle.c_str());
 	}
-	if(getPropsVal("field-font"))
+	if(!sFieldFont.empty())
 	{
 		vp.addItem("field-font");
-		vp.addItem(getPropsVal("field-font"));
+		vp.addItem(sFieldFont.c_str());
 	}
-	if(getPropsVal("start-value"))
+	if(!sStartValue.empty())
 	{
 		vp.addItem("start-value");
-		vp.addItem(getPropsVal("start-value"));
+		vp.addItem(sStartValue.c_str());
 	}
-	if(getPropsVal("list-delim"))
+	if(!sListDelim.empty())
 	{
 		vp.addItem("list-delim");
-		vp.addItem(getPropsVal("list-delim"));
+		vp.addItem(sListDelim.c_str());
 	}
-	if(getPropsVal("margin-left"))
+	if(!sMarginLeft.empty())
 	{
 		vp.addItem("margin-left");
-		vp.addItem(getPropsVal("margin-left"));
+		vp.addItem(sMarginLeft.c_str());
 	}
-	if(getPropsVal("field-font"))
+	// TODO: Why is field-font here twice?
+	if(!sFieldFont.empty())
 	{
 		vp.addItem("field-font");
-		vp.addItem(getPropsVal("field-font"));
+		vp.addItem(sFieldFont.c_str());
 	}
-	if(getPropsVal("list-decimal"))
+
+	if(!sListDecimal.empty())
 	{
 		vp.addItem("list-decimal");
-		vp.addItem(getPropsVal("list-decimal"));
+		vp.addItem(sListDecimal.c_str());
 	}
-	if(getPropsVal("text-indent"))
+	if(!sTextIndent.empty())
 	{
 		vp.addItem("text-indent");
-		vp.addItem(getPropsVal("text-indent"));
+		vp.addItem(sTextIndent.c_str());
 	}
 	pDialog->fillDialogFromVector(&vp);
 //
@@ -798,6 +812,7 @@ void AP_Dialog_Styles::ModifyLists(void)
 			m_ListProps[6] = getVecVal(vo,"text-indent");
 			addOrReplaceVecProp("text-indent",m_ListProps[6].c_str());
 		}
+		// TODO: Why is field-font here twice?
 		if(getVecVal(vo,"field-font"))
 		{
 			m_ListProps[7] = getVecVal(vo,"field-font");
@@ -916,10 +931,10 @@ void AP_Dialog_Styles::updateCurrentStyle(void)
 	if(m_pAbiPreview == NULL)
 		return;
 	const gchar ** props = NULL;
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	if(m_vecAllProps.getItemCount() <= 0)
 		return;
-	UT_uint32 countp = m_vecAllProps.getItemCount() + 1;
+	UT_sint32 countp = m_vecAllProps.getItemCount() + 1;
 	props = (const gchar **) UT_calloc(countp, sizeof(gchar *));
 	countp--;
 	for(i=0; i<countp; i++)
@@ -1216,7 +1231,7 @@ void AP_Dialog_Styles::_createCharPreviewFromGC(GR_Graphics * gc,
 // Set the Background color for the preview.
 //
 	static gchar  background[8];
-	UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
+	const UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
 	sprintf(background, "%02x%02x%02x",bgCol->m_red,bgCol->m_grn,bgCol->m_blu);
 
 	m_pCharPreview = new XAP_Preview_FontPreview(gc,background);
@@ -1232,7 +1247,7 @@ void AP_Dialog_Styles::_createCharPreviewFromGC(GR_Graphics * gc,
 //
 // set our Vector of Character Properties into the preview class.
 //
-	m_pCharPreview->setVecProperties( &m_vecCharProps);
+	m_pCharPreview->setVecProperties( &m_mapCharProps);
 }
 
 /*!
@@ -1335,7 +1350,7 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 	static gchar szFGColor[8];
 	UT_RGBColor FGColor(0,0,0);
 	UT_RGBColor BGColor(255,255,255);
-	UT_RGBColor * pageCol = NULL;
+	const UT_RGBColor * pageCol = NULL;
 	getLView()->setStyle("Normal");
 
 	const gchar ** props_in = NULL;
@@ -1585,7 +1600,7 @@ void AP_Dialog_Styles::event_charPreviewUpdated (void) const
 	UT_return_if_fail (m_pCharPreview); // add this when we make a char preview
 
 	// force a redraw
-	m_pCharPreview->setVecProperties( &m_vecCharProps);
+	m_pCharPreview->setVecProperties( &m_mapCharProps);
 	m_pCharPreview->draw();
 }
 
@@ -1661,8 +1676,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 		}
 
 // Clear out old contents of the char vector if they exist
-		if(m_vecCharProps.getItemCount() > 0)
-			m_vecCharProps.clear();
+		m_mapCharProps.clear();
 
 	    // now loop through and pass out each property:value combination for characters
 
@@ -1696,8 +1710,7 @@ void AP_Dialog_Styles::_populatePreviews(bool isModify)
 //
 // Put them in our property vector for the Character preview
 //
-			m_vecCharProps.addItem((void *) szName);
-			m_vecCharProps.addItem((void *) szValue);
+			m_mapCharProps[szName] = szValue;
 		}
 
 		if (!m_curStyleDesc.empty())
