@@ -93,7 +93,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_About);
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Contents);
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Checkver);
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Search);
-Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Index);
+Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Intro);
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest);
 
 #endif /* AP_MENU_FUNCTIONS_H */
