@@ -190,8 +190,12 @@ bool IE_Exp_MsWord_Hack_Sniffer::getDlgLabels(const char ** pszDesc,
 
 /*****************************************************************/
 /*****************************************************************/
-
 UT_Error IE_Exp_RTF::_writeDocument(void)
+{
+	return _writeDocumentLocal(false);
+}
+
+UT_Error IE_Exp_RTF::_writeDocumentLocal(bool bSkipHeader)
 {
 	// The overall syntax for an RTF file is:
 	//
@@ -211,7 +215,7 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
 	s_RTF_ListenerGetProps * listenerGetProps = new s_RTF_ListenerGetProps(getDoc(),this);
 	if (!listenerGetProps)
 		return UT_IE_NOMEMORY;
-	if (getDocRange())
+	if (getDocRange() && !bSkipHeader)
 		getDoc()->tellListenerSubset(listenerGetProps,getDocRange());
 	else
 		getDoc()->tellListener(listenerGetProps);
@@ -228,10 +232,11 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
 	_selectStyles();
 
 	// write rtf header
-
-	if (!_write_rtf_header())
-		return UT_IE_COULDNOTWRITE;
-
+	if(!bSkipHeader)
+	{
+		if (!_write_rtf_header())
+			return UT_IE_COULDNOTWRITE;
+	}
 	// create and install a listener to receive the document
 	// and write its content in rtf.
 
@@ -246,12 +251,16 @@ UT_Error IE_Exp_RTF::_writeDocument(void)
 
 	// write any rtf trailer matter
 
-	if (!_write_rtf_trailer())
-		return UT_IE_COULDNOTWRITE;
-
+	if(!bSkipHeader)
+	{
+			if (!_write_rtf_trailer())
+				return UT_IE_COULDNOTWRITE;
+	}
 	return ((m_error) ? UT_IE_COULDNOTWRITE : UT_OK);
 }
 
+
+
 /*!
  * This method search for the requested header/footer section within
  * PD_DOCument and writes into the stream at the current write point.
@@ -461,6 +470,15 @@ void IE_Exp_RTF::_rtf_keyword(const char * szKey, UT_sint32 d)
 	m_bLastWasKeyword = true;
 }
 
+
+void IE_Exp_RTF::_rtf_keyword_space(const char * szKey, UT_sint32 d)
+{
+	write("\\");
+	write(szKey);
+	write(UT_String_sprintf(" %d",d));
+	m_bLastWasKeyword = true;
+}
+
 void IE_Exp_RTF::_rtf_keyword(const char * szKey, const char * val)
 {
   write("\\");
@@ -1022,7 +1040,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 		_rtf_semi();
 		_rtf_close_brace();
 		
-		for(UT_uint32 i = 0; i < Revs.getItemCount(); ++i)
+		for(UT_sint32 i = 0; i < Revs.getItemCount(); ++i)
 		{
 			AD_Revision* pRev = Revs.getNthItem(i);
 			UT_continue_if_fail(pRev);
@@ -1093,7 +1111,7 @@ void IE_Exp_RTF::_write_prop_ifyes(const PD_Style * pStyle,
 /*
  * Used to hold tab information by _write_tabdef.
  */
-class _t
+class ABI_EXPORT _t
 {
 public:
 	_t(const char * szTL, const char * szTT, const char * szTK, UT_sint32 tp)
@@ -1591,10 +1609,10 @@ void IE_Exp_RTF::_write_parafmt(const PP_AttrProp * pSpanAP, const PP_AttrProp *
 			}
 			else
 			{
-				UT_uint32 i,len;
+				UT_uint32 j,len;
 				len = UT_UCS4_strlen(tmp);
-				for(i=0;i<=len;i++)
-					p[i] = (char) (unsigned char) *tmp++;
+				for(j=0;j<=len;j++)
+					p[j] = (char) (unsigned char) *tmp++;
 				_rtf_chardata(p,len);
 			}
 			_rtf_close_brace();
@@ -1619,7 +1637,7 @@ void IE_Exp_RTF::_write_parafmt(const PP_AttrProp * pSpanAP, const PP_AttrProp *
 //		if(id != m_currID)
 		{
 			UT_return_if_fail(iOver);
-			fl_AutoNum * pAuto = getDoc()->getListByID(id);
+//			fl_AutoNum * pAuto = getDoc()->getListByID(id);
 			UT_return_if_fail(pAuto);
 			while(pAuto->getParent() != NULL)
 			{
@@ -1717,7 +1735,7 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 	UT_sint32 ndxColor = -1;
 	if(szColor)
 	{
-		UT_sint32 ndxColor = _findColor((char*)szColor);
+		ndxColor = _findColor((char*)szColor);
 		if( ndxColor == -1)
 		{
 			return;
@@ -2127,8 +2145,8 @@ struct NumberedStyle
     const PD_Style * pStyle;
     UT_uint32 n;
 
-    NumberedStyle(const PD_Style * pStyle, UT_uint32 n) :
-	pStyle(pStyle), n(n) {}
+    NumberedStyle(const PD_Style * _pStyle, UT_uint32 _n) :
+		pStyle(_pStyle), n(_n) {}
 };
 
 /*!
@@ -2302,7 +2320,7 @@ void IE_Exp_RTF::_write_stylesheets(void)
  */
 void IE_Exp_RTF::_write_listtable(void)
 {
-	UT_uint32 iCount = getDoc()->getListsCount();
+	UT_sint32 iCount = getDoc()->getListsCount();
     if (iCount == 0) return;
 //
 // Openning RTF comments for the listtable
@@ -2315,7 +2333,7 @@ void IE_Exp_RTF::_write_listtable(void)
 // OK scan the lists in the document to build up the list info.
 // The first loop just builds a vector of parentless lists.
 //
-	UT_uint32 i,j,k =0;
+	UT_sint32 i,j =0;
 	bool bFoundChild = false;
 	fl_AutoNum * pAuto = NULL;
 	fl_AutoNum * pInner = NULL;
@@ -2351,6 +2369,7 @@ void IE_Exp_RTF::_write_listtable(void)
 // OK now fill the MultiLevel list structure.
 //
 
+	UT_sint32 k;
 	for(k=0; k < m_vecMultiLevel.getItemCount(); k++)
 	{
 		pList97 = (ie_exp_RTF_MsWord97ListMulti *) m_vecMultiLevel.getNthItem(k);
@@ -2418,18 +2437,18 @@ void IE_Exp_RTF::_write_listtable(void)
 //
 // MultiLevel lists
 //
-	for(i=0; i< m_vecMultiLevel.getItemCount(); i++)
+	for(k=0; k< m_vecMultiLevel.getItemCount(); k++)
 	{
 		_rtf_nl();
-		_output_MultiLevelRTF(getNthMultiLevel(i));
+		_output_MultiLevelRTF(getNthMultiLevel(k));
 	}
 //
 // Simple Lists
 //
-	for(i=0; i< m_vecSimpleList.getItemCount(); i++)
+	for(k=0; k< m_vecSimpleList.getItemCount(); k++)
 	{
 		_rtf_nl();
-		_output_SimpleListRTF(getNthSimple(i));
+		_output_SimpleListRTF(getNthSimple(k));
 	}
 //
 // \*\listtable is done now!
@@ -2967,7 +2986,7 @@ void IE_Exp_RTF::_output_SimpleListRTF(ie_exp_RTF_MsWord97ListSimple * pSimple)
  * Actually output the RTF from an Overide
  \param ie_exp_RTF_Overide * pOver pointer to an Overide definition
  */
-void IE_Exp_RTF::_output_OveridesRTF(ie_exp_RTF_ListOveride * pOver, UT_uint32 iOver)
+void IE_Exp_RTF::_output_OveridesRTF(ie_exp_RTF_ListOveride * pOver, UT_uint32 /*iOver*/)
 {
 	_rtf_open_brace();
 	_rtf_keyword("listoverride");
@@ -3334,7 +3353,8 @@ ie_exp_RTF_MsWord97List * ie_exp_RTF_MsWord97ListMulti::getListAtLevel(UT_uint32
  */
 UT_uint32 ie_exp_RTF_MsWord97ListMulti::getMatchingID(UT_uint32 listID)
 {
-	UT_uint32 i,j;
+	UT_uint32 i;
+	UT_sint32 j;
 	ie_exp_RTF_MsWord97List * pList97 = NULL;
 	bool bFound = false;
 	UT_uint32 foundID = 0;
