@@ -20,6 +20,10 @@
  * 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <string.h>
 #include "ut_debugmsg.h"
 #include "ut_string.h"
@@ -37,13 +41,14 @@
 #include "px_CR_Strux.h"
 #include "ut_iconv.h"
 #include "ut_wctomb.h"
+#include "xap_App.h"
 #include "xap_EncodingManager.h"
 #include "ap_Dialog_Id.h"
 #include "xap_DialogFactory.h"
 #include "xap_Dlg_Encoding.h"
 #include "ap_Prefs.h"
 #include "ut_string_class.h"
-#ifdef WIN32
+#ifdef TOOLKIT_WIN
   #include "ut_Win32OS.h"
 #endif
 
@@ -66,7 +71,7 @@ IE_Exp_Text::IE_Exp_Text(PD_Document * pDocument, bool bEncoded)
 
 	// Get encoding dialog prefs setting
 	bool bAlwaysPrompt = false;
-	getDoc()->getApp()->getPrefsValueBool(AP_PREF_KEY_AlwaysPromptEncoding, &bAlwaysPrompt);
+	XAP_App::getApp()->getPrefsValueBool(AP_PREF_KEY_AlwaysPromptEncoding, &bAlwaysPrompt);
 
 	m_bIsEncoded = bAlwaysPrompt | bEncoded;
 
@@ -201,11 +206,9 @@ bool IE_Exp_EncodedText_Sniffer::getDlgLabels(const char ** pszDesc,
 PL_Listener * IE_Exp_Text::_constructListener(void)
 {
 	if (!m_bExplicitlySetEncoding) {
-		const UT_UTF8String * prop;
-
-		prop = getProperty ("encoding");
-		if (prop) {
-			_setEncoding (prop->utf8_str());
+		const std::string & prop = getProperty ("encoding");
+		if (!prop.empty()) {
+			_setEncoding (prop.c_str());
 		}
 	}
 
@@ -226,7 +229,7 @@ UT_Error IE_Exp_Text::_writeDocument(void)
 {
 	// Don't call base method if user cancels encoding dialog
 	if (!(!m_bIsEncoded || m_bExplicitlySetEncoding || _doEncodingDialog(m_szEncoding)))
-		return UT_IE_COULDNOTWRITE;
+		return UT_SAVE_CANCELLED;
 
 	// TODO If we're going to the clipboard and the OS supports unicode, set encoding.
 	// TODO Only supports Windows so far.
@@ -263,7 +266,7 @@ bool IE_Exp_Text::_doEncodingDialog(const char *szEncoding)
 	XAP_Dialog_Id id = XAP_DIALOG_ID_ENCODING;
 
 	XAP_DialogFactory * pDialogFactory
-		= static_cast<XAP_DialogFactory *>(getDoc()->getApp()->getDialogFactory());
+		= static_cast<XAP_DialogFactory *>(XAP_App::getApp()->getDialogFactory());
 
 	XAP_Dialog_Encoding * pDialog
 		= static_cast<XAP_Dialog_Encoding *>(pDialogFactory->requestDialog(id));
@@ -272,7 +275,7 @@ bool IE_Exp_Text::_doEncodingDialog(const char *szEncoding)
 	pDialog->setEncoding(szEncoding);
 
 	// run the dialog
-	XAP_Frame * pFrame = getDoc()->getApp()->getLastFocussedFrame();
+	XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
 	UT_return_val_if_fail(pFrame, false);
 
 	pDialog->runModal(pFrame);
@@ -558,7 +561,7 @@ void Text_Listener::_handleDirMarker(PT_AttrPropIndex api)
 	}
 	else
 	{
-		UT_DEBUGMSG(("Text_Listener::_handleDirMarker: no props! (bHaveProp %d, pAP 0x%x)\n",
+		UT_DEBUGMSG(("Text_Listener::_handleDirMarker: no props! (bHaveProp %d, pAP %p)\n",
 					 bHaveProp, pAP));
 		UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
 	}
@@ -747,6 +750,9 @@ bool Text_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			case PTO_Math:
 				return true;
 
+			case PTO_Annotation:
+				return true;
+
 			default:
 				UT_ASSERT_HARMLESS(UT_TODO);
 				return true;
@@ -884,6 +890,8 @@ bool Text_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_EndEndnote:
 	case PTX_SectionTOC:
 	case PTX_EndTOC:
+	case PTX_SectionAnnotation:
+	case PTX_EndAnnotation:
 	    return true ;
 
 	default:
