@@ -3,7 +3,7 @@
  * Copyright (C) 1999 AbiSource, Inc.
  * Copyright (C) 2003 Tomas Frydrych <tomas@frydrych.uklinux.net>
  * Copyright (C) 2003 Martin Sevior <msevior@physics.unimelb.edu.au> 
- * Copyright (C) 2001, 2004 Hubert Figuiere <hfiguiere@teaser.fr>
+ * Copyright (C) 2001, 2004, 2009 Hubert Figuiere <hub@figuiere.net>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,7 +40,8 @@
 #include "fg_Graphic.h"
 #include "fg_GraphicRaster.h"
 #include "fg_GraphicVector.h"
-
+#include "pf_Frag.h"
+#include "pf_Frag_Strux.h"
 
 // move this one away.
 #define MAX_KEYWORD_LEN 256
@@ -52,6 +53,7 @@ iegftForRTF(IE_Imp_RTF::PictFormat format)
 	{
 	case IE_Imp_RTF::picPNG:  return IEGFT_PNG;
 	case IE_Imp_RTF::picJPEG: return IEGFT_JPEG;
+	case IE_Imp_RTF::picSVG:  return IEGFT_SVG;
 	case IE_Imp_RTF::picBMP:  return IEGFT_BMP;
 	case IE_Imp_RTF::picWMF:  return IEGFT_WMF;
 	default:
@@ -142,7 +144,6 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 
 	if ((error == UT_OK) && pFG)
 	{
-		const UT_ByteBuf * buf = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
 		imgProps.width = static_cast<UT_uint32>(pFG->getWidth ());
 		imgProps.height = static_cast<UT_uint32>(pFG->getHeight ());
 		// Not sure whether this is the right way, but first, we should
@@ -154,7 +155,7 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
 			return false;
 		}
 
-		ok = InsertImage (buf, image_name, imgProps);
+		ok = InsertImage (pFG, image_name, imgProps);
 		DELETEP(pFG);
 		if (!ok)
 		{
@@ -180,7 +181,7 @@ bool IE_Imp_RTF::LoadPictData(PictFormat format, const char * image_name,
   Insert and image at the current position.
   Check whether we are pasting or importing
 */
-bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
+bool IE_Imp_RTF::InsertImage (const FG_Graphic *pFG, const char * image_name,
 							  const struct RTFProps_ImageProps & imgProps)
 {
 	UT_String propBuffer;
@@ -196,9 +197,6 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		// non-null file, we're importing a doc
 		// Now, we should insert the picture into the document
 
-		const char * mimetype = NULL;
-		mimetype = g_strdup("image/png");
-
 		switch (imgProps.sizeType)
 		{
 		case RTFProps_ImageProps::ipstGoal:
@@ -269,12 +267,12 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 
 			if (!getDoc()->appendObject(PTO_Image, propsArray))
 			{
-				FREEP(mimetype);
 				return false;
 			}
 		}
 		if (!getDoc()->createDataItem(image_name, false,
-									  buf, static_cast<const void*>(mimetype), NULL))
+									  pFG->getBuffer(), pFG->getMimeType(), 
+									  NULL))
 		{
 			// taken care of by createDataItem
 			//FREEP(mimetype);
@@ -322,10 +320,10 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		/*
 		  Create the data item
 		*/
-		const char * mimetype = NULL;
-		mimetype = g_strdup("image/png");
 		bool bOK = false;
-		bOK = getDoc()->createDataItem(szName.c_str(), false, buf, static_cast<const void *>(mimetype), NULL);
+		bOK = getDoc()->createDataItem(szName.c_str(), false, 
+									   pFG->getBuffer(), 
+									   pFG->getMimeType(), NULL);
 		UT_return_val_if_fail(bOK, false);
 		/*
 		  Insert the object into the document.
@@ -443,6 +441,9 @@ bool IE_Imp_RTF::HandlePicture()
 			case RTF_KW_wmetafile:
 				format = picWMF;
 				break;
+			case RTF_KW_svgblip:
+				format = picSVG;
+				break;
 			case RTF_KW_picwgoal:
 				if (parameterUsed)
 				{
@@ -1048,7 +1049,26 @@ void IE_Imp_RTF::HandleShape(void)
 
 	// Formely in HandleEndFrame()
 	UT_DEBUGMSG((">>>>End frame\n"));
-	if(!bUseInsertNotAppend()) {
+	if(!bUseInsertNotAppend()) 
+	{
+		if(m_bFrameTextBox)
+		{
+			//
+			// Look if we have a bare FrameStrux. If so delete it.
+			//
+			pf_Frag * pf = getDoc()->getLastFrag();
+			if(pf && (pf->getType() == pf_Frag::PFT_Strux))
+			{
+				    pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
+					if(pfs->getStruxType() == PTX_SectionFrame)
+					{
+						getDoc()->deleteFragNoUpdate(pf);
+						m_bFrameTextBox = false;
+						return;
+					}
+			}
+				
+		}
 		getDoc()->appendStrux(PTX_EndFrame, NULL);
 		m_newParaFlagged = false;
 	}
@@ -1080,6 +1100,7 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 	UT_UTF8String sP;
 	UT_UTF8String sV;
 	sP = "frame-type";
+	m_bFrameTextBox = false;
 	if(	frame.m_iFrameType == 1)
 	{
 		sV = "image";
@@ -1098,6 +1119,7 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 	{
 		sV = "textbox";
 		UT_UTF8String_setProperty(sPropString,sP,sV); // fixme make other types
+		m_bFrameTextBox = true;
 	}
 
 	sP = "position-to";
@@ -1274,7 +1296,7 @@ void IE_Imp_RTF::HandleShapePict()
 		default:
 			break;
 		}
-	} while (!((tokenType == RTF_TOKEN_CLOSE_BRACE) && (nested == 0)));
+	} while (!((tokenType == RTF_TOKEN_CLOSE_BRACE) && (nested <= 1)));
 }
 
 
@@ -1372,7 +1394,7 @@ bool IE_Imp_RTF::HandleObject()
 		default:
 			break;
 		}
-	} while (!((tokenType == RTF_TOKEN_CLOSE_BRACE) && (nested == 0)));
+	} while (!((tokenType == RTF_TOKEN_CLOSE_BRACE) && (nested <= 1)));
 
 	return true;
 }
