@@ -31,6 +31,7 @@
 #include "ie_imp_Text.h"
 #include "pd_Document.h"
 #include "ut_growbuf.h"
+#include "xap_App.h"
 #include "xap_EncodingManager.h"
 
 
@@ -671,7 +672,7 @@ IE_Imp_Text::IE_Imp_Text(PD_Document * pDocument, bool bEncoded)
 {
 	// Get encoding dialog prefs setting
 	bool bAlwaysPrompt;
-	getDoc()->getApp()->getPrefsValueBool(AP_PREF_KEY_AlwaysPromptEncoding, &bAlwaysPrompt);
+	XAP_App::getApp()->getPrefsValueBool(AP_PREF_KEY_AlwaysPromptEncoding, &bAlwaysPrompt);
 
 	m_bIsEncoded = bAlwaysPrompt | bEncoded;
 
@@ -812,11 +813,11 @@ UT_Error IE_Imp_Text::_parseStream(ImportStream * pStream)
 	UT_UCSChar c;
 
 	if (!m_bExplicitlySetEncoding) {
-		const UT_UTF8String * prop;
+		std::string prop;
 
 		prop = getProperty ("encoding");
-		if (prop) {
-			_setEncoding (prop->utf8_str());
+		if (!prop.empty()) {
+			_setEncoding (prop.c_str());
 		}
 	}
 
@@ -907,7 +908,7 @@ bool IE_Imp_Text::_doEncodingDialog(const char *szEncoding)
 	XAP_Dialog_Id id = XAP_DIALOG_ID_ENCODING;
 
 	XAP_DialogFactory * pDialogFactory
-		= static_cast<XAP_DialogFactory *>(getDoc()->getApp()->getDialogFactory());
+		= static_cast<XAP_DialogFactory *>(XAP_App::getApp()->getDialogFactory());
 
 	XAP_Dialog_Encoding * pDialog
 		= static_cast<XAP_Dialog_Encoding *>(pDialogFactory->requestDialog(id));
@@ -916,7 +917,7 @@ bool IE_Imp_Text::_doEncodingDialog(const char *szEncoding)
 	pDialog->setEncoding(szEncoding);
 
 	// run the dialog
-	XAP_Frame * pFrame = getDoc()->getApp()->getLastFocussedFrame();
+	XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
 	UT_return_val_if_fail(pFrame, false);
 
 	pDialog->runModal(pFrame);
